#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <wchar.h>
#include "input.h"
#include "unicode.h"
#include "utf8error.h"

/*
 * Read a line into buffer from a file of UTF-8 characters, converting
 * en passant to UTF32.
 * Returns the number of characters in the line, not counting the
 * terminating null. Returns:
 *	 
 *     ENDOFINPUT     if there is no more input
 *     BUFOVERFLOW    if the length of the line exceeds the buffer size 
 *     	 
 */

#define DEFAULT 0
#define OVERFLOW 1
#define MSGSIZE 128

short UGetLineDone = 0;

int
ugetline(FILE *fp,wchar_t *buf, int size)
{

  wchar_t c;
  int cnt = 0;
  int state = DEFAULT;
  char msg[MSGSIZE];

  extern UTF32 GetUTF32FromUTF8();

  if(UGetLineDone){
    UGetLineDone = 0;
    return(ENDOFINPUT);
  }
 
  while( (c = GetUTF32FromUTF8(fileno(fp))) <= UNI_MAX_UTF32){
    if(cnt == size){
      buf[cnt] = '\0';
      state = OVERFLOW;
    }
    if(c == '\n'){
      if(state == OVERFLOW) {
	return(BUFOVERFLOW);
      }
      else{
	buf[cnt] = '\0';
	return(cnt);
      }
    }
    else if(state == DEFAULT) buf[cnt++] = c;
  }
  UGetLineDone = 1;
  switch (c){ 
     case NOTENOUGHBYTES:
       fprintf(stderr,"Truncated UTF-8 sequence encountered.\n");
       exit(1);
       break;
     case BADINCODE:
       fprintf(stderr,"Invalid UTF-8 code encountered.\n");
       exit(1);
       break;
     case BADOUTCODE:
       fprintf(stderr,"Encountered invalid Unicode.\n");
       exit(1);
       break;
     case IOERROR:
       snprintf(msg,MSGSIZE-1,"Error reading input.\n");
       perror(msg);
       exit(1);
       break;
     default:			/* Normal EOF */
       break;
  }
    if(state == OVERFLOW) {
      return(BUFOVERFLOW);
    }
  else{
    buf[cnt] = '\0';
    return(cnt);
  }
}

