#!/bin/sh
#-*- mode: Tcl;time-stamp-start:"TimeStamp[ 	]+\\\\?[\"<]+";-*-
# the next line restarts using wish \
exec wish $0 -- $@
set TimeStamp "2009-11-14 17:56:19 poser"
#
# Copyright (C) 2003-2009 William J. Poser (billposer@alum.mit.edu)
# This program is free software; you can redistribute it and/or modify
# it under the terms of version 3 of the GNU General Public License as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.#
# A copy of the GNU General Public License is contained in the
# procedure "License" in this file.
# If it is not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# or go to the web page:  http://www.gnu.org/licenses/gpl.txt.

#If you are running Mac OS X and find that things are not working properly
#it may be because the program has not correctly detected what system
#it is running on. In this case, things may improve if you make this
#explicit. In particular, if pressing the mouse button
#and the Control key simultaneously does not produce the behavior
#described in the documentation as associated with the right mouse button
#(on other systems), try uncommenting the following line.
#set MacOSX 1;

set Version "2.1";
set PackageVersion "5.2";
set progname "mpg"
set DebugP 0;				# General debugging
#package require Iwidgets

set MSG    .mf.msg;
set PBAR   .mf.pbar 

proc ProgramTimeDateStamp {} {
    set sts [split $::TimeStamp]
    return "[lindex $sts 0] [lindex $sts 1]"
}

proc dmsg {msg} {
    if {$::DebugP} {
	puts $msg;
	flush stdout;
    }
}
package require msgcat
proc _ {s} {return [::msgcat::mc $s]};	# Define shorthand for gettext

# Portability 

if {[info exists ::env(TMP)]} {
    set TempDir $::env(TMP);
} elseif {[info exists ::env(TEMP)]} {
    set TempDir $::env(TEMP);
} else {
    if { $tcl_platform(platform) == "windows"} {
	set TempDir "C:\tmp";
    } else {
	set TempDir "/tmp";
    }
}
set InitFile  ".mpgrc";

#Figure out what system we are running on
if {[string equal $tcl_platform(platform) windows]} {
    set System MSWindows;
    dmsg "Running under MS Windows";
} elseif {[string equal $tcl_platform(platform) unix]} {
    if {[string equal $tcl_platform(os) Darwin]} {
	set System MacOSX;
	dmsg "Running under Mac OS X";
    } else {
	set System Unix;
	dmsg "Running under Unix";
    }
}

#Find out what our base graphics system is
if {[string match X11*  [winfo server .]]} {
    set AquaP 0
    set WindowSystem X11
} else {
    if {[string match $System MSWindows]} {
	set AquaP 0;
	set WindowSystem MSWindows;
    }
    if {[string match $System MacOSX]} {
	set AquaP 1
	set WindowSystem Aqua
    }
}

switch $System {
   Unix {
	event add <<B3>> <ButtonPress-3>
	event add <<B3Release>> <ButtonRelease-3>
    }
    MacOSX {
	event add <<B3>> <Control-ButtonPress-1>
	event add <<B3Release>> <Control-ButtonRelease-1>
	if {$AquaP} {
	}
    }
    MSWindows {
	event add <<B3>> <ButtonPress-3>
	event add <<B3Release>> <ButtonRelease-3>
	set InitFile    "mpgrc"
    }
}

#We need to use different menu entry numbers in Aqua
#because unlike the other Tk implementations, Aq ua has no tearoffs.
#In non-Aqua Tk the tearoff counts as entry 0 when it is present.

proc mdo {elnum} {
    if {$::AquaP} {
	return [expr $elnum -1]
    } else {
	return $elnum
    }
}


set BalloonHelpP 1;
set HPWidth 64;				# Width of help popups.
set HPLines 10;				# Number of lines for help popups.
set ProgressFile minpair.plog;
set IPACIsDisplayedP 0;
set IPADIsDisplayedP 0;
set IPAVIsDisplayedP 0;
set IPAAIsDisplayedP 0;
set CharEntryByCodeIsDisplayedP 0;
set UseMinpairP 1;

set ColorSpecs(Default,Background)		"\#C8C8FF";
set ColorSpecs(DefaultActive,Background)	"\#d4e4fe";
set ColorSpecs(MainFrame,Background)		"\#FF6666";
set ColorSpecs(Menu,ActiveBackground)		salmon
set ColorSpecs(Menubar,ActiveBackground)	"\#50E4A5";
set ColorSpecs(Menubar,Background)	 	"\#c36176";
set ColorSpecs(Menubar,Foreground)		"\#fee4a9";
set ColorSpecs(Messages,Background) 		"\#e9c4dc";
set ColorSpecs(Messages,Foreground) 		"\#000000";
set ColorSpecs(CharacterEntry,Background)	"\#FFFFFF";
set ColorSpecs(CharacterEntry,Foreground)	"\#000000";
set ColorSpecs(IPAHeadings,Background)		"\#09ffb2";
set ColorSpecs(UserTextEntry,Background)	"\#FFFFFF";
set ColorSpecs(UserTextEntry,Foreground)	black
set ColorSpecs(LabelText,Foreground)		"\#d4e4fe";

set DefaultBrowser dillo 
#For this purpose dillo is in many respects arguably the best choice because
#it is very lightweight and starts up fast. However, it isn't as fully
#featured as the others, and in particular, doesn't handle the full range of
#Unicode characters.
set BrowserList [list firefox mozilla epiphany galeon konqueror dillo netscape opera]
set BrowserPIDS [list];

#Defaults
set MinpairAvailableP 0
set MinpairVersion "unknown"
set HowManyDiffs 1
set DefaultMaxWordLength 25;
set MaxWordLength $DefaultMaxWordLength;
set MaximumDistance $DefaultMaxWordLength;
set DefaultDifferenceSiteMarker "_";
set DoIndelsP 0;
set DoTranspositionsP 0;
set DoSingleP 1;
set MultigraphDefinitions "";
set MinpairInProgressP 0;
set ReadInitFileP 1;
set SortOutputP 1;
set DefaultInputFile  [_ "TestInput"];
set DefaultMultigraphFile  [_ "Multigraphs"];
set DefaultOutputFile [_ "TestOutput"];


proc LoadMessageCatalog {} {
    global DebugP;

    if { $DebugP } {
	if { [::msgcat::mcload "msgs"] == 0} {
	    puts "No message catalog loaded."
	} else {
	    puts "Message catalog loaded."
	}
    } else  {
	::msgcat::mcload [file join [file dirname [info script]] msgs];
    }
}

proc ShutDown {} {
    global BrowserPIDS;
    #Shutdown child browsers
    foreach pid $BrowserPIDS {
	catch {exec kill $pid};
    }
    exit 0;
}

proc ShowWebPage {url} {
    global BrowserPIDS;

    if { $::tcl_platform(platform) == "macintosh"} {
	lappend BrowserPIDS [exec osascript -e "\"open location $url\""]
	return 
    }
    set BrowserFound 0;
    foreach Browser $::BrowserList {
	if { [string length [auto_execok $Browser]]} {
	    set BrowserFound 1;
	    break ;
	} else {
	    ShowMessage [format \
	     [_ "The browser %s is not available on this machine or not in your path."]\
		 $Browser];
	}
    }
    if {$BrowserFound} {
	lappend BrowserPIDS [exec $Browser $url &]
    } else {
	ShowMessage [_ "No browser on the browser list was located."]
    }
}

proc ShowMessage {msg} {
    if {[winfo exists $::MSG]} {
	$::MSG configure -state normal;
	$::MSG delete 1.0 end;
	$::MSG insert 1.0 $msg;
	$::MSG configure -state disabled;
    } else {
	puts $msg;
    }
}

proc ClearMessageWindow {} {
     $::MSG configure -state normal
     $::MSG delete 1.0 end;
     $::MSG configure -state disabled
}


#If the filename passed as argument is a pathname
#leading to a file in the current working directory,
#return just the basename+extension. Otherwise
#return the argument.
proc MinimizeFileName {s} {
    set cwd [pwd];
    set sdir [file dirname $s]
    if {[string equal $cwd $sdir]} {
	return [file tail $s]
    } else {
	return $s;
    }
}

proc PopupDown {n} {
    global PopupList;
    if {[info exists PopupList($n)]} {
	if {[winfo exists $PopupList($n)]} {
	    destroy $PopupList($n);
	    return 1;
	}
    }
    return 0;
}

proc SortAvailableP {} {
    if {[string length [auto_execok sort]]} {
	return 1;
    }
    return 0;
}

proc MsortAvailableP {} {
    if {[string length [auto_execok msort]]} {
	return 1;
    }
    return 0;
}

proc MinpairAvailableQ {} {
    if {[string length [auto_execok minpair]]} {
	set ::MinpairVersion [FindoutMinpairVersion];
	return 1;
    }
    return 0;
}

proc FindoutMinpairVersion {} {
    set TmpFile [file join $::TempDir "mpg.txt"]
    catch {[exec minpair -v 2> $TmpFile]}
    if {[catch {open $TmpFile "r"} TmpHandle ] != 0} {
	ShowMessage [format [_ "Mysterious failure to open temporary file %s."] $TmpFile];
	return ;
    }
    if {[gets $TmpHandle vline] > 0} {
	return [lindex [split $vline] 1]
    }
    return [_ "unknown"]
}


proc BindInsertionTarget {v} {
    bind $v <FocusIn> "+SetInsertionTargets $v"
}

proc ScaleMoveBigIncrement {w k x y} {
    set part [$w identify $x $y]
    set inc 0;
    switch -exact -- $part {
	trough1 {
	    set dir -1;
	}
	trough2 {
	    set dir  1;
	}
	default {
	    return ;
	}
    }
    set Resolution [$w cget -resolution]
    set CurrentValue [$w get]
    set Delta [expr $dir * $k * $Resolution]
    $w set [expr $CurrentValue + $Delta]
}

#This provides an addition to the default methods of the Scrollbar
#widget. The default method, bound to <B1>, #increments or decrements
#the value by the resolution, by default 1. If you want a larger
#increment, you can use a larger resolution, but sometimes it is
#convenient to have a finer resolutio available while also being
#able to move in larger increments. A solution is to bind to
#<B3>, or some other event if you wish, a larger movement.
#This procedure moves by the specified multiplier k times
#the resolution of the Scrollbar. 

proc ScrollbarMoveBigIncrement {w f x y} {
    set part [$w identify $x $y]
    switch -exact -- $part {
	trough1 {
	    set dir -1;
	}
	arrow1 {
	    set dir -1;
	}
	trough2 {
	    set dir  1;
	}
	arrow2 {
	    set dir  1;
	}
	default {
	    return ;
	}
    }
    set CurrentFraction [lindex [$w get] 0]
    set NewFraction [expr $CurrentFraction + ($dir * $f)]
    eval [concat [$w cget -command] moveto $NewFraction]
}

set WhichFontToSet MainFont
#Font system begins here

# We set these outside the namespace since FontInfo has to be global.
# Default defaults

set FontInfo(family) courier
set FontInfo(size) 12
set FontInfo(weight) normal
set FontInfo(slant) roman
set FontInfo(underline) 0
set FontInfo(overstrike) 0

namespace eval fontsel {
    variable FontPropertyList [list family size weight slant underline overstrike]
    variable FontPropertyClass
    variable FSCColors

    foreach p $FontPropertyList {
	set FontPropertyClass($p) generic;
    }
    set FontPropertyClass(underline) boolean
    set FontPropertyClass(overstrike) boolean

    #We check the existence of the variable so that this
    #can safely be called AFTER non-default values are set.
    proc SetFontInfoDefaults {} {
	global FontInfo;
	foreach ft $::FontList {
	    if {![info exist FontInfo($ft,family)]} {
		set FontInfo($ft,family)	$FontInfo(family)
	    }
	    if {![info exist FontInfo($ft,size)]} {
		set FontInfo($ft,size)	$FontInfo(size)
	    }
	    if {![info exist FontInfo($ft,weight)]} {
		set FontInfo($ft,weight)	$FontInfo(weight)
	    }
	    if {![info exist FontInfo($ft,slant)]} {
		set FontInfo($ft,slant)	$FontInfo(slant)
	    }
	    if {![info exist FontInfo($ft,underline)]} {
		set FontInfo($ft,underline)	$FontInfo(underline)
	    }
	    if {![info exist FontInfo($ft,overstrike)]} {
		set FontInfo($ft,overstrike)	$FontInfo(overstrike)
	    }
	    if {![info exist FontInfo($ft,gloss)]} {
		set FontInfo($ft,gloss)	$ft;
	    }
	    if {![info exist FontInfo($ft,help)]} {
		set FontInfo($ft,help)	"?"
	    }
	}
    }

    #For a given font assign font-specific properties from generics.
    proc FontSet {which} {
	global FontInfo
	variable FontPropertyList
	foreach prop $FontPropertyList {
	    set FontInfo($which,$prop) $FontInfo($prop);
	}
    }

    #Assign the proprety values of a specified font to the generics.
    proc InverseFontSet {which} {
	global FontInfo
	variable FontPropertyList
	foreach prop $FontPropertyList {
	    set FontInfo($prop) $FontInfo($which,$prop);
	}
    }

    #Configure a specified font according to the font-specific properties.
    proc ConfigureFont {which} {
	global FontInfo
	variable FontPropertyList
	foreach prop $FontPropertyList {
	    font configure $which -$prop $FontInfo($which,$prop)
	}
    }

    #Record as default values the font-specific values of properties at the
    #time of the call. The recorded values are intended for use by
    #ResetToDefaults 
    proc RecordDefaults {} {
	global FontInfo
	global FontList;
	variable FontPropertyList

	foreach f $FontList {
	    foreach p $FontPropertyList {
		set FontInfo($f,$p,Default) $FontInfo($f,$p);
	    }
	}
    }

    #Create all of the fonts on the font list and configure them
    #according to the generic values of the properties, which are
    #also assigned at this time to the font-specific variables.
    proc CreateFonts {} {
	global FontList;

	foreach ft $FontList {
	    font create $ft;
	    InverseFontSet $ft;
	    ConfigureFont $ft;
	}
    
    }

    #Get the values from the three scales, combine them into a ctr RGB spec,
    #assign its value to FontExampleColor, and configure the two examples.
    proc UpdateFontExampleColor {x} {
	variable FSCColors
	global FontExampleColorConfigureWhich

	UpdateFontExampleColorCtr;
	.selectFont.curcan.att.r1c1 configure \
	    -$::FontExampleColorConfigureWhich  \#$::FontExampleColor
	.selectFont.curcan.att.r2c1 configure \
	    -$::FontExampleColorConfigureWhich \#$::FontExampleColor
	set FSCColors(red,$FontExampleColorConfigureWhich)   $::FontRed;
	set FSCColors(green,$FontExampleColorConfigureWhich) $::FontGreen;
	set FSCColors(blue,$FontExampleColorConfigureWhich)  $::FontBlue;
    }

    proc UpdateFontExampleColorCtr {} {
	set ::FontExampleColor [format "%02X%02X%02X" $::FontRed $::FontGreen $::FontBlue];
    }

    proc RestoreCSColors {} {
	variable FSCColors
	global FontExampleColorConfigureWhich
	set ::FontRed   $FSCColors(red,$FontExampleColorConfigureWhich)
	set ::FontGreen $FSCColors(green,$FontExampleColorConfigureWhich)
	set ::FontBlue  $FSCColors(blue,$FontExampleColorConfigureWhich)
	UpdateFontExampleColorCtr;
    }

    proc CloneMainFontAsFontControlPanelFont {} {
	variable FontPropertyList
	global FontInfo

	foreach p $FontPropertyList {
	    font configure FontControlPanelFont -$p $FontInfo(MainFont,$p);
	}
    }

    #This is the main procedure of this package.
    #It creates the font control panel.
    proc CreateFontControlPanel {args} {
	global FontInfo;
	global WhichFontToSet
	global FontList;
	variable FontPropertyList
	variable FSCColors

	set General 1
	set exwid 10

	set w .selectFont
	if {[winfo exists $w]} {
	    wm deiconify $w;
	    raise $w;
	    return ;
	}
	toplevel $w
	wm title $w [_ "Font Selection"]
	BindKeys $w
	wm protocol $w WM_DELETE_WINDOW "wm withdraw $w"

	#We create this separately because we don't want it on the FontList.
	font create FontControlPanelFont;
	
	#The idea is that in general we want to use the main
	#font here, but we keep them separate so that we
	#can wait before switching the control panel to
	#the new main font configuration so that
	#the user isn't confronted with an unusable
	#control panel if he or she sets the mainfont to something
	#crazy.
	CloneMainFontAsFontControlPanelFont;

	if {[llength $args] > 0} {
	    set ::WhichFontToSet [lindex $args 0];
	    set General 0;
	} 
	set FontTypes [llength $FontList];
	if {$FontTypes == 1} {
	    set ::WhichFontToSet [lindex $FontList 0];
	    set General 0;
	}

	frame $w.which -relief ridge -border 3
	if {$General} {
	    set msg [_ "For what aspect of the program\ndo you wish to set the font?"]
	    balloonhelp_for $w.which $msg
	    label $w.which.title -text [_ "Use for which to set font"] \
		-anchor w -font FontControlPanelFont
	    frame $w.which.btns;
	    set k 0;
	    set PerRow 4;
	    set ypad 3
	    set xpad 10
	    foreach ft $FontList {
		set bn $w.which.btns.f$ft;
		radiobutton $bn -text [_ $FontInfo($ft,gloss)]  \
		    -variable WhichFontToSet -value $ft \
		    -command fontsel::SetFontSelectionDefaults \
		    -font FontControlPanelFont \
		    -indicatoron 0 -selectcolor $::ColorSpecs(Default,Background)
		balloonhelp_for $w.which.btns.f$ft $FontInfo($ft,help)
		set row [expr 1+ $k/$PerRow]
		set col [expr $k%$PerRow]
		if {$FontTypes >= $PerRow} {
		    grid $w.which.btns.f$ft -row $row -column $col \
			-sticky we -ipadx 3 -padx $xpad -pady $ypad
		}
		incr k
	    }
	    if {$FontTypes < $PerRow} {
		foreach ft $FontList {
		    pack $w.which.btns.f$ft -side left -expand 1 -fill x \
			-ipadx 5 -ipady 3 -padx 40 -pady 3
		}
	    }
	    frame $w.which.pad  -height 4
	    pack $w.which.title -side top -expand 1 -fill both -anchor w
	    pack $w.which.btns  -side top -expand 1 -fill both -anchor w
	    pack $w.which.pad   -side top -expand 0 -fill x    -anchor w

	}

	frame $w.mid 
	frame $w.mid.opts  -relief ridge -border 3
	frame $w.mid.families  -relief ridge -border 3
	label $w.mid.families.lab -text [_ "Family"] -relief ridge \
	    -justify center -font FontControlPanelFont
	listbox $w.mid.families.lb -height 1 -exportselection 0 \
	    -yscrollcommand "$w.mid.families.sbar set" -selectmode single \
	    -font FontControlPanelFont
	scrollbar $w.mid.families.sbar -orient vertical -command "$w.mid.families.lb yview"
	bind $w.mid.families.sbar <<B3>> \
	    "ScrollbarMoveBigIncrement $w.mid.families.sbar 0.20 %x %y"

	bind $w.mid.families.lb <<B3>> ProvideFontDescription

	pack $w.mid.families.lab  -side top -expand 0 -fill x
	pack $w.mid.families.lb   -side left -expand 1 -fill both
	pack $w.mid.families.sbar -side left -expand 1 -fill y

	label $w.mid.opts.lab -text [_ "Attributes"] -relief ridge \
	    -justify center -font FontControlPanelFont
	label $w.mid.opts.sizel -text [_ "Size:"]  -font FontControlPanelFont
	scale $w.mid.opts.size -orient h -digit 1 -from 5 -to 55 \
	    -variable FontInfo(size)  -tickinterval 0 -length 150 \
	    -font FontControlPanelFont
	bind $w.mid.opts.size <<B3>> "ScaleMoveBigIncrement $w.mid.opts.size 5 %x %y"

	label $w.mid.opts.weightl -text [::msgcat::mc "Bold:"] \
	    -font FontControlPanelFont	    
	checkbutton $w.mid.opts.weight -variable FontInfo(weight) \
	    -onvalue bold -offvalue normal  -font FontControlPanelFont

	label $w.mid.opts.slantl -text [::msgcat::mc "Italic:"] \
	    -font FontControlPanelFont
	checkbutton $w.mid.opts.slant -variable FontInfo(slant) \
	    -onvalue italic -offvalue roman -font FontControlPanelFont

	label $w.mid.opts.ulinel -text [::msgcat::mc "Underline:"] \
	    -font FontControlPanelFont
	checkbutton $w.mid.opts.uline -variable FontInfo(underline) \
	    -onvalue 1 -offvalue 0  -font FontControlPanelFont

	label $w.mid.opts.strkovl -text [::msgcat::mc "Overstrike:"] \
	    -font FontControlPanelFont
	checkbutton $w.mid.opts.strkov -variable FontInfo(overstrike) \
	    -onvalue 1 -offvalue 0  -font FontControlPanelFont

	grid $w.mid.opts.lab   -row 0 -column 0 -columnspan 2 -sticky ew
	grid $w.mid.opts.sizel -row 1 -column 0 -sticky w
	grid $w.mid.opts.size -row 1 -column 1 -sticky w
	grid $w.mid.opts.weightl -row 2 -column 0 -sticky w
	grid $w.mid.opts.weight -row 2 -column 1 -sticky w
	grid $w.mid.opts.slantl -row 3 -column 0 -sticky w
	grid $w.mid.opts.slant -row 3 -column 1 -sticky w
	grid $w.mid.opts.ulinel -row 4 -column 0 -sticky w
	grid $w.mid.opts.uline -row 4 -column 1 -sticky w
	grid $w.mid.opts.strkovl -row 5 -column 0 -sticky w
	grid $w.mid.opts.strkov -row 5 -column 1 -sticky w

	pack $w.mid.families -side left -expand 1 -fill both -padx 5 -pady 2
	pack $w.mid.opts     -side left -expand 1 -fill both -padx 5 -pady 2

	eval $w.mid.families.lb insert 0 [lsort [font families]]

	bind $w.mid.families.lb <ButtonPress-1> {fontsel::SelectFontFamily %W %y}
	bind $w.mid.families.lb <B1-Motion> {fontsel::SelectFontFamily %W %y}
	bind $w.mid.families.lb <Activate> {fontsel::SelectFontFamily %W %y}

	frame $w.cntls -relief ridge -border 3
	button $w.cntls.sav -text [_ "Apply"]  -command fontsel::FontSave \
	    -font FontControlPanelFont
	button $w.cntls.res -text [_ "Reset"] -command  fontsel::ResetToDefaults \
	    -font FontControlPanelFont
	button $w.cntls.can -text [_ "Dismiss"] -command "wm withdraw $w" \
	    -font FontControlPanelFont
	button $w.cntls.cpanel -text [_ "Here Too?"] \
	    -command fontsel::CloneMainFontAsFontControlPanelFont \
	-font FontControlPanelFont
	pack $w.cntls.cpanel -side left  -expand 1 -fill both -padx 3
	pack $w.cntls.can    -side left  -expand 1 -fill both -padx 3
	pack $w.cntls.res    -side left  -expand 1 -fill both -padx 3
	pack $w.cntls.sav    -side right -expand 1 -fill both -padx 3

	set msg [_ "Press this button to dismiss the popup."];
	balloonhelp_for $w.cntls.can $msg
	set msg [_ "Press this button to reset the font to its defaults."];
	balloonhelp_for $w.cntls.res $msg
	set msg [_ "Press this button to make the changes\nyou have made take effect."]
	balloonhelp_for $w.cntls.sav $msg
	set msg [format \
	     [_ "Press this button to set the fonts\nin this control panel to the %s font."] \
		     [_ $::FontInfo(MainFont,gloss)]]
	balloonhelp_for $w.cntls.cpanel $msg

	frame $w.curcan -relief ridge -border 1
	set ATT [frame $w.curcan.att]
	set Row 0
	foreach r {Header Current Candidate} {
	    set Column 2
	    foreach p $FontPropertyList {
		set v $w.curcan.att.r${Row}c${Column};
		label $v  -font FontControlPanelFont
		incr Column;
	    } 
	    incr Row;
	}

	font create CurrentFontExampleFont \
	    -family $FontInfo($::WhichFontToSet,family) \
	    -size $FontInfo($::WhichFontToSet,size) \
	    -weight $FontInfo($::WhichFontToSet,weight) \
	    -slant $FontInfo($::WhichFontToSet,slant) \
	    -underline $FontInfo($::WhichFontToSet,underline) \
	    -overstrike $FontInfo($::WhichFontToSet,overstrike);

	font create CandidateFontExampleFont \
	    -family $FontInfo($::WhichFontToSet,family) \
	    -size $FontInfo($::WhichFontToSet,size) \
	    -weight $FontInfo($::WhichFontToSet,weight) \
	    -slant $FontInfo($::WhichFontToSet,slant) \
	    -underline $FontInfo($::WhichFontToSet,underline) \
	    -overstrike $FontInfo($::WhichFontToSet,overstrike);

	label $w.curcan.att.r0c0 -font FontControlPanelFont
	label $w.curcan.att.r0c1 -font FontControlPanelFont
	label $w.curcan.att.r1c0  -text [_ "Current"] -font FontControlPanelFont
	entry $w.curcan.att.r1c1 -font CurrentFontExampleFont -relief flat -width $exwid
	$w.curcan.att.r1c1 insert 0 [_ "example"]
	label $w.curcan.att.r2c0  -text [_ "Candidate"] -font FontControlPanelFont
	entry $w.curcan.att.r2c1 -font CandidateFontExampleFont -relief flat  -width $exwid
	$w.curcan.att.r2c1 insert 0 [_ "example"]

	#This is for use in conjunction with my character insertion library.
	#It arranges for the various character insertion widgets to insert
	#characters into the focussed window.
	if {[llength [info commands BindInsertionTarget]]} {
	    BindInsertionTarget $w.curcan.att.r1c1;
	    BindInsertionTarget $w.curcan.att.r2c1;
	}

	set CLR [frame $w.curcan.clr]
	set ln 60;
	set wd 10
	set trc \#D4B8C1
	scale $CLR.red -orient v -digit 1 -from 255 -to 0 -tickinterval 0 \
	    -variable FontRed  -showvalue 0 -length $ln -width $wd\
	    -troughcolor $trc -activebackground \#F0AAAA \
	    -bg red -fg yellow -command fontsel::UpdateFontExampleColor
	scale $CLR.grn -orient v -digit 1 -from 255 -to 0 -tickinterval 0 \
	    -variable FontGreen  -showvalue 0 -length $ln -width $wd\
	    -troughcolor $trc -activebackground \#AAF0AA \
	    -bg green -fg yellow -command fontsel::UpdateFontExampleColor
	scale $CLR.blu -orient v -digit 1 -from 255 -to 0 -tickinterval 0 \
	    -variable FontBlue  -showvalue 0 -length $ln -width $wd\
	    -troughcolor $trc -activebackground \#AAAAF0 \
	    -bg blue -fg yellow -command fontsel::UpdateFontExampleColor
	set FSCColors(red,bg)   0x00
	set FSCColors(green,bg) 0x00
	set FSCColors(blue,bg)  0x00
	set FSCColors(red,fg)   0x00
	set FSCColors(green,fg) 0x00
	set FSCColors(blue,fg)  0x00

	bind $CLR.red <<B3>> "ScaleMoveBigIncrement $CLR.red -20 %x %y"
	bind $CLR.grn <<B3>> "ScaleMoveBigIncrement $CLR.grn -20 %x %y"
	bind $CLR.blu <<B3>> "ScaleMoveBigIncrement $CLR.blu -20 %x %y"

	set ::FontRed   255
	set ::FontGreen 255
	set ::FontBlue  255
	set FontExampleColor \#FFFFFF
	label $CLR.rdt -textvariable FontExampleColor -width 7 -relief raised
	bind $CLR.rdt <Button-1> "pack forget $CLR"
	bind $ATT <Button-1> "pack $w.curcan.clr -before $ATT -side left -expand 1 -fill both"
	set msg [_ "Left click here to remove the color selector."]
	balloonhelp_for $CLR.rdt $msg;
	set msg [_ "Use this to set the colors in the example text boxes."]
	balloonhelp_for $CLR.red $msg;
	balloonhelp_for $CLR.grn $msg;
	balloonhelp_for $CLR.blu $msg;

	frame $CLR.whi
	set ::FontExampleColorConfigureWhich bg;
	radiobutton $CLR.whi.t -variable FontExampleColorConfigureWhich \
	    -value fg -command fontsel::RestoreCSColors -text [_ "fg"] \
	    -indicatoron 0 -activebackground gray -selectcolor gray \
	    -font FontControlPanelFont
	radiobutton $CLR.whi.b -variable FontExampleColorConfigureWhich \
	    -value bg -command fontsel::RestoreCSColors -text [_ "bg"] \
	    -indicatoron 0 -activebackground gray -selectcolor gray \
	    -font FontControlPanelFont
	pack $CLR.whi.t -side left -expand 1 -fill both
	pack $CLR.whi.b -side left -expand 1 -fill both

	pack $CLR.rdt -side top -expand 1  -fill both
	pack $CLR.whi -side bottom -expand 1  -fill both
	pack $CLR.red -side left -expand 1 -fill y
	pack $CLR.grn -side left -expand 1 -fill y
	pack $CLR.blu -side left -expand 1 -fill y

	set msg [_ "Choose the color for the sample text."]
	balloonhelp_for $CLR.whi.t $msg;
	set msg [_ "Choose the background color for the sample text."]
	balloonhelp_for $CLR.whi.b $msg;

	#Now grid them all
	set xp 5
	set yp 3
	set CLimit [expr [llength $FontPropertyList] + 2];
	for {set r 0} {$r < 3} {incr r} {
	    for {set c 0} {$c < $CLimit} {incr c} {
		set v $w.curcan.att.r${r}c${c};
		grid $v -row $r -column $c -sticky w -padx $xp -pady $yp
	    } 
	}

	set xp 3
	set yp 1
	if {!$General} {
	    $w.curcan.att.r0c0 configure -text [string totitle [_ $FontInfo($::WhichFontToSet,gloss)]]
	}

	pack $w.curcan.clr -side left -expand 1 -fill both -padx 9 -pady 5
	pack $w.curcan.att -side left -expand 1 -fill both -padx 3 -pady 5

	#Put it all together
	frame $w.botsep -height 4
	pack $w.which -side top -expand 1 -fill x -padx 8 -pady 6
	pack $w.curcan -side top -expand 1 -fill x -padx 8 -pady 6
	pack $w.mid   -side top -expand 1 -fill x -padx 8 -pady 1
	pack $w.cntls -side top -expand 1 -fill x -padx 8 -pady 2
	pack $w.botsep -side top -expand 1 -fill x -padx 8 -pady 2
	if {$General} {
	    $w.which.btns.f[lindex $FontList 0] invoke;#  Default to first on list
	}
	foreach f $FontPropertyList {
	    trace variable FontInfo($f) w fontsel::FontShow;
	}
	return $w;
    }

    #This resets the properties of the font specified by WhichFontToSet
    #to their default values, where "default" means whatever values
    #are stored in FontInfo(<font>,<property>,Default). The intention
    #is that these values will have been set by RecordDefaults at
    #a suitable point. A suitable point might be program startup or it
    #might be after init files have been read.
    #After resetting the attributes for the current font to their
    #default values, it configures the fonts for both the current and candidate
    #examples in the control panel to match.
    proc ResetToDefaults {} {
	global FontInfo;
	global WhichFontToSet
	variable FontPropertyList

	foreach prop $FontPropertyList {
	    set FontInfo($::WhichFontToSet,$prop) $FontInfo($::WhichFontToSet,$prop,Default);
	    set FontInfo($prop) $FontInfo($::WhichFontToSet,$prop,Default);
	}
	ConfigureFont $::WhichFontToSet;
	ConfigureCurrentExampleFont;
	ConfigureCandidateExampleFont;
	DescribeBothFonts;
    }

    #This is the procedure called from the control panel when the user decides
    #to apply and save the values set on the control panel. It sets the font-specific
    #attributes from the generic ones, configures the target font as well as the
    #current example font, and updates the description of the current font
    #in the control panel.
    proc FontSave {} {
	global FontInfo

	set f $::WhichFontToSet;
	FontSet $f;
	ConfigureFont $f
	ConfigureCurrentExampleFont;
	DescribeBothFonts;
    } 

    #This procedure configures the font used to display the candidate example
    #in the control panel. It is intended to be called whenever a property
    #is changed in the control panel or when the target font is changed.
    proc ConfigureCandidateExampleFont {} {
	global FontInfo
	global WhichFontToSet
	variable FontPropertyList

	foreach prop $FontPropertyList {
	    font configure CandidateFontExampleFont -$prop $FontInfo($prop)
	}
    }

    #This procedure configures the font used to display the current example
    #in the control panel. It is intended to be called when the user
    #applies new values or when the target font is changed.
    proc ConfigureCurrentExampleFont {} {
	global FontInfo
	global WhichFontToSet
	variable FontPropertyList

	foreach prop $FontPropertyList {
	    font configure CurrentFontExampleFont -$prop $FontInfo($::WhichFontToSet,$prop)
	}
    }

    #This procedure configures the candidate example font#and updates the
    #description of the candidate font in the control panel. It is intended
    #to be called whenever a proprty of the candidate font is changed.
    proc FontShow {args} {
	ConfigureCandidateExampleFont;
	DescribeBothFonts;
    }

    #This procedure generates the descriptions of the current and candidate
    #fonts that are shown in the control panel.
    proc DescribeBothFonts {} {
	global FontInfo
	global WhichFontToSet;
	variable FontPropertyList

	set f $::WhichFontToSet;
	set c 2;
	foreach prop $FontPropertyList {
	    .selectFont.curcan.att.r0c$c configure -text  [string totitle [_ $prop]] 
	    set value $FontInfo($f,$prop)
	    if {$value == 1} {set value [_ "Yes"]} else {
		if {$value == 0} {set value [_ "No"]}}
	    .selectFont.curcan.att.r1c$c configure -text $value;
	    set value $FontInfo($prop)
	    if {$value == 1} {set value [_ "Yes"]} else {
		if {$value == 0} {set value [_ "No"]}}
	    .selectFont.curcan.att.r2c$c configure -text $value; 
	    incr c;
	}
    }

#This is to be called when we change font targets.
#It resets the control panel to show the current
#configuration for the newly targeted font.
    proc SetFontSelectionDefaults {} {
	global FontInfo;
	global WhichFontToSet;
	variable FontPropertyList

	foreach prop $FontPropertyList {
	    set FontInfo($prop) $FontInfo($WhichFontToSet,$prop);
	}
	set FontChoices [.selectFont.mid.families.lb size];
	set k 1;
	while {$k <= $FontChoices} {
	    set Family [.selectFont.mid.families.lb get $k];
	    if {[string equal $Family [string tolower $FontInfo($WhichFontToSet,family)]]} {
		.selectFont.mid.families.lb selection clear 0 end;
		.selectFont.mid.families.lb see $k
		.selectFont.mid.families.lb selection set $k $k;
		.selectFont.mid.families.lb activate $k;
		set FontInfo(family) [.selectFont.mid.families.lb get $k]
		break
	    }
	    incr k;
	}
	ConfigureCandidateExampleFont
	ConfigureCurrentExampleFont;
	DescribeBothFonts;
    }

    #This is the callback that extracts a new font family
    #setting from the control panel.
    proc SelectFontFamily {w y} {
	global FontInfo

	set index [$w nearest $y]
	if {$index != ""} {
	    set FontInfo(family) [$w get $index]
	}
    }

    #This is the callback that extracts a new font size
    #from the control panel.
    proc SelectFontSize {w} {
	global WhichFontToSet
	global FontInfo

	set size [$w get]
	if {[catch {expr round($size)} size] == 0} {
	    if { ($size > 0) && ($size <= 60) } {
		set FontInfo(size) $size
		return ;
	    }
	}
	$w delete 0 end
	$w insert 0 $FontInfo($WhichFontToSet,size)
    }

    #This is presently a stub.
    #It is intended to be called on an event, probably <B3>,
    #in the font family listbox in the control panel.
    #Its purpose is to provide the user with information about
    #the particular font. 
    proc ProvideFontDescription {w y} {
	variable FontDescriptions

	set index [$w nearest $y]
	if {$index != ""} {
	    FontDescriptions([$w get $index])
	}
    }

    #Create a command for setting each combination of font and property.
    proc DefineFontSettingProcs {} {
	variable FontPropertyList
	variable FontPropertyClass
	global FontList;
	global FontProcList;

	foreach ft $FontList {
	    foreach prop $fontsel::FontPropertyList {
		if {[string equal $FontPropertyClass($prop) boolean]} {
		    set ProcName  [format "::fontsel::Set%s%sP" $ft [string totitle $prop]];
		    lappend FontProcList $ProcName;
		    set cmd [list "proc"  $ProcName  "\{v\}" \
			 "set ::FontInfo($ft,$prop) \[SlaveBoolean \$v\];\nConfigureFont $ft;"]
		} else {
		    set ProcName  [format "::fontsel::Set%s%s" $ft [string totitle $prop]];
		    lappend FontProcList $ProcName;
		    set cmd [list "proc" $ProcName "\{v\}" \
			 "set ::FontInfo($ft,$prop) \$v;\nConfigureFont $ft;"]
		}
		eval $cmd;
	    }
	}
    }

    #Create aliases in the named daughter interpreter for the font setting commands
    proc AliasFontSettings {interp} {
	variable FontPropertyList
	variable FontPropertyClass
	global FontList;
	
	foreach ft $FontList {
	    foreach prop $FontPropertyList {
		set Tprop [string totitle $prop]
		set Tprop2 $Tprop;
		if {[string equal $FontPropertyClass($prop) boolean]} {
		    set Tprop ${Tprop}P
		}
		eval [split [format "%s alias %s%s ::fontsel::Set%s%s" \
			 $interp $ft $Tprop $ft $Tprop]]
	    }
	}
    }

    #Returns a list of init file commands representing the current font settings.
    proc SaveFontSettings {} {
	variable FontPropertyList
	variable FontPropertyClass
	global FontInfo
	global FontList;
	
	set cl [list]
	foreach ft $FontList {
	    foreach prop $FontPropertyList {
		set Tprop [string totitle $prop]
		if {[string equal $FontPropertyClass($prop) boolean]} {
		    set Tprop ${Tprop}P
		}
		lappend cl [format "%s%s %s" $ft $Tprop [list $FontInfo($ft,$prop)]]
	    }
	}
	return [lsort $cl];
    }
}


namespace import ::msgcat::mcset
mcset de "Dismiss" "Abbrechen"
mcset de "bold" "Fett"
mcset de "Bold:" "Fett:"
mcset de "italic" "Kursiv"
mcset de "Italic:" "Kursiv:"
mcset de "Yes" "Ja"
mcset de "No" "Nein"
mcset de "underline" "Unterstrichen"
mcset de "Underline:" "Unterstrichen:"
mcset de "overstrike" "Durchgestrichen"
mcset de "Overstrike:" "Durchgestrichen:"
mcset de "size" "Gr\u00F6\u00DFe"
mcset de "Size:" "Gr\u00F6\u00DFe:"
mcset de "Font Selection" "Schriftart ausw\u00E4hlen"

# End of namespace fontsel

# This is the stuff specific to this program.

set FontList { \
MainFont\
BalloonHelpFont\
MenuFont\
UserPlainTextFont\
UserTechnicalTextFont\
CharacterEntryFont\
CharacterEntryLabelFont\
DiacriticFont\
AccentedLetterFont}

set FontInfo(family) code2000

set FontInfo(ExclusionEntryFont,size) 15;
set FontInfo(ExclusionEntryFont,gloss)  [_ "exclusion entry"]
set FontInfo(ExclusionEntryFont,help) [_ "This font is used in the exclusion entry box"]

set FontInfo(BalloonHelpFont,family) lucida
set FontInfo(BalloonHelpFont,size) 13
set FontInfo(BalloonHelpFont,gloss) "balloon help"
set FontInfo(BalloonHelpFont,help) "This font is used in help balloons."

set FontInfo(MainFont,family) "courier"
set FontInfo(MainFont,size) 12
set FontInfo(MainFont,weight) bold
set FontInfo(MainFont,gloss) [_ "general"]
set FontInfo(MainFont,help) [_ "This font is used for most things."]

set FontInfo(UserPlainTextFont,gloss) [_ "plain user text"]
set FontInfo(UserPlainTextFont,help) [_ "This font is used for ordinary text entered\nby the user, such as file names."]

set FontInfo(UserTechnicalTextFont,gloss) [_ "technical user text"]
set FontInfo(UserTechnicalTextFont,help) [_ "This font is used for text entered by the\nuser that is likely to contain unusual\ncharacters, such as multigraph definitions."]
set FontInfo(UserTechnicalTextFont,family) "doulos sil"

set FontInfo(MenuFont,family) courier
set FontInfo(MenuFont,size) 12
set FontInfo(MenuFont,gloss) [_ "menu"]
set FontInfo(MenuFont,help) [_ "This font is used for menu labels."]

set FontInfo(CharacterEntryFont,gloss) "character entry"
set FontInfo(CharacterEntryFont,help) \
"This font is used on the labels of\ncharacter insertion widgets."
set FontInfo(AccentedLetterFont,gloss) "accented letters"
set FontInfo(AccentedLetterFont,help) \
"This font is used on the labels of the\naccented character insertion widget."
set FontInfo(AccentedLetterFont,family) "doulos sil"

set FontInfo(CharacterEntryLabelFont,family) bible
set FontInfo(CharacterEntryLabelFont,size) 13
set FontInfo(CharacterEntryLabelFont,gloss) "character entry labels"
set FontInfo(CharacterEntryLabelFont,help) \
"This font is used for the row and column\nlabelsin character entry widgets."

set FontInfo(DiacriticFont,gloss) "diacritics"
set FontInfo(DiacriticFont,help) \
"This font is used on the labels of the\ndiacritic insertion widget."

set FontInfo(CharacterEntryFont,size) 14
set FontInfo(CharacterEntryFont,family) "doulos sil"
set FontInfo(AccentedLetterFont,size) 14
set FontInfo(DiacriticFont,size)  [expr $FontInfo(CharacterEntryFont,size) + 40];

fontsel::SetFontInfoDefaults;
fontsel::CreateFonts;

option add *selectFont*Font MainFont 100
option add *Label.Font MainFont 100
option add *Menu.Font MenuFont 100
option add *Radiobutton.Font MainFont 100
option add *Scale.Font MainFont 100

#End of font stuff

#Because of the strangeness needed in ExecuteMinpair, this constructs everything
#except for the diversion of stdout to the output file.
proc ConstructCommandLine {} {
    set cl [list minpair];
    lappend cl "-q";
    if {$::DoSingleP == 1} {
	lappend cl "-s";
    }
    if {$::DoTranspositionsP == 1} {
	lappend cl "-t";
    }
    if {$::DoIndelsP == 1} {
	lappend cl "-i";
    }
    if {[string equal $::DifferenceSiteMarker ""] ==0} {
	lappend cl "-m"
	lappend cl $::DifferenceSiteMarker
    }
    lappend cl "-l"
    lappend cl $::MaxWordLength;
    if {$::DefineMultigraphsP} {
	set mdf [.gen.mlg.bot.ent get]
	if {[string equal $mdf ""]} {
	    ShowMessage [_ "No input file has been specified. You must supply one."]
	    return "err";
	}
	if {[file exists $mdf] == 0} {
	    ShowMessage [format [_ "Input file \`%s\' does not exist."] $mdf]
	    return "err";
	}
	if {[file readable $mdf] == 0} {
	    ShowMessage [format [_ "Input file \`%s\' is not readable."] $mdf]
	    return "err";
	}
	lappend cl "-d"
	lappend cl $mdf;
    }

    lappend cl "<";
    set inf [.gen.if.ent get]
    if {[string equal $inf ""]} {
	ShowMessage [_ "No input file has been specified. You must supply one."]
	return "err";
    }
    if {[file exists $inf] == 0} {
	ShowMessage [format [_ "Input file \`%s\' does not exist."] $inf]
	return "err";
    }
    if {[file readable $inf] == 0} {
	ShowMessage [format [_ "Input file \`%s\' is not readable."] $inf]
	return "err";
    }
    lappend cl $inf;
    set outf [.gen.of.ent get]
    if {[string equal $outf ""]} {
	ShowMessage [_ "No output file has been specified. You must supply one."]
	return "err";
    }
    if {[file exists $outf]} {
	if {[file writable $outf] == 0} {
	    ShowMessage [format [_ "Output file \`%s\' is not writable."] $outf]
	    return "err";
	}
    }
    return [join $cl]; 
}

set ElapsedTime 0;
set StdoutResult "";
proc MinpairPipeEvent {PipeHandle Start} {
    global ElapsedTime
    global StdoutResult
    fconfigure $PipeHandle -blocking 0
    set data [read $PipeHandle]
    if {[eof $PipeHandle]} {
	set ElapsedTime [expr {[clock seconds] - $Start}]
	if {$::MinpairAbortedP == 0} {
	    ShowMessage [format [_ "Elapsed time: %d seconds"] $ElapsedTime];
	}
	if {[catch {close $PipeHandle} msg]} {
            ShowMessage [_ "Minpair exited abnormally."]
	}
    } elseif {![fblocked $PipeHandle]} {
	append StdoutResult $data
    }
}

proc Execute {} {
    set ::DifferenceSiteMarker  [.gen.ctr.dsm.ent get];
    set ::MaxWordLength [.gen.ctr.mwl.spb get]
    set ::MaximumDistance [.gen.ctr.mds.spb get]
    if {$::MinpairAvailableP && $::UseMinpairP} {
	ExecuteMinpair
    } else {
	set StartTime [clock seconds]
	set PairsFound [FindMinimalPairs \
	    [MinimizeFileName [.gen.if.ent get]] \
	    [MinimizeFileName [.gen.of.ent get]] \
	    $::HowManyDiffs \
	    $::MaximumDistance \
	    [MinimizeFileName [.gen.mlg.bot.ent get]]]
	set TimeElapsed [expr [clock seconds] - $StartTime]
	ShowMessage [format [_ "Found %d minimal pairs in %d seconds."] $PairsFound $TimeElapsed]
    }
}

set minpairpid 0;
proc ExecuteMinpair {} {
    set ::MinpairAbortedP 0;
    ClearMessageWindow;
    set cl [ConstructCommandLine]
    if {[string match minpair* $cl] == 0} {
	if {[string equal $cl ""]} {
	    ShowMessage [_ "Obligatory information is missing: cannot execute."];
	}
	return ;
    }
    set DoSortP $::SortOutputP;
    if {$DoSortP} {
	set SortCommand "";
	if {[MsortAvailableP]} {
	    set SortCommand msort
	} elseif {[SortAvailable]} {
	    set SortCommand sort
	} else {
	    set DoSortP 0;
	    ShowMessage [_ "Neither msort nor sort is available so output will not be sorted."]
	}
    }
    set cl [split $cl];
    lappend cl "2>"
    lappend cl "/dev/null"
    set ::MinpairInProgressP 1;
    set StartTime [clock seconds]
    catch {open "|$cl" "r"} PipeID;
#    fconfigure $PipeID -blocking 0 -buffering none
    fileevent $PipeID readable [list MinpairPipeEvent $PipeID $StartTime]
    pack forget $::MSG;
    PackProgressBar
    PackMessageRegion;
    set plogfd [open "|tail -f $::ProgressFile" r]
    fconfigure $plogfd -blocking 0 -buffering none
    fileevent $plogfd readable [list UpdateProgress $plogfd $StartTime]
    set ::minpairpid [lindex [pid $PipeID] 0]
    tkwait variable ::ElapsedTime;
    set ::MinpairInProgressP 0;
    fileevent $plogfd readable {}
    SetProgress $::PBAR 0;
    pack forget $::PBAR
    if {$::MinpairAbortedP} { return }
    if {[string equal $::errorCode "NONE"] == 0} {
	set spec [split $::errorCode]
	switch -exact -- [lindex $spec 0] {
	    CHILDSTATUS {
		set MinpairErrorCode [lindex $spec 2];
		lappend msg [_ $::MinpairErrors($MinpairErrorCode)];
	    }
	    POSIX {
		lappend msg [lindex $spec 2]
	    }
	}
    }
    set OutputFile [.gen.of.ent get];
    if {$DoSortP} {
	#Pipe the output of minpair into the sort program
	#and have the sort program write its output into OutputFile.
	if {[string equal $SortCommand msort]} {
	    set cl [list msort -q -l -w]
	    if {$::DefineMultigraphsP} {
		set mdf [.gen.mlg.bot.ent get]
		lappend cl -d
		lappend cl $mdf
	    }
	    lappend cl ">";
	    lappend cl $OutputFile
	} else {
	    set cl [list sort ">" $OutputFile];
	}
	catch {open "|$cl" "w"} SortPipeID;
	puts -nonewline $SortPipeID $::StdoutResult
	close $SortPipeID;
    } else {
	if {[catch {open $OutputFile "w"} OutputHandle ] != 0} {
	    ShowMessage [format [_ "Unable to open file %s."] $OutputFile];
	    return ;
	}
	puts -nonewline $OutputHandle $::StdoutResult
	close $OutputHandle;
    }
    pack forget $::PBAR
}


proc UpdateProgress {fd Start} {
    if {[eof $fd]} {
	fileevent $fd readable {}
	close $fd;
	return ;
    }
    if {[gets $fd Percent] > 0} {
	SetProgress $::PBAR $Percent;
	set ET [expr [clock seconds] - $Start];
	set ETR [expr int(((100.0 / double($Percent)) - 1.0) * $ET)]
	ShowMessage [format [_ "Time Elapsed: %3ds Remaining: %3ds."] $ET $ETR]
    } else {
	dmsg "Empty read from plogfd." 
    }
}

proc ExplicateErrorStatus {} {
    global FeatureTestDebugP;
    global FeatureTestLogHandle;
    switch -exact -- [lindex $::errorCode 0] {
	CHILDKILLED {
	    foreach { - pid sigName msg } $::errorCode break
	    if {$FeatureTestDebugP} {
		puts $FeatureTestLogHandle [format "Child process %d died on signal %s" $pid $sigName];
		puts $FeatureTestLogHandle [format "Error message: %s" $msg];
	    }
	}
	CHILDSTATUS {
	    foreach { - pid code } $::errorCode break
	    if {$FeatureTestDebugP} {
		puts $FeatureTestLogHandle [format "Child process %d exited with error code %d" $pid $code];
	    }
	}
	CHILDSUSP {
	    foreach { - pid sigName msg } $::errorCode break
	    if {$FeatureTestDebugP} {
		puts $FeatureTestLogHandle [format "Child process %d has been suspended due to signal %d" $pid $sigName];
		puts $FeatureTestLogHandle [format "Error message: %s" $msg];
	    }
	}
	POSIX {
	    foreach { - errName msg } $::errorCode break
	    if {$FeatureTestDebugP} {
		puts $FeatureTestLogHandle [format "One of the kernel calls to launch the command failed ith error code %d" $code];
		puts $FeatureTestLogHandle [format "Error message: %s" $msg];
	    }
	}
    }
}


array set MinpairErrors {
    1 "Failed to open file"\
    2 "I/o error"\
    3 "Not really an error - provided information"\
    4 "Limit exceeded"\
    5 "Command line option not recognized"\
    6 "Invalid argument to command line option"\
    7 "Out of memory"\
    8 "Ill-formed record"\
    9 "Other error"
}

proc AbortMinpair {} {
    dmsg "AbortMnpair called"
    if {$::MinpairInProgressP} {
	if {$::tcl_platform(platform) eq "unix"} {
	    catch {exec kill -9 $::minpairpid}
	    set ::MinpairAbortedP 1;
	    ShowMessage [_ "Minpair aborted."]
	} else {
	    ShowMessage [_ "Sorry, this must be an MS Windows system."];
	}
    } else {
	ShowMessage [_ "No search is in progress."]
    }
}

proc SaveCommandLine {} {
    set cl [ConstructCommandLine];
    if {[string match minpair* $cl] == 0} {
	if {[string equal $cl ""]} {
	    ShowMessage [_ "Failed to construct command line."]
	}
	return ;
    }
    set outf [.gen.of.ent get]
    set cl [split $cl];
    lappend cl ">"
    lappend cl $outf 
    set cl [join $cl];
    set SaveFile [tk_getSaveFile -initialfile [_ "SavedMinpairCommandLine"]];
    if {$SaveFile == ""} {
	ShowMessage [_ "File selection cancelled."]
	return ;
    }
    set SaveFile [MinimizeFileName $SaveFile]
    if {[catch {open $SaveFile "w"} SaveHandle ] != 0} {
	ShowMessage [format [_ "Unable to open file %s."] $SaveFile];
	return ;
    }
    puts $SaveHandle $cl;
    close $SaveHandle;
    set ::LastCommandLineSaveTime  [clock seconds];
}

proc ShowCommandLine {} {
    set cl [ConstructCommandLine];
    if {[string match minpair* $cl] == 0} {
	if {[string equal $cl ""]} {
	    ShowMessage [_ "Failed to construct command line."]
	}
	return ;
    } else {
	set outf [.gen.of.ent get]
	set cl [split $cl];
	lappend cl ">"
	lappend cl $outf 
	ShowMessage [join $cl];
    }
}

proc InsertClipboardContents {w} {
    set sel "";
    if {[catch {set sel [selection get -selection CLIPBOARD -type UTF8_STRING]}]} {
	ShowMessage [_ "The clipboard is empty."];
	return;
    } else {
	$w insert insert $sel;
    }
}

proc SelectInputFile {} {
    set InputFile [tk_getOpenFile];
    if {$InputFile == ""} {
	ShowMessage [_ "File selection aborted."]
	return
    } else { 
	.gen.if.ent delete 0 end
	.gen.if.ent insert 0 [MinimizeFileName $InputFile]
    }
}

proc SelectOutputFile {} {
    set OutputFile [tk_getSaveFile -initialfile [_ "MinpairOutput"]];
    if {$OutputFile == ""} {
	ShowMessage [_ "File selection aborted."]
	return
    } else { 
	.gen.of.ent delete 0 end
	.gen.of.ent insert 0 [MinimizeFileName $OutputFile]
    }
}

array set DefaultKeyBindings {
"<Control-c>" 		  AbortMinpair
"<Control-d>"             ToggleIPAD
"<Control-b>" 		  ToggleBalloonHelp
"<Control-j>"             ToggleIPAC
"<Control-k>"             DescribeKeyBindings
"<Control-u>"             ToggleCharEntryByCode
"<Control-w>"             ToggleIPAV
};

if {[string equal $tcl_platform(platform) "unix"]} {
    set DefaultKeyBindings(<Control-c>) AbortMinpair;
}

proc SetCommandGlosses {} {
    global CommandGlosses;

    set Pairs [list\
	   AbortMinpair "[_ "Abort the currently running minpair job"]"\
	   ToggleCharEntryByCode  "[_ "Enter characters by their ctradecimal Unicode codes"]"\
	   ToggleIPAC "[_ "Toggle the IPA consonant entry widget."]"\
	   ToggleIPAV "[_ "Toggle the IPA vowel entry widget."]"\
		   ToggleBalloonHelp "[_ "Toggle Balloon Help"]"\
	   DescribeKeyBindings "[_ "Toggle Key Binding Help"]"\
	   ToggleIPAD "[_ "Toggle the IPA diacritic entry widget."]"]
    array set CommandGlosses $Pairs;
}

proc BindKeys {w {bl DefaultKeyBindings}} {
    upvar \#0 $bl kb;
    foreach KeySeq [array names kb] {
	bind $w $KeySeq $kb($KeySeq);
    }
}

proc DescribeKeyBindings {{bl DefaultKeyBindings}} {
    global CommandGlosses;
    upvar \#0 $bl kb;

    if {[PopupDown DescribeKeyBindings] ==1} {return}
    set po [CreateTextDisplay [_ "Key Bindings"] 88 6]
    set ::PopupList(DescribeKeyBindings) $po;
    BindKeys $po;

    #Invert array and get maximum length of gloss while we are at it.
    set MaxGlossLength 0;
    foreach KeySeq [array names kb] {
	set Gloss $CommandGlosses($kb($KeySeq));
	set GlossLength [string length $Gloss];
	if {$MaxGlossLength < $GlossLength} {set MaxGlossLength $GlossLength}
	set GlossToKeySeq($Gloss) $KeySeq;
    }
    #Now generate entries in alphabetical order of command glosses
    foreach Gloss [lsort [array names GlossToKeySeq]] {
	AppendToTextDisplay $po [format "%-*s\t%s\n" $MaxGlossLength $Gloss $GlossToKeySeq($Gloss)];
    }
}

proc ExplainMultigraphs {} {
    global HPWidth;
    global HPLines;
    global PopupList;
    if {[info exists PopupList(Multigraphs)]} {
	if {[winfo exists $PopupList(Multigraphs)]} {
	    destroy $PopupList(Multigraphs);
	    return;
	}
    }
    set po [CreateTextDisplay [_ "Multigraphs"] $HPWidth 10]
    BindKeys $po;
    set PopupList(Multigraphs) $po;
    AppendToTextDisplay $po [_ "By default every character is counted as a distinct segment, even if two or more characters represent a single sound. As a result, if we consider true minimal pairs to be those differing in exactly one sound segment, some true minimal pairs will be missed. In order to deal with this problem, you may instruct the program to treat specified sequences of characters as single segments. For example, in English you might define \'ch\' as a single segment since it usually represents a voiceless palatal affricate, not a sequence of two sounds. You may define multigraphs by reading them from a file or by entering them interactively. In either case, enter one multigraph per line. Any sequence of characters not containing whitespace will be treated as a multigraph.\n"]
    AppendToTextDisplay $po "\n"
    AppendToTextDisplay $po [_ "Note that you may also use this facility to deal with diacritics that are represented in your data by distinct Unicode codepoints. Some combinations of base character and diacritic can only be obtained by using a sequence of codepoints. Others can be obtained either as a sequence of codepoints or as a single codepoint. For example, the letter \'\u00E9\'  \'e with acute accent\' has a codepoint of its own (0x00E9), but may also be represented by the code for \'e\' followed by the code for the acute accent (0x0301). If your data uses the uncombined representation and you want words differing only in that one has plain \'e\' and the other has \'\u00E9\' to be treated as a minimal pair, you should define the sequence of the two codeponts as a multigraph."];
}

proc ExplainSingleColumnInput {} {
    global HPWidth;
    global HPLines;
    global PopupList;
    if {[info exists PopupList(SingleColumnInput)]} {
	if {[winfo exists $PopupList(SingleColumnInput)]} {
	    destroy $PopupList(SingleColumnInput);
	    return;
	}
    }
    set po [CreateTextDisplay [_ "Single- vs. Two-Column Input"] $HPWidth $HPLines]
    BindKeys $po;
    set PopupList(SingleColumnInput) $po;
    AppendToTextDisplay $po [_ "The input must consist of one entry per line. All entries must be either single-column or two-column. Single-column entires consist of a single word. Two-column entries consist of a word and an identifier, separated by a tab. The identifier is intended to identify the word. It may be a gloss or a record identifier. Minpair does not interpret the identifier - it merely includes it in the output along with the word."];
}

proc ExplainSorting {} {
    global PopupList;
    if {[info exists PopupList(Sorting)]} {
	if {[winfo exists $PopupList(Sorting)]} {
	    destroy $PopupList(Sorting);
	    return;
	}
    }
    set po [CreateTextDisplay [_ "Sorting of Output"] 76 12]
    BindKeys $po;
    set PopupList(Sorting) $po;
    AppendToTextDisplay $po [_ "Minimal pairs are written out by minpair\
in the order in which they are found, which is essentially random.\
It is usually more convenient if all pairs of words\
exemplifying the same minimal pair are grouped together. This may be accomplished\
by sorting minpair's output."];
    AppendToTextDisplay $po [_ "\n\n"]
    AppendToTextDisplay $po [_ "If requested, sorting is accomplished by means of\
an available sort utility program. If "];
    AppendLinkToTextDisplay $po "msort (http://www.billposer.org/Software/msort.htm)" {ShowWebPage http://www.billposer.org/Software/msort.html};
    AppendToTextDisplay $po [_ " is available, it will be used. The\
advantage of msort is that it understands multigraphs. If you have specified\
multigraphs for use in finding minimal pairs, they will also be used by msort.\
If msort is not found but the sort utility found on all Unix-type systems,\
or a comparable utility of the same name, is available, it will be used instead."]
}

proc ExplainIndels {} {
    global HPWidth;
    global HPLines;
    global PopupList;
    if {[info exists PopupList(Indels)]} {
	if {[winfo exists $PopupList(Indels)]} {
	    destroy $PopupList(Indels);
	    return;
	}
    }
    set po [CreateTextDisplay [_ "Insertions and Deletions"] $HPWidth $HPLines]
    BindKeys $po;
    set PopupList(Indels) $po;
    AppendToTextDisplay $po [_ "By default, only words of the same length are counted as minimal pairs. If this option is chosen, words that differ in length by one segment, that is, words such that one can be derived from the other by a single insertion or deletion, are also treated as minimal pairs. For example, \'spat\' and \'spats\' will be considered a minimal pair, as will be \'spat\' and \'pat\'."];
}

proc ExplainTranspositions {} {
    global HPWidth;
    global HPLines;
    global PopupList;
    if {[info exists PopupList(Trans)]} {
	if {[winfo exists $PopupList(Trans)]} {
	    destroy $PopupList(Trans);
	    return;
	}
    }
    set po [CreateTextDisplay [_ "Transpositions"] $HPWidth $HPLines]
    BindKeys $po;
    set PopupList(Trans) $po;
    AppendToTextDisplay $po [_ "By default two words are counted as a minimal pair only if they are of the same length and differ in only one segment. If this option is chosen, pairs of words that differ only in that two segments occur in the opposite order will also be counted as minimal pairs. For example, the words \'mast\' and \'mats\' will be treated as a minimal pair."];
}

proc ToggleIPAC {} {
    global IPACIsDisplayedP;
    global m;

    if { $IPACIsDisplayedP == 0} {
	if {[winfo exists .ipaec]} {
	    wm deiconify .ipaec;
	    raise .ipaec
	} else { 
	    ipaentry::PopupIPAEntryC;
	}
	$m.charentry entryconfigure $::ToggleIPACIndex -label [_ "Remove IPA Consonant Chart"];
	set IPACIsDisplayedP 1;
    } else {
	wm iconify .ipaec;
	set IPACIsDisplayedP 0;
	$m.charentry entryconfigure $::ToggleIPACIndex -label [_ "Display IPA Consonant Chart"];
    }
}

proc ToggleIPAA {} {
    global IPAAIsDisplayedP;
    global m;

    if { $IPAAIsDisplayedP == 0} {
	if {[winfo exists .ipaea]} {
	    wm deiconify .ipaea;
	    raise .ipaea
	} else { 
	    ipaentry::PopupIPAEntryA;
	}
	$m.charentry entryconfigure $::ToggleIPAAIndex -label [_ "Remove Accented Letter Chart"];
	set IPAAIsDisplayedP 1;
    } else {
	wm iconify .ipaea;
	set IPAAIsDisplayedP 0;
	$m.charentry entryconfigure $::ToggleIPAAIndex -label [_ "Display Accented Letter Chart"];
    }
}

proc ToggleIPAV {} {
    global IPAVIsDisplayedP;
    global m;

    if { $IPAVIsDisplayedP == 0} {
	if {[winfo exists .ipaev]} {
	    wm deiconify .ipaev;
	    raise .ipaev
	} else { 
	    ipaentry::PopupIPAEntryV;
	}
	set IPAVIsDisplayedP 1;
	$m.charentry entryconfigure $::ToggleIPAVIndex -label [_ "Remove IPA Vowel Chart"];
    } else {
	wm iconify .ipaev;
	set IPAVIsDisplayedP 0;
	$m.charentry entryconfigure $::ToggleIPAVIndex -label [_ "Display IPA Vowel Chart"];
    }
}

proc ToggleIPAD {} {
    global IPADIsDisplayedP;
    global m;

    if { $IPADIsDisplayedP == 0} {
	if {[winfo exists .ipaed]} {
	    wm deiconify .ipaed;
	    raise .ipaed
	} else { 
	    ipaentry::PopupIPAEntryD;
	}
	set IPADIsDisplayedP 1;
	$m.charentry entryconfigure $::ToggleIPADIndex -label [_ "Remove IPA Diacritic Chart"];
    } else {
	wm iconify .ipaed;
	set IPADIsDisplayedP 0;
	$m.charentry entryconfigure $::ToggleIPADIndex -label [_ "Display IPA Diacritic Chart"];
    }
}

proc ToggleCharEntryByCode {} {
    global CharEntryByCodeIsDisplayedP;
    global m;

    if { $CharEntryByCodeIsDisplayedP == 0} {
	if {[winfo exists .charent]} {
	    wm deiconify .charent;
	    raise .charent;
	} else { 
	    PopupCharEntryByCode;
	}
	set CharEntryByCodeIsDisplayedP 1;
	$m.charentry entryconfigure $::ToggleCharEntryByCodeIndex -label [_ "Remove Widget for Entering Characters by Unicode Code"];
    } else {
	wm iconify .charent;
	set CharEntryByCodeIsDisplayedP 0;
	$m.charentry entryconfigure $::ToggleCharEntryByCodeIndex -label [_ "Display Widget for Entering Characters by Unicode Code"];
    }
}

#Character entry code begins here
#Set this to the path to character entry menu
set CEM .menubar.charentry
#Set the default insertion target here.
set InsertionTarget .gen.if.ent

proc BindInsertionTarget {v} {
    bind $v <FocusIn> "+SetInsertionTargets $v"
}

set CharentryMenuItems 6;
set IPAAIsDisplayedP 0;
set IPACIsDisplayedP 0;
set IPAVIsDisplayedP 0;
set IPADIsDisplayedP 0;
set CharEntryByCodeIsDisplayedP 0;

proc SetInsertionTargets {tgt} {
    set ::InsertionTarget $tgt;
    set ::CharByCodeInsertionTarget $tgt;
}

set ProximateInsertionTarget "";
set InsertionTarget "";


namespace eval ipaentry {
    option add *Label.Font CharacterEntryLabelFont 100
    option add *Button.Font CharacterEntryFont 100
    variable IPAECColumnLabelList [list];
    variable IPAECRowLabelList [list];
    variable IPAEVColumnLabelList [list];
    variable IPAEVRowLabelList [list];
    variable DisplayConsonantChartColumnLabelsP 1
    variable DisplayConsonantChartRowLabelsP 1
    variable DisplayVowelChartColumnLabelsP 1
    variable DisplayVowelChartRowLabelsP 1

    set row 0;
    variable poa $row;
    incr row;
    variable vlstop $row;
    incr row;
    variable vdstop $row;
    incr row;
    variable vdimplosive $row;
    incr row;
    variable click $row;
    incr row;
    variable nasal  $row;
    incr row;
    variable trill $row;
    incr row;
    variable tap $row;
    incr row;
    variable vlfric $row;
    incr row;
    variable vdfric $row;
    incr row;
    variable vlaffric $row;
    incr row;
    variable vdaffric $row;
    incr row;
    variable approx $row;
    incr row; 
    variable latapprox $row;
    variable lastrow $row;

    set col 0;
    variable manner $col; 
    incr col;
    variable labial $col; 
    incr col;
    variable labiodental $col; 
    incr col;
    variable dental $col; 
    incr col;
    variable alveolar $col;
    incr col;
    variable postalveolar $col; 
    incr col;
    variable retroflex $col; 
    incr col;
    variable palatal $col; 
    incr col;
    variable velar $col; 
    incr col;
    variable uvular $col; 
    incr col;
    variable pharyngeal $col; 
    incr col;
    variable epiglottal $col; 
    incr col;
    variable glottal $col;
    variable lastcolumn $col;

    proc cfesh {} {
	variable vlfric
	variable postalveolar
	eval [list [format ".ipaec.r%dc%d" $vlfric $postalveolar] configure -text "\u0161"];
    }
    proc cresh {} {
	variable vlfric
	variable postalveolar
	eval [list [format ".ipaec.r%dc%d" $vlfric $postalveolar] configure -text "\u0283"];
    }

    proc cfeshr {} {
	variable vlfric
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vlfric $retroflex] configure -text "\u1E63"];
    }
    proc creshr {} {
	variable vlfric
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vlfric $retroflex] configure -text "\u0282"];
    }

    proc cryod {} {
	variable approx
	variable palatal
	eval [list [format ".ipaec.r%dc%d" $approx $palatal] configure -text "j"];
    }

    proc cfyod {} {
	variable approx
	variable palatal
	eval [list [format ".ipaec.r%dc%d" $approx $palatal] configure -text "y"];
    }

    proc crny {} {
	variable nasal
	variable palatal
	eval [list [format ".ipaec.r%dc%d" $nasal $palatal] configure -text "\u0272"];
    }
    proc cfny {} {
	variable nasal
	variable palatal
	eval [list [format ".ipaec.r%dc%d" $nasal $palatal] configure -text "\u00F1"];
    }

    proc crch {} {
	variable vlaffric
	variable postalveolar
	eval [list [format ".ipaec.r%dc%d" $vlaffric $postalveolar] configure -text "t\u0283"];
    }
    proc cfch {} {
	variable vlaffric
	variable postalveolar
	eval [list [format ".ipaec.r%dc%d" $vlaffric $postalveolar] configure -text "\u010D"];
    }

    proc crchr {} {
	variable vlaffric
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vlaffric $retroflex] configure -text "\u0288\u0282"];
    }

    proc cfchr {} {
	variable vlaffric
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vlaffric $retroflex] configure -text "\u1E6D\u1E63"];
    }


    proc crdj {} {
	eval [list [format ".ipaec.r%dc%d" $ipaentry::vdaffric $ipaentry::postalveolar] configure -text "d\u0292"];
    }

    proc cfdj {} {
	eval [list [format ".ipaec.r%dc%d" $ipaentry::vdaffric $ipaentry::postalveolar] configure -text "\u01C6"];
    }

    proc crdjr {} {
	eval [list [format ".ipaec.r%dc%d" $ipaentry::vdaffric $ipaentry::retroflex] configure -text "\u0256\u0290"];
    }

    proc cfdjr {} {
	eval [list [format ".ipaec.r%dc%d" $ipaentry::vdaffric $ipaentry::retroflex] configure -text "\u1E0D\u1E93"];
    }

    proc crzh {} {
	variable vdfric
	variable postalveolar
	eval [list [format ".ipaec.r%dc%d" $vdfric $postalveolar] configure -text "\u0292"];
    }
    proc cfzh {} {
	variable vdfric
	variable postalveolar
	eval [list [format ".ipaec.r%dc%d" $vdfric $postalveolar] configure -text "\u017E"];
    }

    proc crtr {} {
	variable vlstop
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vlstop $retroflex] configure -text "\u0288"];
    }
    proc cftr {} {
	variable vlstop
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vlstop $retroflex] configure -text "\u1E6D"];
    }

    proc crdr {} {
	variable vdstop
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vdstop $retroflex] configure -text "\u0256"];
    }
    proc cfdr {} {
	variable vdstop
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vdstop $retroflex] configure -text "\u1E0D"];
    }


    proc crnr {} {
	variable nasal
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $nasal $retroflex] configure -text "\u0273"];
    }
    proc cfnr {} {
	variable nasal
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $nasal $retroflex] configure -text "\u1E47"];
    }


    proc crzhr {} {
	variable vdfric
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vdfric $retroflex] configure -text "\u0290"];
    }
    proc cfzhr {} {
	variable vdfric
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $vdfric $retroflex] configure -text "\u1E93"];
    }

    proc crflapr {} {
	variable tap;
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $tap $retroflex] configure -text "\u027D"];
    }
    proc cfflapr {} {
	variable tap;
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $tap $retroflex] configure -text "\u1E5B"];
    }

    proc crlr {} {
	variable latapprox;
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $latapprox $retroflex] configure -text "\u026D"];
    }
    proc cflr {} {
	variable latapprox;
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $latapprox $retroflex] configure -text "\u1E37"];
    }

    proc crrr {} {
	variable approx;
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $approx $retroflex] configure -text "\u027B"];
    }
    proc cfrr {} {
	variable approx;
	variable retroflex
	eval [list [format ".ipaec.r%dc%d" $approx $retroflex] configure -text "\u1E5B"];
    }

    proc bhcham {w} {
	balloonhelp_for $w "The standard IPA character is shown by default. When the mouse\npointer is over the button, the \"North American\" variant is shown.\nRight click to insert the variant."
    }

    proc bhsp {w} {
	balloonhelp_for $w "The characters below and to the right are characters\nthat do not fit neatly into the chart."
    }

    proc UnpackConsonantColumnLabels {} {
	variable IPAECColumnLabelList;
	foreach l $IPAECColumnLabelList {
	    grid forget $l;
	}
    }

    proc UnpackConsonantRowLabels {} {
	variable IPAECRowLabelList;
	foreach l $IPAECRowLabelList {
	    grid forget $l;
	}
    }

    proc UnpackVowelColumnLabels {} {
	variable IPAEVColumnLabelList;
	foreach l $IPAEVColumnLabelList {
	    grid forget $l;
	}
    }

    proc UnpackVowelRowLabels {} {
	variable IPAEVRowLabelList;
	foreach l $IPAEVRowLabelList {
	    grid forget $l;
	}
    }

    proc SetDisplayConsonantChartColumnLabelsP {args} {
	variable DisplayConsonantChartColumnLabelsP
	if {[llength $args]} {
	    set DisplayConsonantChartColumnLabelsP [Boolean [lindex $args 0]];
	} else {
	    set DisplayConsonantChartColumnLabelsP;
	}
    }

    proc SetDisplayConsonantChartRowLabelsP {args} {
	variable DisplayConsonantChartRowLabelsP
	if {[llength $args]} {
	    set DisplayConsonantChartRowLabelsP [Boolean [lindex $args 0]] 
	} else {
	    set DisplayConsonantChartRowLabelsP;
	}
    }

    proc SetDisplayVowelChartColumnLabelsP {args} {
	variable DisplayVowelChartColumnLabelsP
	if {[llength $args]} {
	    set DisplayVowelChartColumnLabelsP [Boolean [lindex $args 0]];
	} else {
	    set DisplayVowelChartColumnLabelsP;
	}
    }

    proc SetDisplayVowelChartRowLabelsP {args} {
	variable DisplayVowelChartRowLabelsP
	if {[llength $args]} {
	    set DisplayVowelChartRowLabelsP [Boolean [lindex $args 0]];
	} else {
	    set DisplayVowelChartRowLabelsP;
	}
    }

    proc PopupIPAEntryC {} {
	variable poa
	variable vlstop 
	variable vdstop
	variable vdimplosive 
	variable click 
	variable nasal  
	variable trill 
	variable tap 
	variable vlfric 
	variable vdfric 
	variable vlaffric 
	variable vdaffric 
	variable approx 
	variable latapprox
	variable lastrow

	variable manner 
	variable labial 
	variable labiodental 
	variable dental 
	variable alveolar 
	variable postalveolar 
	variable retroflex 
	variable palatal 
	variable velar 
	variable uvular 
	variable pharyngeal 
	variable epiglottal 
	variable glottal
	variable lastcolumn

	variable IPAECColumnLabelList;
	variable IPAECRowLabelList;
	variable DisplayConsonantChartColumnLabelsP
	variable DisplayConsonantChartRowLabelsP

	set IPAECColumnLabelList [list];
	set IPAECRowLabelList [list];
	
	set xp 2;
	set yp 3;
	set spcolor 	\#c46276;
	toplevel .ipaec -borderwidth 3 -class CharEntry
	wm title .ipaec [_ "Consonant Symbols"]
	BindKeys .ipaec;
	set DownMsg [_ "Display IPA Consonant Chart"];
	set UpMsg   [_ "Remove IPA Consonant Chart"];
	bind .ipaec <Destroy> \
	    "set ::IPACIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPACIndex -label \"$DownMsg\""
	bind .ipaec <Unmap> \
	    "set ::IPACIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPACIndex -label \"$DownMsg\""
	bind .ipaec <Map> \
	    "set ::IPACIsDisplayedP 1;$::m.charentry entryconfigure $::ToggleIPACIndex -label \"$UpMsg\""
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $poa] \
	  -text [_ "IPA\nConsonants"]  -padx 5 -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $labial] \
	    -text [_ "labial"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $labiodental] \
	    -text [_ "labio\ndental"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $dental] \
	    -text [_ "dental"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $alveolar] \
	    -text [_ "alveolar"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $postalveolar] \
	    -text [_ "post\nalveolar"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $retroflex] \
	    -text [_ "retro\nflex"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $palatal] \
	    -text [_ "palatal"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $velar] \
	    -text [_ "velar"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $uvular] \
	    -text [_ "uvular"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $pharyngeal] \
	    -text [_ "pharyn\ngeal"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $epiglottal] \
	    -text [_ "epi\nglottal"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAECColumnLabelList [label [format ".ipaec.r%dc%d" $manner $glottal] \
	    -text [_ "glottal"]   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]

	#Voiceless oral stops
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $vlstop $poa] \
	       -text [_ "voiceless stops"] -anchor w  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list p]}
	button [format ".ipaec.r%dc%d"  $vlstop $labial] -text "p" \
	    -padx $xp -pady $yp -command  $cmd -fg $::ColorSpecs(UserTextEntry,Foreground); #debug

	label [format ".ipaec.r%dc%d" $vlstop $labiodental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list t\u032A]}
	button [format ".ipaec.r%dc%d"  $vlstop $dental] -text "t\u032A"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list t]}
	button [format ".ipaec.r%dc%d"  $vlstop $alveolar] -text "t"   -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $vlstop $postalveolar] -text " "   -padx $xp -pady $yp

	set cmd {$::InsertionTarget insert insert [list \u0288]}
	set bn [format ".ipaec.r%dc%d"  $vlstop $retroflex]
	button $bn  -text \u0288   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E6D]}
	bind $bn <Enter> {ipaentry::cftr}
	bind $bn <Leave> {ipaentry::crtr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list c]}
	button [format ".ipaec.r%dc%d"  $vlstop $palatal] -text "c"   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list k]}
	button [format ".ipaec.r%dc%d" $vlstop $velar] -text "k"   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list q]}
	button [format ".ipaec.r%dc%d" $vlstop $uvular] -text "q"   -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vlstop $pharyngeal] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $vlstop $epiglottal] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u0294]}
    button [format ".ipaec.r%dc%d" $vlstop $glottal] -text \u0294   -padx $xp -pady $yp -command $cmd


	#Voiced oral stops
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $vdstop $poa] \
	   -text [_ "voiced stops"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list b]}
	button [format ".ipaec.r%dc%d" $vdstop $labial] -text "b"   -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vdstop $labiodental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list d\u032A]}
	button [format ".ipaec.r%dc%d" $vdstop $dental] -text "d\u032A"   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list d]}
	button [format ".ipaec.r%dc%d" $vdstop $alveolar] -text "d"   -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $vdstop $postalveolar] -text " "   -padx $xp -pady $yp

	set cmd {$::InsertionTarget insert insert [list \u0256]}
	set bn [format ".ipaec.r%dc%d"  $vdstop $retroflex]
	button $bn  -text \u0256   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E0D]}
	bind $bn <Enter> {ipaentry::cfdr}
	bind $bn <Leave> {ipaentry::crdr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u025F]}
	button [format ".ipaec.r%dc%d" $vdstop $palatal] -text \u025F   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list g]}
	button [format ".ipaec.r%dc%d" $vdstop $velar] -text "g"   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0262]}
	button [format ".ipaec.r%dc%d" $vdstop $uvular] -text \u0262  -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vdstop $pharyngeal] -text " "  -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u02A1]}
	button [format ".ipaec.r%dc%d" $vdstop $epiglottal] -text \u02A1  -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vdstop $glottal] -text " "  -padx $xp -pady $yp

	#Nasals
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $nasal $poa] \
	       -text [_ "nasals"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list m]}
	button [format ".ipaec.r%dc%d" $nasal $labial] -text "m"      -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0271]}
	button [format ".ipaec.r%dc%d" $nasal $labiodental] -text \u0271   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list n\u032A]}
	button [format ".ipaec.r%dc%d" $nasal $dental] -text n\u032A   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list n]}
	button [format ".ipaec.r%dc%d" $nasal $alveolar] -text "n"      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $nasal $postalveolar] -text " "   -padx $xp -pady $yp

	set cmd {$::InsertionTarget insert insert [list \u0273]}
	set bn [format ".ipaec.r%dc%d"  $nasal $retroflex]
	button $bn  -text \u0273   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E47]}
	bind $bn <Enter> {ipaentry::cfnr}
	bind $bn <Leave> {ipaentry::crnr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u0272]}
	set bn [format ".ipaec.r%dc%d" $nasal $palatal]
	button $bn  -text \u0272    -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u00F1]}
	bind $bn <Enter> {ipaentry::cfny}
	bind $bn <Leave> {ipaentry::crny}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u014B]}
	button [format ".ipaec.r%dc%d" $nasal $velar] -text \u014B    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0274]}
	button [format ".ipaec.r%dc%d" $nasal $uvular] -text \u0274    -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $nasal $pharyngeal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $nasal $epiglottal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $nasal $glottal] -text " "    -padx $xp -pady $yp

	#Voiceless fricatives
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $vlfric $poa] \
	       -text [_ "voiceless fricatives"] -anchor w  -padx $xp -pady $yp \
	       -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list \u03C6]}
	button [format ".ipaec.r%dc%d" $vlfric $labial] -text \u03C6      -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list f]}
	button [format ".ipaec.r%dc%d" $vlfric $labiodental] -text "f"   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u03B8]}
	button [format ".ipaec.r%dc%d" $vlfric $dental] -text \u03B8   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "s"]}
	button [format ".ipaec.r%dc%d" $vlfric $alveolar] -text "s"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0283]}
	set bn [format ".ipaec.r%dc%d" $vlfric $postalveolar];
	button $bn -text \u0283   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u0161]}
	bind $bn <Enter> {ipaentry::cfesh}
	bind $bn <Leave> {ipaentry::cresh}
	bhcham $bn;
	set cmd {$::InsertionTarget insert insert [list \u0282]}
	set bn [format ".ipaec.r%dc%d" $vlfric $retroflex];
	button $bn -text \u0282   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E63]}
	bind $bn <Enter> {ipaentry::cfeshr}
	bind $bn <Leave> {ipaentry::creshr}
	bhcham $bn;
	set cmd {$::InsertionTarget insert insert [list \u0255]}
	button [format ".ipaec.r%dc%d" $vlfric $palatal] -text \u0255    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "x"]}
	button [format ".ipaec.r%dc%d" $vlfric $velar] -text "x"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "\u03C7"]}
	button [format ".ipaec.r%dc%d" $vlfric $uvular] -text "\u03C7"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0127]}
	button [format ".ipaec.r%dc%d" $vlfric $pharyngeal] -text \u0127    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u029C]}
	button [format ".ipaec.r%dc%d" $vlfric $epiglottal] -text \u029C   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "h"]}
	button [format ".ipaec.r%dc%d" $vlfric $glottal] -text "h"   -padx $xp -pady $yp -command $cmd

	#Voiced fricatives
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $vdfric $poa] \
	       -text [_ "voiced fricatives"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list \u03B2]}
	button [format ".ipaec.r%dc%d" $vdfric $labial] -text \u03B2      -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list v]}
	button [format ".ipaec.r%dc%d" $vdfric $labiodental] -text "v"   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "\u00F0"]}
	button [format ".ipaec.r%dc%d" $vdfric $dental] -text "\u00F0"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "z"]}
	button [format ".ipaec.r%dc%d" $vdfric $alveolar] -text "z"    -padx $xp -pady $yp -command $cmd

	set cmd {$::InsertionTarget insert insert [list \u0292]}
	set bn [format ".ipaec.r%dc%d" $vdfric $postalveolar]
	button $bn -text \u0292  -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u017E]}
	bind $bn <Enter> {ipaentry::cfzh}
	bind $bn <Leave> {ipaentry::crzh}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u0290]}
	set bn [format ".ipaec.r%dc%d" $vdfric $retroflex]
	button $bn -text \u0290   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E93]}
	bind $bn <Enter> {ipaentry::cfzhr}
	bind $bn <Leave> {ipaentry::crzhr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u029D]}
	button [format ".ipaec.r%dc%d" $vdfric $palatal] -text  \u029D  -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0263]}
	button [format ".ipaec.r%dc%d" $vdfric $velar] -text \u0263    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "\u0281"]}
	button [format ".ipaec.r%dc%d" $vdfric $uvular] -text "\u0281"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0295]}
	button [format ".ipaec.r%dc%d" $vdfric $pharyngeal] -text \u0295    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u02A2]}
	button [format ".ipaec.r%dc%d" $vdfric $epiglottal] -text \u02A2    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0266]}
	button [format ".ipaec.r%dc%d" $vdfric $glottal] -text \u0266    -padx $xp -pady $yp -command $cmd


	#trills
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $trill $poa] \
	       -text [_ "trills"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list \u0299]}
	button [format ".ipaec.r%dc%d" $trill $labial] -text \u0299      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $trill $labiodental] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $trill $dental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list r]}
	button [format ".ipaec.r%dc%d" $trill $alveolar] -text "r"      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $trill $postalveolar] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d"  $trill $retroflex] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $trill $palatal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $trill $velar] -text " "    -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u0280]}
	button [format ".ipaec.r%dc%d" $trill $uvular] -text \u0280    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $trill $pharyngeal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $trill $epiglottal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $trill $glottal] -text " "    -padx $xp -pady $yp

	#taps and flaps
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $tap $poa] \
	       -text [_ "taps/flaps"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	label [format ".ipaec.r%dc%d" $tap $labial] -text " "      -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $tap $labiodental] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $tap $dental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u027E]}
	button [format ".ipaec.r%dc%d" $tap $alveolar] -text \u027E      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $tap $postalveolar] -text " "   -padx $xp -pady $yp

	set cmd {$::InsertionTarget insert insert [list \u027D]}
	set bn [format ".ipaec.r%dc%d"  $tap $retroflex];
	button $bn -text \u027D   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E5B]}
	bind $bn <Enter> {ipaentry::cfflapr}
	bind $bn <Leave> {ipaentry::crflapr}
	bhcham $bn;

	label [format ".ipaec.r%dc%d" $tap $palatal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $tap $velar] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $tap $uvular] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $tap $pharyngeal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $tap $epiglottal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $tap $glottal] -text " "    -padx $xp -pady $yp

	#approximants
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $approx $poa] \
	       -text [_ "approximants"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	label [format ".ipaec.r%dc%d" $approx $labial] -text " "      -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u028B]}
	button [format ".ipaec.r%dc%d" $approx $labiodental] -text \u028B   -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $approx $dental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u0279]}
	button [format ".ipaec.r%dc%d" $approx $alveolar] -text  \u0279     -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $approx $postalveolar] -text " "   -padx $xp -pady $yp

	set cmd {$::InsertionTarget insert insert [list \u027B]}
	set bn [format ".ipaec.r%dc%d"  $approx $retroflex]
	button $bn -text \u027B   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E5B]}
	bind $bn <Enter> {ipaentry::cfrr}
	bind $bn <Leave> {ipaentry::crrr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list "j"]}
	set bn [format ".ipaec.r%dc%d" $approx $palatal]
	button $bn -text "j"    -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list "y"]}
	bind $bn <Enter> {ipaentry::cfyod}
	bind $bn <Leave> {ipaentry::cryod}
   bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u0270]}
	button [format ".ipaec.r%dc%d" $approx $velar] -text \u0270    -padx $xp -pady $yp -command $cmd

	label [format ".ipaec.r%dc%d" $approx $uvular] -text " "    -padx $xp -pady $yp -bg $spcolor
	label [format ".ipaec.r%dc%d" $approx $pharyngeal] -text " "    -padx $xp -pady $yp -bg $spcolor

#	label [format ".ipaec.r%dc%d" $approx $epiglottal] -text " "    -padx $xp -pady $yp -bg $spcolor
#	label [format ".ipaec.r%dc%d" $approx $glottal] -text " "    -padx $xp -pady $yp -bg $spcolor

	#lateral approximants
				   lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $latapprox $poa] -text [_ "lateral approximants"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	label [format ".ipaec.r%dc%d" $latapprox $labial] -text " "      -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $latapprox $labiodental] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $latapprox $dental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list "l"]}
	button [format ".ipaec.r%dc%d" $latapprox $alveolar] -text "l"    -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $latapprox $postalveolar] -text " "   -padx $xp -pady $yp

	set cmd {$::InsertionTarget insert insert [list \u026D]}
	set bn [format ".ipaec.r%dc%d"  $latapprox $retroflex]
	button $bn  -text \u026D   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E37]}
	bind $bn <Enter> {ipaentry::cflr}
	bind $bn <Leave> {ipaentry::crlr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u028E]}
	button [format ".ipaec.r%dc%d" $latapprox $palatal] -text \u028E    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u029F]}
	button [format ".ipaec.r%dc%d" $latapprox $velar] -text \u029F    -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $latapprox $uvular] -text " "    -padx $xp -pady $yp -bg $spcolor
#	label [format ".ipaec.r%dc%d" $latapprox $pharyngeal] -text " "    -padx $xp -pady $yp -bg $spcolor
#	label [format ".ipaec.r%dc%d" $latapprox $epiglottal] -text " "    -padx $xp -pady $yp
#	label [format ".ipaec.r%dc%d" $latapprox $glottal] -text " "    -padx $xp -pady $yp

	#voiced implosives
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $vdimplosive $poa] \
	       -text [_ "voiced implosives"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list \u0253]}
	button [format ".ipaec.r%dc%d" $vdimplosive $labial] -text \u0253      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vdimplosive $labiodental] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $vdimplosive $dental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u0257]}
	button [format ".ipaec.r%dc%d" $vdimplosive $alveolar] -text \u0257   -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d"  $vdimplosive $postalveolar] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list ]}
	button [format ".ipaec.r%dc%d"  $vdimplosive $retroflex] -text ""   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0284]}
	button [format ".ipaec.r%dc%d" $vdimplosive $palatal] -text \u0284  -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u0260]}
	button [format ".ipaec.r%dc%d" $vdimplosive $velar] -text \u0260    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u029B]}
	button [format ".ipaec.r%dc%d" $vdimplosive $uvular] -text \u029B    -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vdimplosive $pharyngeal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $vdimplosive $epiglottal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $vdimplosive $glottal] -text " "    -padx $xp -pady $yp

	#clicks
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $click $poa] \
	       -text [_ "clicks"] -anchor w  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list \u0298]}
	button [format ".ipaec.r%dc%d" $click $labial] -text \u0298      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $click $labiodental] -text " "   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u01C0]}
	button [format ".ipaec.r%dc%d" $click $dental] -text \u01C0   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u01C2]}
	button [format ".ipaec.r%dc%d" $click $alveolar] -text \u01C2   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list \u01C3]}
	button [format ".ipaec.r%dc%d"  $click $postalveolar] -text \u01C3   -padx $xp -pady $yp  -command $cmd
	label [format ".ipaec.r%dc%d"  $click $retroflex] -text " "   -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $click $palatal] -text " "  -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $click $velar] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $click $uvular] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $click $pharyngeal] -text " "    -padx $xp -pady $yp
	label [format ".ipaec.r%dc%d" $click $epiglottal] -text " "    -padx $xp -pady $yp
	set cmd {BackDelete $::InsertionTarget}
	set bn [format ".ipaec.r%dc%d" $click $glottal];
	button $bn -text [_ "Delete"]  -padx $xp -pady $yp -command $cmd -background "\#E0D0FF";
	set DeleteButton $bn;

	#Voiceless affricates
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $vlaffric $poa] \
	       -text [_ "voiceless affricates"] -anchor w -padx $xp -pady $yp \
	       -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list p\u03C6]}
	button [format ".ipaec.r%dc%d" $vlaffric $labial] -text "p\u03C6"      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vlaffric $labiodental] -text ""   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list t\u03B8]}
	button [format ".ipaec.r%dc%d" $vlaffric $dental] -text "t\u03B8"   -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "ts"]}
	button [format ".ipaec.r%dc%d" $vlaffric $alveolar] -text "ts"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list t\u0283]}
	set bn [format ".ipaec.r%dc%d"  $vlaffric $postalveolar]
	button $bn -text "t\u0283"   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u010D]}
	bind $bn <Enter> {ipaentry::cfch}
	bind $bn <Leave> {ipaentry::crch}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u0288\u0282]}
	set bn [format ".ipaec.r%dc%d"  $vlaffric $retroflex]
	button $bn -text \u0288\u0282   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E6D\u1E63]}
	bind $bn <Enter> {ipaentry::cfchr}
	bind $bn <Leave> {ipaentry::crchr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list t\u0255]}
	button [format ".ipaec.r%dc%d" $vlaffric $palatal] -text t\u0255    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "kx"]}
	button [format ".ipaec.r%dc%d" $vlaffric $velar] -text "kx"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "q\u03C7"]}
	button [format ".ipaec.r%dc%d" $vlaffric $uvular] -text "q\u03C7"    -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vlaffric $pharyngeal] -text ""    -padx $xp -pady $yp -bg $spcolor
	label [format ".ipaec.r%dc%d" $vlaffric $epiglottal] -text ""   -padx $xp -pady $yp -bg $spcolor
	label [format ".ipaec.r%dc%d" $vlaffric $glottal] -text ""   -padx $xp -pady $yp -bg $spcolor

	#Voiced affricate
	lappend IPAECRowLabelList [label [format ".ipaec.r%dc%d" $vdaffric $poa] \
	       -text [_ "voiced affricates"] -anchor w   -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list b\u03B2]}
	button [format ".ipaec.r%dc%d" $vdaffric $labial] -text b\u03B2      -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vdaffric $labiodental] -text ""   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list "d\u00F0"]}
	button [format ".ipaec.r%dc%d" $vdaffric $dental] -text "d\u00F0"    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "dz"]}
	button [format ".ipaec.r%dc%d" $vdaffric $alveolar] -text "dz"    -padx $xp -pady $yp -command $cmd

	set cmd {$::InsertionTarget insert insert [list d\u0292]}
	set bn [format ".ipaec.r%dc%d" $vdaffric $postalveolar]
	button $bn -text d\u0292  -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u01C6]}
	bind $bn <Enter> {ipaentry::cfdj}
	bind $bn <Leave> {ipaentry::crdj}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u0256\u0290]}
	set bn [format ".ipaec.r%dc%d" $vdaffric $retroflex]
	button $bn -text \u0256\u0290   -padx $xp -pady $yp -command $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u1E0D\u1E93]}
	bind $bn <Enter> {ipaentry::cfdjr}
	bind $bn <Leave> {ipaentry::crdjr}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list d\u029D]}
	button [format ".ipaec.r%dc%d" $vdaffric $palatal] -text  d\u029D  -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list g\u0263]}
	button [format ".ipaec.r%dc%d" $vdaffric $velar] -text g\u0263    -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "\u0262\u0281"]}
	button [format ".ipaec.r%dc%d" $vdaffric $uvular] -text "\u0262\u0281"    -padx $xp -pady $yp -command $cmd
	label [format ".ipaec.r%dc%d" $vdaffric $pharyngeal] -text ""    -padx $xp -pady $yp -bg $spcolor
#	label [format ".ipaec.r%dc%d" $vdaffric $epiglottal] -text ""    -padx $xp -pady $yp -bg $spcolor
#	label [format ".ipaec.r%dc%d" $vdaffric $glottal] -text ""    -padx $xp -pady $yp -bg $spcolor

	set cmd {$::InsertionTarget insert insert [list "\u028D"]}
	button .ipaec.r13c10 -text \u028D -padx $xp -pady $yp -command $cmd
	balloonhelpd_for .ipaec.r13c10  [_ "Voiceless labio-velar fricative"]
	set cmd {$::InsertionTarget insert insert [list "\u0265"]}
	button .ipaec.r11c11 -text \u0265 -padx $xp -pady $yp -command $cmd
	balloonhelpd_for .ipaec.r11c11  [_ "Voiced labio-palatal approximant"]
	set cmd {$::InsertionTarget insert insert [list "\u0077"]}
	button .ipaec.r11c12 -text \u0077 -padx $xp -pady $yp -command $cmd
	balloonhelpd_for .ipaec.r11c12  [_ "Voiced labio-velar approximant"]
	set cmd {$::InsertionTarget insert insert [list "\u01C1"]}
	button .ipaec.r12c11 -text \u01C1 -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "\u02A0"]}
	button .ipaec.r12c12 -text \u02A0 -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "\u026C"]}
	button .ipaec.r13c11 -text \u026C -padx $xp -pady $yp -command $cmd
	set cmd {$::InsertionTarget insert insert [list "\u026E"]}
	button .ipaec.r13c12 -text \u026E -padx $xp -pady $yp -command $cmd

	for {set row 0} {$row <= $lastrow} {incr row} {
	    set line [list];
	    for {set col 0} {$col <= $lastcolumn} {incr col} {
		set cell [format ".ipaec.r%dc%d" $row $col]
		lappend line $cell
		if {[string equal [winfo class $cell] "Label"]} {
		    balloonhelp_for $cell  [_ "Left click to insert standard characters.\nRight click to insert alternative characters."]
		}
	    }
	    eval grid $line -sticky news;
	}
	balloonhelpd_for .ipaec.r12c11  [_ "Lateral click"]
	balloonhelpd_for .ipaec.r12c12  [_ "Voiceless uvular implosive"]
	balloonhelpd_for .ipaec.r13c11  [_ "Voiceless lateral fricative"]
	balloonhelpd_for .ipaec.r13c12  [_ "Voiced lateral fricative"]
	bhsp [format ".ipaec.r%dc%d" $latapprox $uvular]
	bhsp [format ".ipaec.r%dc%d" $approx $uvular]
	bhsp [format ".ipaec.r%dc%d" $approx $pharyngeal]
	bhsp [format ".ipaec.r%dc%d" $vlaffric $pharyngeal]
	bhsp [format ".ipaec.r%dc%d" $vlaffric $epiglottal]
	bhsp [format ".ipaec.r%dc%d" $vlaffric $glottal]
	bhsp [format ".ipaec.r%dc%d" $vdaffric $pharyngeal]
	balloonhelpd_for $DeleteButton [_ "No, there isn't a glottal click  called \"Delete\".\nThis is so that you can correct mistakes while using\nthe mouse, without having to go back to the keyboard."]
	after idle {
	    update idletasks
	    raise .ipaec;
	}
	if {!$DisplayConsonantChartColumnLabelsP} {UnpackConsonantColumnLabels}
	if {!$DisplayConsonantChartRowLabelsP} {UnpackConsonantRowLabels}
    }

    set row 0;
    variable frontness $row;
    incr row;
    variable close $row;
    incr row;
    variable closeclosemid $row;
    incr row;
    variable closemid $row;
    incr row;
    variable midmid $row;
    incr row;
    variable openmid $row;
    incr row;
    variable openopenmid $row;
    incr row;
    variable open $row;
    variable lastvrow $row;

    set col 0;
    variable height $col; 
    incr col;
    variable fronturd $col;
    incr col;
    variable frontrnd $col;
    incr col;
    variable centurd $col;
    incr col;
    variable centrnd $col;
    incr col;
    variable backurd $col;
    incr col;
    variable backrnd $col;
    variable lastvcolumn $col;

    proc vry {} {
	variable close
	variable frontrnd
	eval [list [format ".ipaev.r%dc%d" $close $frontrnd] configure -text "y"];
	}
    
    proc vfy {} {
	variable close
	variable frontrnd
	eval [list [format ".ipaev.r%dc%d" $close $frontrnd] configure -text "\u00FC"];
	}

    proc vrphi {} {
	variable frontrnd;
	variable closemid
	eval [list [format ".ipaev.r%dc%d" $closemid $frontrnd] configure -text "\u00F8"];
    }
    
    proc vfphi {} {
	variable frontrnd;
	variable closemid
	eval [list [format ".ipaev.r%dc%d" $closemid $frontrnd] configure -text "o\u0308"];
    }

    proc vroe {} {
	variable frontrnd;
	variable openmid
	eval [list [format ".ipaev.r%dc%d" $openmid $frontrnd] configure -text "\u0153"];
    }

    proc vfoe {} {
	variable frontrnd;
	variable openmid
	eval [list [format ".ipaev.r%dc%d" $openmid $frontrnd] configure -text "\u0254\0308"];
    }


    proc PopupIPAEntryV {} {
	variable frontness
	variable close
	variable closeclosemid
	variable closemid
	variable openmid
	variable midmid
	variable openopenmid
	variable open
	variable lastvrow

	variable height
	variable fronturd
	variable frontrnd
	variable centurd
	variable centrnd
	variable backurd
	variable backrnd
	variable lastvcolumn

	variable IPAEVColumnLabelList
	variable IPAEVRowLabelList
	variable DisplayVowelChartColumnLabelsP
	variable DisplayVowelChartRowLabelsP

	set xp 2;
	set yp 3;
	toplevel .ipaev -borderwidth 3
	wm title .ipaev [_ "Vowel Symbols"]
	BindKeys .ipaev;
	set DownMsg [_ "Display IPA Vowel Chart"];
	set UpMsg   [_ "Remove IPA Vowel Chart"];
	bind .ipaev <Destroy> \
	    "set ::IPAVIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPAVIndex -label \"$DownMsg\""
	bind .ipaev <Unmap> \
	    "set ::IPAVIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPAVIndex -label \"$DownMsg\""
	bind .ipaev <Map> \
	    "set ::IPAVIsDisplayedP 1;$::m.charentry entryconfigure $::ToggleIPAVIndex -label \"$UpMsg\""

	#Column headers
	lappend IPAEVColumnLabelList [label [format ".ipaev.r%dc%d" $height $frontness] \
	     -text [_ "IPA\nVowels"]  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAEVColumnLabelList [label [format ".ipaev.r%dc%d" $height $fronturd] \
	     -text [_ "Front\nUnrounded"]  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAEVColumnLabelList [label [format ".ipaev.r%dc%d" $height $frontrnd] \
	     -text [_ "Front\nRounded"]  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAEVColumnLabelList [label [format ".ipaev.r%dc%d" $height $centurd] \
	     -text [_ "Central\nUnrounded"]  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAEVColumnLabelList [label [format ".ipaev.r%dc%d" $height $centrnd] \
	     -text [_ "Central\nRounded"]  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAEVColumnLabelList [label [format ".ipaev.r%dc%d" $height $backurd] \
	     -text [_ "Back\nUnrounded"]  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]
	lappend IPAEVColumnLabelList [label [format ".ipaev.r%dc%d" $height $backrnd] \
	     -text [_ "Back\nRounded"]  -padx $xp -pady $yp -bg $::ColorSpecs(IPAHeadings,Background)]


	#Close vowels
	lappend IPAEVRowLabelList [label [format ".ipaev.r%dc%d" $close $frontness] \
	   -text [_ "Close"] -anchor w  -padx $xp -pady $yp \
  	   -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list i]}
	button [format ".ipaev.r%dc%d"  $close $fronturd] \
	     -text "i"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list y]}
	set bn  [format ".ipaev.r%dc%d"  $close $frontrnd]
	button $bn -text "y"   -padx $xp -pady $yp -command  $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u00FC]}
	bind $bn <Enter> {ipaentry::vfy}
	bind $bn <Leave> {ipaentry::vry}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u0268]}
	button [format ".ipaev.r%dc%d"  $close $centurd] \
	     -text "\u0268"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u0289]}
	button [format ".ipaev.r%dc%d"  $close $centrnd] \
	     -text "\u0289"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u0268]}
	button [format ".ipaev.r%dc%d"  $close $backurd] \
	     -text "\u0268"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list u]}
	button [format ".ipaev.r%dc%d"  $close $backrnd] \
	     -text "u"   -padx $xp -pady $yp -command  $cmd

	# Close close mid vowels
	lappend IPAEVRowLabelList [label [format ".ipaev.r%dc%d" $closeclosemid $frontness] \
	     -text [_ "Close Close Mid"] -anchor w  -padx $xp -pady $yp \
	  -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list I]}
	button [format ".ipaev.r%dc%d"  $closeclosemid $fronturd] \
	     -text "I"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list Y]}
	button [format ".ipaev.r%dc%d"  $closeclosemid $frontrnd] \
	     -text "Y"   -padx $xp -pady $yp -command  $cmd
	label [format ".ipaev.r%dc%d"  $closeclosemid $centurd] \
	     -text ""   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u028A]}
	button [format ".ipaev.r%dc%d"  $closeclosemid $centrnd] \
	     -text "\u028A"   -padx $xp -pady $yp -command  $cmd
	label [format ".ipaev.r%dc%d"  $closeclosemid $backurd] \
	     -text ""   -padx $xp -pady $yp
	label [format ".ipaev.r%dc%d"  $closeclosemid $backrnd] \
	     -text ""   -padx $xp -pady $yp

	# Close mid vowels
	lappend IPAEVRowLabelList [label [format ".ipaev.r%dc%d" $closemid $frontness] \
	     -text [_ "Close Mid"] -anchor w  -padx $xp -pady $yp \
	  -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list e]}
	button [format ".ipaev.r%dc%d"  $closemid $fronturd] \
	     -text "e"   -padx $xp -pady $yp -command  $cmd

	set cmd {$::InsertionTarget insert insert [list \u00F8]}
	set bn [format ".ipaev.r%dc%d"  $closemid $frontrnd]
	button $bn -text "\u00F8"   -padx $xp -pady $yp -command  $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list o\u0308]}
	bind $bn <Enter> {ipaentry::vfphi}
	bind $bn <Leave> {ipaentry::vrphi}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u0258]}
	button [format ".ipaev.r%dc%d"  $closemid $centurd] \
	     -text "\u0258"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u0275]}
	button [format ".ipaev.r%dc%d"  $closemid $centrnd] \
	     -text "\u0275"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u0264]}
	button [format ".ipaev.r%dc%d"  $closemid $backurd] \
	     -text "\u0264"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list o]}
	button [format ".ipaev.r%dc%d"  $closemid $backrnd] \
	     -text "o"   -padx $xp -pady $yp -command  $cmd

	# Mid mid vowels
	lappend IPAEVRowLabelList [label [format ".ipaev.r%dc%d" $midmid $frontness] \
	     -text [_ "Mid Mid"] -anchor w  -padx $xp -pady $yp \
	  -bg $::ColorSpecs(IPAHeadings,Background)]
	label [format ".ipaev.r%dc%d"  $midmid $fronturd] \
	     -text ""   -padx $xp -pady $yp
	label [format ".ipaev.r%dc%d"  $midmid $frontrnd] \
	     -text ""   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u0259]}
	button [format ".ipaev.r%dc%d"  $midmid $centurd] \
	     -text "\u0259"   -padx $xp -pady $yp -command  $cmd
	set cmd {BackDelete $::InsertionTarget}
	set bn [format ".ipaev.r%dc%d" $midmid $centrnd];
	button $bn -text [_ "Delete"]  \
	     -padx $xp -pady $yp -command $cmd -background "\#E0D0FF";
	set DeleteButton $bn;
	label [format ".ipaev.r%dc%d"  $midmid $backurd] -text ""   -padx $xp -pady $yp
	label [format ".ipaev.r%dc%d"  $midmid $backrnd] -text ""   -padx $xp -pady $yp

	# Open mid vowels
	lappend IPAEVRowLabelList [label [format ".ipaev.r%dc%d" $openmid $frontness] \
	     -text [_ "Open Mid"] -anchor w  -padx $xp -pady $yp \
  	     -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list \u025B]}
	button [format ".ipaev.r%dc%d"  $openmid $fronturd] \
	     -text \u025B   -padx $xp -pady $yp -command  $cmd

	set cmd {$::InsertionTarget insert insert [list \u0153]}
	set bn [format ".ipaev.r%dc%d"  $openmid $frontrnd]
	button $bn -text "\u0153"   -padx $xp -pady $yp -command  $cmd
	bind $bn <<B3>> {$::InsertionTarget insert insert [list \u0254\u0308]}
	bind $bn <Enter> {ipaentry::vfoe}
	bind $bn <Leave> {ipaentry::vroe}
	bhcham $bn;

	set cmd {$::InsertionTarget insert insert [list \u025C]}
	button [format ".ipaev.r%dc%d"  $openmid $centurd] \
	     -text "\u025C"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u025E]}
	button [format ".ipaev.r%dc%d"  $openmid $centrnd] \
	     -text "\u025E"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u028C]}
	button [format ".ipaev.r%dc%d"  $openmid $backurd] \
	     -text "\u028C"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u0254]}
	button [format ".ipaev.r%dc%d"  $openmid $backrnd] \
	     -text "\u0254"   -padx $xp -pady $yp -command  $cmd

	# Open open mid vowels
	lappend IPAEVRowLabelList [label [format ".ipaev.r%dc%d" $openopenmid $frontness] \
	     -text [_ "Open Open Mid"] -anchor w  -padx $xp -pady $yp \
  	     -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list \u00E6]}
	button [format ".ipaev.r%dc%d"  $openopenmid $fronturd] \
	     -text \u00E6   -padx $xp -pady $yp -command  $cmd
	label [format ".ipaev.r%dc%d"  $openopenmid $frontrnd] -text ""   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u0250]}
	button [format ".ipaev.r%dc%d"  $openopenmid $centurd] \
	     -text "\u0250"   -padx $xp -pady $yp -command  $cmd
	label [format ".ipaev.r%dc%d"  $openopenmid $centrnd] \
		     -text ""   -padx $xp -pady $yp
	label [format ".ipaev.r%dc%d"  $openopenmid $backurd] \
	     -text ""   -padx $xp -pady $yp
	label [format ".ipaev.r%dc%d"  $openopenmid $backrnd] \
	     -text ""   -padx $xp -pady $yp

	# Open  vowels
	lappend IPAEVRowLabelList [label [format ".ipaev.r%dc%d" $open $frontness] \
	   -text [_ "Open"] -anchor w  -padx $xp -pady $yp \
	   -bg $::ColorSpecs(IPAHeadings,Background)]
	set cmd {$::InsertionTarget insert insert [list a]}
	button [format ".ipaev.r%dc%d"  $open $fronturd] \
	     -text "a"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u0276]}
	button [format ".ipaev.r%dc%d"  $open $frontrnd] \
	     -text "\u0276"   -padx $xp -pady $yp -command  $cmd
	label [format ".ipaev.r%dc%d"  $open $centurd] \
	     -text ""   -padx $xp -pady $yp
	label [format ".ipaev.r%dc%d"  $open $centrnd] \
	     -text ""   -padx $xp -pady $yp
	set cmd {$::InsertionTarget insert insert [list \u0251]}
	button [format ".ipaev.r%dc%d"  $open $backurd] \
	     -text "\u0251"   -padx $xp -pady $yp -command  $cmd
	set cmd {$::InsertionTarget insert insert [list \u0252]}
	button [format ".ipaev.r%dc%d"  $open  $backrnd] \
	     -text "\u0252"   -padx $xp -pady $yp -command  $cmd

	for {set row 0} {$row <= $lastvrow} {incr row} {
	    set line [list];
	    for {set col 0} {$col <= $lastvcolumn} {incr col} {
		set cell [format ".ipaev.r%dc%d" $row $col]
		lappend line $cell
		balloonhelp_for $cell  \
		    [_ "Left click to insert standard characters.\n\
Right click to insert alternative characters."]
	    }
	    eval grid $line -sticky news;
	}
	balloonhelpd_for $DeleteButton [_ "No, there isn't a mid-mid central rounded vowel called \"Delete\".\nThis is so that you can correct mistakes while using\nthe mouse, without having to go back to the keyboard."]
	if {!$DisplayVowelChartColumnLabelsP} {UnpackVowelColumnLabels}
	if {!$DisplayVowelChartRowLabelsP} {UnpackVowelRowLabels}
    }

#Rearrange into functional groups, e.g. tone marks, vowel quality marks,
#etc. Separate IPA and non-IPA.

    proc PopupIPAEntryD {} {
	set PerRow 10;
	set BaseChar "o";
	set xp 2;
	set yp 3;
	toplevel .ipaed -borderwidth 3
	wm title .ipaed [_ "Phonetic Diacritics"]
	BindKeys .ipaed
	set DownMsg [_ "Display IPA Diacritic Chart"];
	set UpMsg   [_ "Remove IPA Diacritic Chart"];
	bind .ipaed <Destroy> \
	    "set ::IPADIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPADIndex -label \"$DownMsg\""
	bind .ipaed <Unmap> \
	    "set ::IPADIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPADIndex -label \"$DownMsg\""
	bind .ipaed <Map> \
	    "set ::IPADIsDisplayedP 1;$::m.charentry entryconfigure $::ToggleIPADIndex -label \"$UpMsg\""
	array set Diacritics [list\
				  0  {"\u02D0" "long"}\
				  1  {"\u02D1" "half long"}\
				  2  {"\u0303" "nasalized"}\
				  3  {"\u0324" "breathy voice"}\
				  4  {"\u0325" "voiceless"}\
				  5  {"\u032C" "voiced"}\
				  6  {"\u031A" "unreleased"}\
				  7  {"\u031C" "lower/open variety of vowel"}\
				  8  {"\u031D" "raised/closed variety of vowel"}\
				  9  {"\u031E" "lower/open variety of vowel"}\
				  10  {"\u031F" "advanced/fronted"}\
				  11  {"\u0320" "retracted/backed"}\
				  12  {"\u0321" "palatalized"}\
				  13  {"\u0322" "retroflex"}\
				  14  {"\u0323" "closer variety of vowel/retroflex"}\
				  15  {"\u0329" "syllabic"}\
				  16  {"\u032A" "dental"}\
				  17  {"\u032B" "labialized"}\
				  18  {"\u0330" "creaky voice"}\
				  19  {"\u0334" "velarized/pharyngealized"}\
				  20  {"\u0346" "dentolabial"}\
				  21  {"\u0347" "alveolar"}\
				  22  {"\u0348" "strong articulation"}\
				  23  {"\u0349" "weak articulation"}\
				  24  {"\u02CA" "high tone"}\
				  25  {"\u02CB" "low tone"}\
				  26  {"\u0302" "falling tone"}\
				  27  {"\u0304" "long"}\
				  28  {"\u0306" "short"}\
				  29  {"\u0308" "non-canonical backness"}\
				  30  {"\u02CC" "secondary stress"}\
				  31  {"\u02C8" "primary stress/downstep"}\
				  32  {"\u02B8" "palatalized"}\
				  33  {"\u02DE" "rhotacized"}\
				  34  {"\u0328" "nasalized"}\
				  35  {"\u034A" "denasal"}\
				  36  {"\u034B" "nasal escape"}\
				  37  {"\u034C" "velopharyngeal friction"}\
				  38  {"\u034D" "labial spreading"}\
				  39  {"\u034E" "whistled"}
				 ];

	set DiacriticCnt [llength [array names Diacritics]];
	set Rows [expr ceil(double($DiacriticCnt)/double($PerRow))]
	set Total [expr $PerRow * $Rows]
	#Generate buttons with blank labels for empty padding slots
	for {set k 0} {$k < $Total} {incr k} {
	    set row [expr $k/$PerRow]
	    set col [expr $k%$PerRow]
	    if {[info exist Diacritics($k)]} {
		button .ipaed.r${row}c${col} -text "$BaseChar[lindex $Diacritics($k) 0]"\
		    -padx $xp -pady $yp\
		    -command "\$::InsertionTarget insert insert  [lindex $Diacritics($k) 0]"
		balloonhelpd_for .ipaed.r${row}c${col} [_ [lindex $Diacritics($k) 1]]  
	    } else {
		label .ipaed.r${row}c${col} -text "";
	    }
	    set LastRow $row;
	    set LastCol $col;
	}
	#Lay the buttons and labels out in a grid.
	for {set row 0} {$row <= $LastRow} {incr row} {
	    set line [list];
	    for {set col 0} {$col <= $LastCol} {incr col} {
		set cell [format ".ipaed.r%dc%d" $row $col]
		lappend line $cell
	    }
	    eval grid $line -sticky news;
	}
    }

#Accented letters
    proc PopupIPAEntryA {} {
	set PerRow 12;
	set xp 3;
	set yp 3;
	toplevel .ipaea -borderwidth 4
	wm title .ipaea [_ "Accented Letters"]
	BindKeys .ipaea
	set DownMsg [_ "Display Accented Letter Chart"]
	set UpMsg   [_ "Display Accented Letter Chart"]
	bind .ipaea <Destroy> \
	    "set ::IPAAIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPAAIndex -label \"$DownMsg\""
	bind .ipaea <Unmap> \
	    "set ::IPAAIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPAAIndex -label \"$DownMsg\""
	bind .ipaea <Map> \
	    "set ::IPAAIsDisplayedP 0;$::m.charentry entryconfigure $::ToggleIPAAIndex -label \"$UpMsg\""
	set AccentedLetters [list \
				 {"\u00E0" "a with grave"}\
				 {"\u00E1" "a with acute"}\
				 {"\u00E2" "a with circumflex"}\
				 {"\u00E3" "a with tilde"}\
				 {"\u00E4" "a with diaresis"}\
				 {"\u00E5" "a with ring above"}\
				 {"\u0101" "a with macron"}\
				 {"\u0103" "a with breve"}\
				 {"\u0105" "a with ogonek"}\
				 {"\u01CE" "a with caron"}\
				 {"\u0201" "a with double grave"}\
				 {"\u0203" "a with inverted breve"}\
				 {"\u01DF" "a with diaresis and macron"}\
				 {"\u01E1" "a with dot above and macron"}\
				 {"\u01FB" "a with ring above"}\
				 {"\u0227" "a with dot above"}\
				 {"\u1EA1" "a with dot below"}\
				 {"\u01E3" "ae with macron"}\
				 {"\u01FD" "ash with acute"}\
				 {"\u0180" "b with stroke"}\
				 {"\u0183" "b with topbar"}\
				 {"\u1E03" "b with dot above"}\
				 {"\u1E05" "b with dot below"}\
				 {"\u1E07" "b with line below"}\
				 {"\u0188" "c with hook"}\
				 {"\u0107" "c with acute"}\
				 {"\u0109" "c with circumflex"}\
				 {"\u010B" "c with dot above"}\
				 {"\u00E7" "c with cedilla"}\
				 {"\u1E09" "c with cedilla and acute"}\
				 {"\u010D" "c with caron"}\
				 {"\u018c" "d with topbar"}\
				 {"\u010F" "d with caron"}\
				 {"\u0111" "d with stroke"}\
				 {"\u0221" "d with curl"}\
				 {"\u1E0B" "d with dot above"}\
				 {"\u1E0D" "d with dot below"}\
				 {"\u1E0F" "d with line below"}\
				 {"\u021F" "h with caron"}\
				 {"\u00E8" "e with grave"}\
				 {"\u00E9" "e with acute"}\
				 {"\u00EA" "e with circumflex"}\
				 {"\u00EB" "e with diaresis"}\
				 {"\u0113" "e with macron"}\
				 {"\u0115" "e with breve"}\
				 {"\u0117" "e with dot above"}\
				 {"\u1EB9" "e with dot below"}\
				 {"\u0119" "e with ogonek"}\
				 {"\u011B" "e with caron"}\
				 {"\u0229" "e with cedilla"}\
				 {"\u0205" "e with double grave"}\
				 {"\u0207" "e with inverted breve"}\
				 {"\u01F5" "g with acute"}\
				 {"\u011D" "g with circumflex"}\
				 {"\u011F" "g with breve"}\
				 {"\u0121" "g with dot above"}\
				 {"\u0123" "g with cedilla"}\
				 {"\u01E5" "g with stroke"}\
				 {"\u01E7" "g with caron"}\
				 {"\u0125" "h with circumflex"}\
				 {"\u0127" "h with stroke"}\
				 {"\u1E25" "h with dot below"}\
				 {"\u00EC" "i with grave"}\
				 {"\u00ED" "i with acute"}\
				 {"\u00EE" "i with circumflex"}\
				 {"\u00EF" "i with diaresis"}\
				 {"\u0129" "i with tilde"}\
				 {"\u012B" "i with macron"}\
				 {"\u012D" "i with breve"}\
				 {"\u012F" "i with ogonek"}\
				 {"\u01D0" "i with caron"}\
				 {"\u0209" "i with double grave"}\
				 {"\u020B" "i with inverted breve"}\
				 {"\u1ECB" "i with dot below"}\
				 {"\u0131" "dotless i"}\
				 {"\u0135" "j with circumflex"}\
				 {"\u01F0" "j with caron"}\
				 {"\u01E9" "k with caron"}\
				 {"\u0199" "k with hook"}\
				 {"\u0137" "k with cedilla "}\
				 {"\u1E33" "k with dot below"}\
				 {"\u013A" "l with acute"}\
				 {"\u013C" "l with cedilla"}\
				 {"\u013E" "l with caron"}\
				 {"\u0140" "l with middle dot"}\
				 {"\u0142" "l with stroke"}\
				 {"\u019A" "l with bar"}\
				 {"\u0234" "l with curl"}\
				 {"\u1E37" "l with dot below"}\
				 {"\u1E39" "l with dot below and macron"}\
				 {"\u1E3B" "l with line below"}\
				 {"\u019B" "lambda with stroke"}\
				 {"\u0235" "n with a curl"}\
				 {"\u0146" "n with cedilla"}\
				 {"\u0148" "n with caron"}\
				 {"\u01F9" "n with grave"}\
				 {"\u00F1" "n with tilde"}\
				 {"\u019E" "n with long right leg"}\
				 {"\u0144" "n with acute"}\
				 {"\u1E43" "m with dot below"}\
				 {"\u1E45" "n with dot above"}\
				 {"\u1E47" "n with dot below"}\
				 {"\u1E49" "n with line below"}\
				 {"\u00F2" "o with grave"}\
				 {"\u00F3" "o with acute"}\
				 {"\u00F4" "o with circumflex"}\
				 {"\u00F5" "o with tilde"}\
				 {"\u00F6" "o with diaresis"}\
				 {"\u00F8" "o with stroke"}\
				 {"\u014D" "o with macron"}\
				 {"\u014F" "o with breve"}\
				 {"\u0151" "o with double acute"}\
				 {"\u01A1" "o with horn"}\
				 {"\u01EB" "o with ogonek"}\
				 {"\u01ED" "o with ogonek and macron"}\
				 {"\u01D2" "o with caron"}\
				 {"\u022B" "o with diaresis and macron"}\
				 {"\u022D" "o with tilde and macron"}\
				 {"\u022F" "o with dot above"}\
				 {"\u01FF" "o with stroke and acute"}\
				 {"\u020D" "o with double grave"}\
				 {"\u020F" "o with inverted breve"}\
				 {"\u0231" "o with dot above and macron"}\
				 {"\u1ECD" "o with dot below"}\
				 {"\u0223" "ou"}\
				 {"\u01A5" "p with hook"}\
				 {"\u0155" "r with acute"}\
				 {"\u0157" "r with cedilla"}\
				 {"\u0159" "r with caron"}\
				 {"\u0211" "r with double grave"}\
				 {"\u0213" "r with inverted breve"}\
				 {"\u1E5B" "r with dot below"}\
				 {"\u027F" "reversed r with fishhook"}\
				 {"\u015B" "s with acute"}\
				 {"\u015D" "s with circumflex"}\
				 {"\u015F" "s with cedilla"}\
				 {"\u0161" "s with caron"}\
				 {"\u0219" "s with comma below"}\
				 {"\u1E63" "s with dot below"}\
				 {"\u021B" "t with comma below"}\
				 {"\u01AB" "t with palatal hook"}\
				 {"\u01AD" "t with hook"}\
				 {"\u0163" "t wwith cedilla"}\
				 {"\u0165" "t with caron"}\
				 {"\u0167" "t with stroke"}\
				 {"\u0236" "t with curl"}\
				 {"\u00F9" "u with grave"}\
				 {"\u00FA" "u with acute"}\
				 {"\u00FB" "u with circumflex"}\
				 {"\u00FC" "u with diaresis"}\
				 {"\u01D4" "u with caron"}\
				 {"\u01D6" "u with diaresis and macron"}\
				 {"\u01D8" "u with diaresis and acute"}\
				 {"\u01DA" "u with diaresis and caron"}\
				 {"\u01DC" "u with diaresis and grave"}\
				 {"\u01B0" "u with horn"}\
				 {"\u0169" "u with tilde"}\
				 {"\u016B" "u with macron"}\
				 {"\u016D" "u with breve"}\
				 {"\u016F" "u with ring above"}\
				 {"\u0171" "u with double acute"}\
				 {"\u0173" "u with ogonek"}\
				 {"\u0215" "u with double grave"}\
				 {"\u0217" "u with inverted breve"}\
				 {"\u1EE5" "u with dot below"}\
				 {"\u01B4" "y with hook"}\
				 {"\u00FD" "y with acute"}\
				 {"\u00FF" "y with diaresis"}\
				 {"\u0233" "y with macron"}\
				 {"\u0175" "w with circumflex"}\
				 {"\u0177" "y with circumflex"}\
				 {"\u017A" "z with acute"}\
				 {"\u017C" "z with dot above"}\
				 {"\u017E" "z with caron"}\
				 {"\u01B6" "z with stroke"}\
				 {"\u0225" "z with hook"}\
				 {"\u01EF" "ezh with caron"}\
							 ];

#Done through 0236 			 {"\u" ""}\

	set AccentedLetterCnt [llength $AccentedLetters];
	set Rows [expr ceil(double($AccentedLetterCnt)/double($PerRow))]
	set Total [expr $PerRow * $Rows]
	#Generate buttons with blank labels for empty padding slots
	for {set k 0} {$k < $Total} {incr k} {
	    set row [expr $k/$PerRow]
	    set col [expr $k%$PerRow]
	    if {$k < $AccentedLetterCnt} {
		button .ipaea.r${row}c${col} -text "[lindex [lindex $AccentedLetters $k] 0]"\
		    -padx $xp -pady $yp\
		    -command "\$::InsertionTarget insert insert  [lindex [lindex $AccentedLetters $k] 0]"
		balloonhelpd_for .ipaea.r${row}c${col} [_ [lindex [lindex $AccentedLetters $k] 1] ]  
	    } else {
		label .ipaea.r${row}c${col} -text "";
	    }
	}
	set LastRow $row;
	set LastCol $col;
	set LastCell .ipaea.r${row}c${col}
	destroy $LastCell
	button $LastCell -image $::LeftArrowImage -command {BackDelete $::InsertionTarget}
	balloonhelpd_for $LastCell  [_ "Delete, so that you can correct mistakes while using\nthe mouse, without having to go back to the keyboard."]
	
	#Lay the buttons and labels out in a grid.
	for {set row 0} {$row <= $LastRow} {incr row} {
	    set line [list];
	    for {set col 0} {$col <= $LastCol} {incr col} {
		set cell [format ".ipaea.r%dc%d" $row $col]
		lappend line $cell
	    }
	    eval grid $line -sticky news;
	}
    }

}


set SccCnt 0;
proc PopupSpecialPalette {Title {PerRow 8} } {
    set cdefs [lindex $::SpecialCharacterPalette($Title) 0];
    incr ::SccCnt;
    set name [format ".scc%d" $::SccCnt] 
    set xp 3;
    set yp 3;
    toplevel $name -borderwidth 4
    wm title $name $Title
    BindKeys $name
    set CdefCnt [llength $cdefs];
    set Rows [expr int(ceil(double($CdefCnt)/double($PerRow)))]
    set Total [expr $PerRow * $Rows]
    if {$CdefCnt == $Total} {
	incr Rows;
	set Total [expr $PerRow * $Rows]
    }

    for {set k 0} {$k < $Total} {incr k} {
	set row [expr $k/$PerRow]
	set col [expr $k%$PerRow]
	if {$k < $CdefCnt} {
	    set entry [lindex $cdefs $k];
	    set cs [lindex $entry 0];
	    set gloss [lindex $entry 1];
	    button $name.r${row}c${col} -text $cs\
		-padx $xp -pady $yp\
		-command "\$::InsertionTarget insert insert $cs"
	    balloonhelpd_for $name.r${row}c${col} $gloss;
	} else {
	    set ln $name.r${row}c${col};
	    #Generate buttons with blank labels for empty padding slots
	    label $ln  -text "";
	    balloonhelp_for $ln [_ "This space for rent."]
	}
    }
    set LastRow $row;
    set LastCol $col;
    set LastCell $name.r${row}c${col}
    destroy $LastCell
    button $LastCell -image $::LeftArrowImage -command {BackDelete $::InsertionTarget}
    balloonhelpd_for $LastCell  [_ "Delete, so that you can correct mistakes while using\nthe mouse, without having to go back to the keyboard."]
    
    #Lay the buttons and labels out in a grid.
    for {set row 0} {$row <= $LastRow} {incr row} {
	set line [list];
	for {set col 0} {$col <= $LastCol} {incr col} {
	    set cell [format "%s.r%dc%d" $name $row $col]
	    lappend line $cell
	}
	eval grid $line -sticky news;
    }
    return $name;
}

proc LoadCustomCharacterChart {args} {
    if {[llength $args]} {
	set fn [lindex $args 0]
    } else {
	set fn [tk_getOpenFile -title [_ "Load Custom Character Chart"]];
	if {[string equal $fn ""]} {
	    ShowMessage [_ "File selection cancelled."];
	    return ;
	}
    }
    if { [catch {open $fn "r"} fhd ] != 0} {
	ShowMessage [format [_ "Unable to open character chart definition file %s."] $fn];
	return ;
    }
    set LineCnt 0
    while { [gets $fhd line] > 0} {
	lappend Lines $line;
	incr LineCnt
    }
    close $fhd;
    if {$LineCnt < 1} {
	ShowMessage [_ "File %s is empty" $fn]
	return "";
    }
    ShowMessage [format [_ "Loaded custom character chart definition from %s."] $fn]
    set w [DefineCustomCharacterChart $Lines $fn];
    unset Lines;
    return $w;
}

proc DefineCustomCharacterChart {LineList {fn NONE}} {
    #The first line is special. It contains the number of buttons
    #per row and the title.
    set line [lindex $LineList 0]
    set flds [split $line "|"]
    set PerRow [lindex $flds 0];
    set Title [lindex $flds 1];
    set LineList [lrange $LineList 1 end]

    #The remaining lines contain pairs of code sequences and glosses.
    set cd [list];
    foreach line $LineList {
	set f [split $line "|"];
	set gloss [lindex $f 1];
	set c [string trim [lindex $f 0] \"];
	set cf [split $c];
	set str "\{\"";
	foreach n $cf {
#	    append str [format "\\u%s" $n]
	    append str [format "%s" $n]
	}
	append str "\""
	append str [format " \"%s\"\}" $gloss]
	lappend cd $str;
    }
    set info [list [join $cd] $PerRow];
    if {![string equal $fn NONE]} {
	lappend info $fn;
    }
    set ::SpecialCharacterPalette($Title) $info;
    $::CEM add command -label $Title -command "PopupSpecialPalette [list $Title] $PerRow"
    incr ::CharentryMenuItems;
    return [PopupSpecialPalette $Title $PerRow];
}


set LeftArrowImage [image create photo -data {
R0lGODlhFAAUAIAAAAAAAL+/vyH5BAEAAAEALAAAAAAUABQAAAIqjI+py43gGIAxTVrNxXVz
54WTIopWGZ7oRq7X4o4wm2lvbX+ZjGv9/ysAADs=
}]

proc BackDelete {w} {
    if {[string equal [winfo class $w] "Text"]} {
	set Insert [$w index insert]
	set FirstHalf  [$w get 1.0 $Insert-1chars]
	set SecondHalf [string trimright [$w get insert end]]
	$w delete 1.0 end;
	$w insert 1.0 $FirstHalf$SecondHalf;
    } else {
	set delind [expr [$w index insert] -1];
	$w delete $delind;
    }
}

proc PopupCharEntryByCode {} {
    toplevel .charent -borderwidth 4 -relief raised
    BindKeys .charent;
    wm title .charent "Entry By Codepoint";
    after idle {
	update idletasks
	set xmax [winfo screenwidth .charent]
	set ymax [winfo screenheight .charent]
	set x0 [expr 1 * ($xmax -[winfo reqwidth .charent])/3];
	set y0 [expr 1 * ($ymax -[winfo reqheight .charent])/3];
	wm geometry .charent "+$x0+$y0";
    }
    label .charent.title -text [_ "Insert Character by Numerical Code"]
    frame .charent.ef;
    entry .charent.ef.ent -width 8 -font MainFont -relief flat -bg \#E0EEFF;
    label .charent.ef.prefix -text "0x" -font MainFont
    pack .charent.ef.prefix -side left  -expand 1 -fill x -anchor e
    pack .charent.ef.ent    -side right -expand 1 -fill x -anchor w
    button .charent.d  -text [_ "Dismiss"] -command {destroy .charent}
    button .charent.x  -text [_ "Delete"] -command [list BackDelete $::CharByCodeInsertionTarget]
    button .charent.i  -text [_ "Insert"] -command {InsertUnicode}
    pack .charent.title -side top
    pack .charent.ef -side top
    pack .charent.d -side left -expand 1 -fill both
    pack .charent.x -side left -expand 1 -fill both
    pack .charent.i -side right -expand 1 -fill both
    focus .charent.ef.ent;
    set DownMsg [_ "Display Widget for Entering Characters by Unicode Code"];
    set UpMsg   [_ "Remove Widget for Entering Characters by Unicode Code"];
    bind .charent <Destroy> \
	"set ::CharEntryByCodeIsDisplayedP 0;$::CEM entryconfigure $::ToggleCharEntryByCodeIndex -label \"$DownMsg\""
    bind .charent <Unmap> \
	"set ::CharEntryByCodeIsDisplayedP 0;$::CEM entryconfigure $::ToggleCharEntryByCodeIndex -label \"$DownMsg\""
    bind .charent <Map> \
	"set ::CharEntryByCodeIsDisplayedP 1;$::CEM entryconfigure $::ToggleCharEntryByCodeIndex -label \"$UpMsg\""
    bind .charent.ef.ent <Destroy> {set ::InsertionTarget [focus -lastfor .gen]}
    bind .charent.ef.ent <Return> {InsertUnicode}
    bind .charent.ef.ent <Control-k> {.charent.ef.ent delete 0 end;break}


    set charentbh  [_ "Enter a character by its Unicode code, as a sequence of four ctradecimal digits.\nFor example, you may specify \'\u0298\', the International Phonetic Alphabet\nsymbol for the bilabial click, as 0298, and the Chinese character \'\u4E39\' \"egg\" as 4E39.\nDo not type the prefix 0x shown to the left of the entry box.\nIt is entered for you automatically. Return inserts the current character.\nControl-k erases the current character code."]
    balloonhelp_for .charent $charentbh
    balloonhelp_for .charent.title $charentbh
    balloonhelp_for .charent.ef.ent $charentbh
    balloonhelp_for .charent.i $charentbh
    balloonhelp_for .charent.d $charentbh
    balloonhelp_for .charent.x $charentbh
    set ::CharEntryByCodeIsDisplayedP 1;
}

#Get entry from popup, validate it, and insert it into the insertion target.
proc InsertUnicode {} {
    ClearMessageWindow;
    set str [.charent.ef.ent get];
    if {[string length $str] == 0} {
	ShowMessage [_ "The empty string is not a valid Unicode codepoint"]
	return ;
    }
    #Validate
    set BadValueP 0;
    #Make sure all digits are ctr and that prefix is appropriate.
    if {[regexp {[[:xdigit:]]{4,4}} $str] == 0} {
	ShowMessage [_ "$str is not a well-formed ctradecimal Unicode value"]
	return;
    }
    set str [format "0x%s" $str]
    if {[scan $str "%x" num] < 1} {
	ShowMessage [format [_ "Ill-formed code %s"] $str]
	return;
    }

    #Reject illegal codepoints.
    if {$num  >  65535} {
	ShowMessage [_ "Tcl/Tk may not support codepoints outside the BMP (Plane 0)."]
    }
    set BadValueP 0;
    set BadRanges [list\
  	       0x0750 0x077F\
	       0x07C0 0x08FF\
	       0x1380 0x139F\
	       0x18B0 0x18FF\
	       0x1980 0x19DF\
	       0x1A00 0x1CFF\
	       0x1D80 0x1DFF\
	       0x2C00 0x2E7F\
	       0x2FE0 0x2FEF\
	       0x31C0 0x31EF\
	       0x9FB0 0x9FFF\
	       0xA4D0 0xABFF\
	       0xD7B0 0xD7FF\
	       0xD800 0xDBFF\
	       0xDC00 0xDFFF\
	       0xFE10 0xFE1F\
	       0x10140 0x102FF\
	       0x104B0 0x107FF\
	       0x10840 0x1CFFF\
	       0x1D200 0x1D2FF\
	       0x1D360 0x1D3FF\
	       0x1D800 0x1FFFF\
	       0x2A6E0 0x2F7FF\
	       0x2FAB0 0x2FFFF\
	       0xE0080 0xE00FF\
	       0xE01F0 0xEFFFF\
	       0xFFFFE 0xFFFFF];

    for {set k 0} {$k < [llength $BadRanges]} {incr k} {
	if { ($num >= [lindex $BadRanges [expr 2 * $k]]) &&\
		 ($num <= [lindex $BadRanges [expr (2 * $k) + 1]])} { 
	    set BadValueP 1;
	    break; 
	}
    }
    if {$BadValueP} {
	ShowMessage [format [_ "%s is not a valid Unicode codepoint"] $str]
	return;
    }
    if {$num >= 1114110} {
	ShowMessage [_ "Warning: codepoints above 0x10FFFD have not been assigned as of version 4.0."]
    }

    #Insert
    $::CharByCodeInsertionTarget insert insert [format "%c" $num]
}

#Set up balloon help
option add *Balloonhelp*background white widgetDefault
option add *Balloonhelp*foreground black widgetDefault
option add *Balloonhelpinfo.wrapLength 3i  widgetDefault
option add *Balloonhelp.info.justify left widgetDefault
toplevel .balloonhelp -class Balloonhelp -background black -borderwidth 1 -relief flat
#label .balloonhelp.arrow -anchor nw -bitmap @arrow.xbm
#pack .balloonhelp.arrow -side left -fill y
label .balloonhelp.info -font BalloonHelpFont;
pack .balloonhelp.info -side left -fill y
wm overrideredirect .balloonhelp 1
wm withdraw .balloonhelp
set bhInfo(active) 1

proc balloonhelp_control {state} {
     global bhInfo
     if {$state} {
          set bhInfo(active) 1
     } else {
	balloonhelp_cancel
	set bhInfo(active) 0
     }
}

proc balloonhelp_for {win mesg} {
    global bhInfo
    set bhInfo($win) $mesg
    set ::bhOverlapP($win) 1; 
    bind $win <Enter> {+balloonhelp_pending %W}
    bind $win <Leave> {+balloonhelp_cancel}
}

proc balloonhelpd_for {win mesg} {
    global bhInfo
    set ::bhOverlapP($win) 0;
    set bhInfo($win) $mesg
    bind $win <Enter> {+balloonhelp_show %W}
    bind $win <Leave> {+wm withdraw .balloonhelp}
}

proc balloonhelp_pending {win} {
     global bhInfo
     balloonhelp_cancel
     set bhInfo(pending) [after 1000 [list balloonhelp_show $win]]
}

proc balloonhelp_cancel {} {
    global bhInfo
    if { [info exists bhInfo(pending)]} {
	after cancel $bhInfo(pending)
	unset bhInfo(pending)
    }
    wm withdraw .balloonhelp
}

proc balloonhelp_show {win} {
    global bhInfo;
    global bhOverlapP;
    if {$bhOverlapP($win)} {
	set Overlap 25;
    } else {
	set Overlap -10;
    }
    if {[winfo exists $win]} {
	if {$bhInfo(active)} {
	    .balloonhelp.info configure -text $bhInfo($win)
	    #Set abcissa
	    set MaxStringWidth 0;
	    foreach line [split $bhInfo($win) "\n"] {
		set StringWidth [font measure BalloonHelpFont -displayof .balloonhelp.info $line]
		if {$StringWidth > $MaxStringWidth} {
		    set MaxStringWidth $StringWidth;
		}
	    }
	    set ScreenWidth [winfo screenwidth $win]
	    set Width [winfo width $win];
	    set LeftEdge  [winfo rootx $win];
	    set RightEdge [expr $LeftEdge + $Width];
	    if {$ScreenWidth - $RightEdge < $MaxStringWidth} { 
		if {$LeftEdge > $MaxStringWidth} {
		    set x [expr $LeftEdge - $MaxStringWidth + $Overlap];
		} else {
		    if {$ScreenWidth - $MaxStringWidth > 0} {
			set x [expr $RightEdge - $MaxStringWidth];
		    } else {
			set x [expr $ScreenWidth - $MaxStringWidth];
		    }
		}
	    } else {
		set x [expr $RightEdge - $Overlap];
	    }
	    #Set ordinate
	    set Height [winfo height $win];
	    set TopEdge [winfo rooty $win];
#	    set y [expr $TopEdge + ($Height/2)];
	    set y [expr $TopEdge + int(($Height/1.25))];
	    wm geometry .balloonhelp +$x+$y
	    wm deiconify .balloonhelp
	    raise .balloonhelp
	}
    }
    if {[info exist bhInfo(pending)]} {
	unset bhInfo(pending)
    }
}

proc ToggleBalloonHelp {} {
    global BalloonHelpP;
    global BalloonHelpIndex;
    global m;

    if {$BalloonHelpP} {
	set BalloonHelpP 0;
	balloonhelp_control 0
	ShowMessage [_ "Irritating Balloon Help Disabled"];
	$m.configure entryconfigure $BalloonHelpIndex -label [_ "Show Balloon Help"];
    } else {
	set BalloonHelpP 1;
	balloonhelp_control 1
	ShowMessage [_ "Balloon Help Enabled"];
	$m.configure entryconfigure $BalloonHelpIndex -label [_ "Hide Irritating Balloon Help"];
    }
}

proc new_dialog_create {class {win "auto"}} {
    if {$win == "auto"} {
        set count 0
        set win ".ndialog[incr count]"
        while {[winfo exists $win]} {
            set win ".ndialog[incr count]"
        }
    }
    toplevel $win -class $class;
    frame $win.info
    pack $win.info -expand yes -fill both -padx 4 -pady 4
    wm title $win $class
    wm group $win .

    after idle [format {
        update idletasks
        wm minsize %s [winfo reqwidth %s] [winfo reqheight %s]
    } $win $win $win]

    return $win
}

# The following code is taken from the Efftcl library by Mark Harrison and
# Michael McLennan, copyrighted by Mark Harrison and Lucent Technologies, available
# from http://www.awprofessional.com/content/images/0201634740/sourcecode/efftcl.zip.
# As the authors explicitly give permission to "steal the code for your own applications"
# the relevant portions are included here so as not to require the user to install
# to install the library. If you install the library, remove the following and
# uncomment the line "#package require Efftcl" by deleting the crosshatch.

#  Effective Tcl/Tk Programming
#    Mark Harrison, DSC Communications Corp.
#    Michael McLennan, Bell Labs Innovations for Lucent Technologies
#    Addison-Wesley Professional Computing Series
# ======================================================================
#  Copyright (c) 1996-1997  Lucent Technologies Inc. and Mark Harrison
# ======================================================================

proc dialog_create {class {win "auto"}} {
    if {$win == "auto"} {
        set count 0
        set win ".dialog[incr count]"
        while {[winfo exists $win]} {
            set win ".dialog[incr count]"
        }
    }
    toplevel $win -class $class -background \#2200DD;

    frame $win.info
    pack $win.info -expand yes -fill both -padx 4 -pady 4
    frame $win.sep -height 2 -borderwidth 1 -relief sunken
    pack $win.sep -fill x -pady 4
    frame $win.controls
    pack $win.controls -fill x -padx 4 -pady 1

    wm title $win $class
    wm group $win .

    after idle [format {
        update idletasks
        wm minsize %s [winfo reqwidth %s] [winfo reqheight %s]
    } $win $win $win]

    return $win
}

proc dialog_info {win} {
    return "$win.info"
}

proc dialog_controls {win} {
    return "$win.controls"
}

proc dialog_wait {win varName} {
    dialog_safeguard $win

    set x [expr [winfo rootx .]+50]
    set y [expr [winfo rooty .]+50]
    wm geometry $win "+$x+$y"

    wm deiconify $win
    grab set $win

    vwait $varName

    grab release $win
    wm withdraw $win
}

bind modalDialog <ButtonPress> {
    wm deiconify %W
    raise %W
}
proc dialog_safeguard {win} {
    if {[lsearch [bindtags $win] modalDialog] < 0} {
        bindtags $win [linsert [bindtags $win] 0 modalDialog]
    }
}

proc CreateTextDisplay {title width height {bg "\#96C4A3"} {fg "\#000000"} } {
    set top [new_dialog_create Textdisplay]
    wm title $top $title
    set info [dialog_info $top]
    scrollbar $info.sbar -command "$info.text yview" 
    pack $info.sbar -side right -fill y
    text $info.text -height $height -width $width -font MainFont -wrap word -yscrollcommand "$info.sbar set"  -background $bg -foreground $fg -exportselection 1;
    pack $info.text -side left -expand yes -fill both
    bind $info.sbar <<B3>>  "ScrollbarMoveBigIncrement $info.sbar 0.20 %x %y"
    $info.text configure -state disabled
    return $top
}

proc AppendToTextDisplay {top mesg} {
    set info [dialog_info $top]
    $info.text configure -state normal
    $info.text insert end $mesg
    $info.text configure -state disabled
}

set linkNum 0;
proc AppendLinkToTextDisplay {top mesg LinkCode} {
    global linkNum
    set info [dialog_info $top]
    $info.text configure -state normal
    set tag "link[incr linkNum]"
    bind $info.text <ButtonPress> break
    $info.text insert end $mesg [list body $tag]
    $info.text tag configure $tag -foreground red -underline 1
    $info.text tag bind $tag <Enter> \
        "$info.text tag configure $tag -foreground blue"
    $info.text tag bind $tag <Leave> \
        "$info.text tag configure $tag -foreground red"
    $info.text tag bind $tag <ButtonPress> \
        "$LinkCode"
    $info.text configure -state disabled
}

proc About {} {
    global Version;
    global PackageVersion;

    if {[PopupDown About] ==1} {return}
    set po [CreateTextDisplay [_ "About"] 55 10];
    set ::PopupList(About) $po
    BindKeys $po;
    AppendToTextDisplay $po [format [_ "This is mpg version %s."] $Version];
    AppendToTextDisplay $po [format [_ "It is part of the minpair package, version %s."] $PackageVersion];
    AppendToTextDisplay $po [_ "It is a graphical front end to Minpair, a program for finding minimal pairs in a list of words. In order for it to be of any use, you must have Minpair installed."];
    AppendToTextDisplay $po [_ "You can obtain both Minpair and MinpairG from:\n"];
    AppendLinkToTextDisplay $po "http://www.billposer.org/Software/minpair.html" {ShowWebPage http://www.billposer.org/Software/minpair.html};
    AppendToTextDisplay $po "\n";
    AppendToTextDisplay $po [_ "Both programs were written by Bill Poser (billposer@alum.mit.edu)."];
    AppendToTextDisplay $po "\n";
}

proc BugReports {} {
    global Version;
    global PackageVersion;
    global tcl_platform;
    global BugReportPopup

    if {[PopupDown BugReports] ==1} {return}
    set po [CreateTextDisplay [_ "Bug Reports"] 50 21];
    set ::PopupList(BugReports) $po
    BindKeys $po;
    AppendToTextDisplay $po [_ "Report bugs to: billposer@alum.mit.edu.\n"];
    AppendToTextDisplay $po [_ "Please include the following information:\n\n"];
    AppendToTextDisplay $po [_ "\tWhat version of the program are you using?\n"];
    AppendToTextDisplay $po [format [_ "\t\t(This is version %s.)\n\n"] $Version];
    AppendToTextDisplay $po [_ "\tWhat version of minpair are you using?\n"];
    AppendToTextDisplay $po [format [_ "\t\t(Version located: %s.)\n\n"] [FindoutMinpairVersion]];
    AppendToTextDisplay $po [_ "\tWhat operating system are you running?.\n"];
    set OS $tcl_platform(os);
    if {$OS == "Linux"} {set OS "GNU/Linux"};
    AppendToTextDisplay $po [format [_ "\t\t(This is %s  %s.)\n\n"] $OS  $tcl_platform(osVersion)];
    AppendToTextDisplay $po [_ "\tWhat window system are you using?.\n"];
    AppendToTextDisplay $po [format [_ "\t\t(This is %s.)\n\n"] $::WindowSystem]
    AppendToTextDisplay $po [_ "\tWhat version of tcl/tk are you using?.\n"];
    AppendToTextDisplay $po [format [_ "\t\t(This is version %s.)\n\n"] [info patchlevel]];
    AppendToTextDisplay $po [_ "Bug reports may be sent in any language that I can read without too much trouble or am trying to learn or improve. These include:\n\n"];
    AppendLinkToTextDisplay $po [_ "\tCatalan"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=cat};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tDakelh (Carrier)"] {ShowWebPage http://ydli.org}
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tDutch"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=nld};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tEnglish"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=eng};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tEsperanto"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=epo};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tFrench"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=fra};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tGerman"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=deu};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tItalian"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=ita};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tJapanese"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=jpn};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tKorean"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=kor};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tLatin"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=lat};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tPortuguese"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=por};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tSpanish"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=spa};
    AppendToTextDisplay $po "\n";
    AppendLinkToTextDisplay $po [_ "\tTurkish"] {ShowWebPage http://www.ethnologue.com/show_language.asp?code=tur};
    AppendToTextDisplay $po "\n";
}


proc HowTo {} {
    if {[PopupDown HowTo] ==1} {return}
    set po [CreateTextDisplay [_ "How To Use This Program"] 55 20];
    set ::PopupList(HowTo) $po
    BindKeys $po;
    AppendToTextDisplay $po [_ "This program provides a graphical interface to Minpair, a program for finding minimal pairs in a list of words. Once you have chosen the input and output files, you can execute Minpair by pressing the \'Execute\' button. The rest of the interface allows you to set various options.\n"];
    AppendToTextDisplay $po "\n";
    AppendToTextDisplay $po "If you want to see the command that is executed, use the command \'Show Command Line\' on the \'File\' menu. If you want to save the settings used in a file or to create a shell script that will run Minpair with the same settings, use the command \'Save Command Line\' on the \'File\' menu.\n";
    AppendToTextDisplay $po "\n";
    AppendToTextDisplay $po [_ "Most sections of the interface have balloon help associated with them. When you move the pointer over a region with balloon help, a text balloon containing a short explanation of that section will pop up. When you move the pointer away, it will disappear.\n"];
    AppendToTextDisplay $po "\n";
    AppendToTextDisplay $po [_ "Balloon help is very useful when you are learning to use a program, but once you are acquainted with it, it can be irritating. You can turn balloon help on and off by means of an entry on the Help menu.\n"];
    AppendToTextDisplay $po "\n";
    AppendToTextDisplay $po [_ "In some cases a more detailed explanation than is appropriate for balloon help may be needed. In such cases, you can pop up a window containing help text by right-clicking on the appropriate section. A second right-click in the same place will remove the help text.\n"];
    AppendToTextDisplay $po "\n";
    AppendToTextDisplay $po "In order to assist you in defining multigraphs, a number of character entry tools are available on the \'Character Entry\' menu. These tools insert characters into the entry field that was last in focus. To insert a character, pass the pointer over the place that you want to enter it and then move it to the character entry widget without passing over any other entry field. If you have disabled balloon help you may wish to reenable it when using these tools as some of them use balloon help to identify the characters that they insert, which may be hard to read.\n";
}

proc ShowGPL {} {
    if {[PopupDown ShowGPL] ==1} {return}
    set po [CreateTextDisplay [_ "License"] 72 24]
    set ::PopupList(ShowGPL) $po;
    BindKeys $po;
    AppendToTextDisplay $po [format "%s%s" [format "%s\n\t%s\n" [_ "For this license in your language see:"] [_ "http://www.gnu.org/copyleft/gpl.html"]]  "\
\

		GNU GENERAL PUBLIC LICENSE\

   Version 3, 29 June 2007\


 Copyright (C) 2007 Free Software Foundation, Inc. <http://fsf.org/>
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.
\

Preamble
\

  The GNU General Public License is a free, copyleft license for
software and other kinds of works.
\

  The licenses for most software and other practical works are designed
to take away your freedom to share and change the works.  By contrast,
the GNU General Public License is intended to guarantee your freedom to
share and change all versions of a program--to make sure it remains free
software for all its users.  We, the Free Software Foundation, use the
GNU General Public License for most of our software; it applies also to
any other work released this way by its authors.  You can apply it to
your programs, too.
\

  When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
							them if you wish), that you receive source code or can get it if you
want it, that you can change the software or use pieces of it in new
free programs, and that you know you can do these things.
\

To protect your rights, we need to prevent others from denying you
these rights or asking you to surrender the rights.  Therefore, you have
certain responsibilities if you distribute copies of the software, or if
you modify it: responsibilities to respect the freedom of others.
\

For example, if you distribute copies of such a program, whether
gratis or for a fee, you must pass on to the recipients the same
freedoms that you received.  You must make sure that they, too, receive
or can get the source code.  And you must show them these terms so they
know their rights.
\

Developers that use the GNU GPL protect your rights with two steps:
(1) assert copyright on the software, and (2) offer you this License
giving you legal permission to copy, distribute and/or modify it.
\

For the developers' and authors' protection, the GPL clearly explains
that there is no warranty for this free software.  For both users' and
authors' sake, the GPL requires that modified versions be marked as
changed, so that their problems will not be attributed erroneously to
authors of previous versions.
\

Some devices are designed to deny users access to install or run
modified versions of the software inside them, although the manufacturer
can do so.  This is fundamentally incompatible with the aim of
protecting users' freedom to change the software.  The systematic
pattern of such abuse occurs in the area of products for individuals to
use, which is precisely where it is most unacceptable.  Therefore, we
have designed this version of the GPL to prohibit the practice for those
products.  If such problems arise substantially in other domains, we
stand ready to extend this provision to those domains in future versions
of the GPL, as needed to protect the freedom of users.
\

Finally, every program is threatened constantly by software patents.
States should not allow patents to restrict development and use of
software on general-purpose computers, but in those that do, we wish to
avoid the special danger that patents applied to a free program could
make it effectively proprietary.  To prevent this, the GPL assures that
patents cannot be used to render the program non-free.
\

The precise terms and conditions for copying, distribution and
modification follow.
\

TERMS AND CONDITIONS
\

0. Definitions.
\

\"This License\" refers to version 3 of the GNU General Public License.
\

\"Copyright\" also means copyright-like laws that apply to other kinds of
works, such as semiconductor masks.
\

\"The Program\" refers to any copyrightable work licensed under this
License.  Each licensee is addressed as \"you\".  \"Licensees\" and
\"recipients\" may be individuals or organizations.
\

To \"modify\" a work means to copy from or adapt all or part of the work
in a fashion requiring copyright permission, other than the making of an
exact copy.  The resulting work is called a \"modified version\" of the
earlier work or a work \"based on\" the earlier work.
\

A \"covered work\" means either the unmodified Program or a work based
on the Program.
\

To \"propagate\" a work means to do anything with it that, without
permission, would make you directly or secondarily liable for
infringement under applicable copyright law, except executing it on a
computer or modifying a private copy.  Propagation includes copying,
distribution (with or without modification), making available to the
public, and in some countries other activities as well.
\

To \"convey\" a work means any kind of propagation that enables other
parties to make or receive copies.  Mere interaction with a user through
a computer network, with no transfer of a copy, is not conveying.
\

An interactive user interface displays \"Appropriate Legal Notices\"
to the extent that it includes a convenient and prominently visible
feature that (1) displays an appropriate copyright notice, and (2)
tells the user that there is no warranty for the work (except to the
						       extent that warranties are provided), that licensees may convey the
work under this License, and how to view a copy of this License.  If
the interface presents a list of user commands or options, such as a
menu, a prominent item in the list meets this criterion.
\

1. Source Code.
\

The \"source code\" for a work means the preferred form of the work
for making modifications to it.  \"Object code\" means any non-source
form of a work.
\

A \"Standard Interface\" means an interface that either is an official
standard defined by a recognized standards body, or, in the case of
interfaces specified for a particular programming language, one that
is widely used among developers working in that language.
\

The \"System Libraries\" of an executable work include anything, other
than the work as a whole, that (a) is included in the normal form of
packaging a Major Component, but which is not part of that Major
Component, and (b) serves only to enable use of the work with that
Major Component, or to implement a Standard Interface for which an
implementation is available to the public in source code form.  A
\"Major Component\", in this context, means a major essential component
(kernel, window system, and so on) of the specific operating system
(if any) on which the executable work runs, or a compiler used to
produce the work, or an object code interpreter used to run it.
\

The \"Corresponding Source\" for a work in object code form means all
the source code needed to generate, install, and (for an executable
						  work) run the object code and to modify the work, including scripts to
control those activities.  However, it does not include the work's
System Libraries, or general-purpose tools or generally available free
programs which are used unmodified in performing those activities but
which are not part of the work.  For example, Corresponding Source
includes interface definition files associated with source files for
the work, and the source code for shared libraries and dynamically
linked subprograms that the work is specifically designed to require,
such as by intimate data communication or control flow between those
subprograms and other parts of the work.
\

The Corresponding Source need not include anything that users
can regenerate automatically from other parts of the Corresponding
Source.
\

The Corresponding Source for a work in source code form is that
same work.
\

2. Basic Permissions.
\

All rights granted under this License are granted for the term of
copyright on the Program, and are irrevocable provided the stated
conditions are met.  This License explicitly affirms your unlimited
permission to run the unmodified Program.  The output from running a
covered work is covered by this License only if the output, given its
content, constitutes a covered work.  This License acknowledges your
rights of fair use or other equivalent, as provided by copyright law.
\

You may make, run and propagate covered works that you do not
convey, without conditions so long as your license otherwise remains
in force.  You may convey covered works to others for the sole purpose
of having them make modifications exclusively for you, or provide you
with facilities for running those works, provided that you comply with
the terms of this License in conveying all material for which you do
not control copyright.  Those thus making or running the covered works
for you must do so exclusively on your behalf, under your direction
and control, on terms that prohibit them from making any copies of
your copyrighted material outside their relationship with you.
\

Conveying under any other circumstances is permitted solely under
the conditions stated below.  Sublicensing is not allowed; section 10
makes it unnecessary.
\

3. Protecting Users' Legal Rights From Anti-Circumvention Law.
\

No covered work shall be deemed part of an effective technological
measure under any applicable law fulfilling obligations under article
11 of the WIPO copyright treaty adopted on 20 December 1996, or
similar laws prohibiting or restricting circumvention of such
measures.
\

When you convey a covered work, you waive any legal power to forbid
circumvention of technological measures to the extent such circumvention
is effected by exercising rights under this License with respect to
the covered work, and you disclaim any intention to limit operation or
modification of the work as a means of enforcing, against the work's
users, your or third parties' legal rights to forbid circumvention of
technological measures.
\

4. Conveying Verbatim Copies.
\

You may convey verbatim copies of the Program's source code as you
receive it, in any medium, provided that you conspicuously and
appropriately publish on each copy an appropriate copyright notice;
keep intact all notices stating that this License and any
non-permissive terms added in accord with section 7 apply to the code;
keep intact all notices of the absence of any warranty; and give all
recipients a copy of this License along with the Program.
\

You may charge any price or no price for each copy that you convey,
and you may offer support or warranty protection for a fee.
\

5. Conveying Modified Source Versions.
\

You may convey a work based on the Program, or the modifications to
produce it from the Program, in the form of source code under the
terms of section 4, provided that you also meet all of these conditions:
\

a) The work must carry prominent notices stating that you modified
it, and giving a relevant date.
\

b) The work must carry prominent notices stating that it is
released under this License and any conditions added under section
7.  This requirement modifies the requirement in section 4 to
\"keep intact all notices\".
\

c) You must license the entire work, as a whole, under this
License to anyone who comes into possession of a copy.  This
License will therefore apply, along with any applicable section 7
additional terms, to the whole of the work, and all its parts,
regardless of how they are packaged.  This License gives no
permission to license the work in any other way, but it does not
invalidate such permission if you have separately received it.
\

d) If the work has interactive user interfaces, each must display
Appropriate Legal Notices; however, if the Program has interactive
interfaces that do not display Appropriate Legal Notices, your
work need not make them do so.
\

A compilation of a covered work with other separate and independent
works, which are not by their nature extensions of the covered work,
and which are not combined with it such as to form a larger program,
in or on a volume of a storage or distribution medium, is called an
\"aggregate\" if the compilation and its resulting copyright are not
used to limit the access or legal rights of the compilation's users
beyond what the individual works permit.  Inclusion of a covered work
in an aggregate does not cause this License to apply to the other
parts of the aggregate.
\

6. Conveying Non-Source Forms.
\

You may convey a covered work in object code form under the terms
of sections 4 and 5, provided that you also convey the
machine-readable Corresponding Source under the terms of this License,
in one of these ways:
\

a) Convey the object code in, or embodied in, a physical product
(including a physical distribution medium), accompanied by the
Corresponding Source fixed on a durable physical medium
customarily used for software interchange.
\

b) Convey the object code in, or embodied in, a physical product
(including a physical distribution medium), accompanied by a
written offer, valid for at least three years and valid for as
long as you offer spare parts or customer support for that product
model, to give anyone who possesses the object code either (1) a
copy of the Corresponding Source for all the software in the
product that is covered by this License, on a durable physical
medium customarily used for software interchange, for a price no
more than your reasonable cost of physically performing this
conveying of source, or (2) access to copy the
Corresponding Source from a network server at no charge.
\

c) Convey individual copies of the object code with a copy of the
written offer to provide the Corresponding Source.  This
alternative is allowed only occasionally and noncommercially, and
only if you received the object code with such an offer, in accord
with subsection 6b.
\

d) Convey the object code by offering access from a designated
place (gratis or for a charge), and offer equivalent access to the
Corresponding Source in the same way through the same place at no
further charge.  You need not require recipients to copy the
Corresponding Source along with the object code.  If the place to
copy the object code is a network server, the Corresponding Source
may be on a different server (operated by you or a third party)
that supports equivalent copying facilities, provided you maintain
clear directions next to the object code saying where to find the
Corresponding Source.  Regardless of what server hosts the
Corresponding Source, you remain obligated to ensure that it is
available for as long as needed to satisfy these requirements.
\

e) Convey the object code using peer-to-peer transmission, provided
you inform other peers where the object code and Corresponding
Source of the work are being offered to the general public at no
charge under subsection 6d.
\

A separable portion of the object code, whose source code is excluded
from the Corresponding Source as a System Library, need not be
included in conveying the object code work.
\

A \"User Product\" is either (1) a \"consumer product\", which means any
tangible personal property which is normally used for personal, family,
or household purposes, or (2) anything designed or sold for incorporation
into a dwelling.  In determining whether a product is a consumer product,
doubtful cases shall be resolved in favor of coverage.  For a particular
product received by a particular user, \"normally used\" refers to a
typical or common use of that class of product, regardless of the status
of the particular user or of the way in which the particular user
actually uses, or expects or is expected to use, the product.  A product
is a consumer product regardless of whether the product has substantial
commercial, industrial or non-consumer uses, unless such uses represent
the only significant mode of use of the product.
\

\"Installation Information\" for a User Product means any methods,
procedures, authorization keys, or other information required to install
and execute modified versions of a covered work in that User Product from
a modified version of its Corresponding Source.  The information must
suffice to ensure that the continued functioning of the modified object
code is in no case prevented or interfered with solely because
modification has been made.
\

If you convey an object code work under this section in, or with, or
specifically for use in, a User Product, and the conveying occurs as
part of a transaction in which the right of possession and use of the
User Product is transferred to the recipient in perpetuity or for a
fixed term (regardless of how the transaction is characterized), the
Corresponding Source conveyed under this section must be accompanied
by the Installation Information.  But this requirement does not apply
if neither you nor any third party retains the ability to install
modified object code on the User Product (for example, the work has
					  been installed in ROM).
\

The requirement to provide Installation Information does not include a
requirement to continue to provide support service, warranty, or updates
for a work that has been modified or installed by the recipient, or for
the User Product in which it has been modified or installed.  Access to a
network may be denied when the modification itself materially and
adversely affects the operation of the network or violates the rules and
protocols for communication across the network.
\

Corresponding Source conveyed, and Installation Information provided,
in accord with this section must be in a format that is publicly
documented (and with an implementation available to the public in
	    source code form), and must require no special password or key for
unpacking, reading or copying.
\

7. Additional Terms.
\

\"Additional permissions\" are terms that supplement the terms of this
License by making exceptions from one or more of its conditions.
Additional permissions that are applicable to the entire Program shall
be treated as though they were included in this License, to the extent
that they are valid under applicable law.  If additional permissions
apply only to part of the Program, that part may be used separately
under those permissions, but the entire Program remains governed by
this License without regard to the additional permissions.
\

When you convey a copy of a covered work, you may at your option
remove any additional permissions from that copy, or from any part of
it.  (Additional permissions may be written to require their own
      removal in certain cases when you modify the work.)  You may place
additional permissions on material, added by you to a covered work,
for which you have or can give appropriate copyright permission.
\

Notwithstanding any other provision of this License, for material you
add to a covered work, you may (if authorized by the copyright holders of
				that material) supplement the terms of this License with terms:
\

a) Disclaiming warranty or limiting liability differently from the
terms of sections 15 and 16 of this License; or
\

b) Requiring preservation of specified reasonable legal notices or
author attributions in that material or in the Appropriate Legal
Notices displayed by works containing it; or
\

c) Prohibiting misrepresentation of the origin of that material, or
requiring that modified versions of such material be marked in
reasonable ways as different from the original version; or
\

d) Limiting the use for publicity purposes of names of licensors or
authors of the material; or
\

e) Declining to grant rights under trademark law for use of some
trade names, trademarks, or service marks; or
\

f) Requiring indemnification of licensors and authors of that
material by anyone who conveys the material (or modified versions of
					     it) with contractual assumptions of liability to the recipient, for
any liability that these contractual assumptions directly impose on
those licensors and authors.
\

All other non-permissive additional terms are considered \"further
restrictions\" within the meaning of section 10.  If the Program as you
received it, or any part of it, contains a notice stating that it is
governed by this License along with a term that is a further
restriction, you may remove that term.  If a license document contains
a further restriction but permits relicensing or conveying under this
License, you may add to a covered work material governed by the terms
of that license document, provided that the further restriction does
not survive such relicensing or conveying.
\

If you add terms to a covered work in accord with this section, you
must place, in the relevant source files, a statement of the
additional terms that apply to those files, or a notice indicating
where to find the applicable terms.
\

Additional terms, permissive or non-permissive, may be stated in the
form of a separately written license, or stated as exceptions;
the above requirements apply either way.
\

8. Termination.
\

You may not propagate or modify a covered work except as expressly
provided under this License.  Any attempt otherwise to propagate or
modify it is void, and will automatically terminate your rights under
this License (including any patent licenses granted under the third
	      paragraph of section 11).
\

However, if you cease all violation of this License, then your
license from a particular copyright holder is reinstated (a)
provisionally, unless and until the copyright holder explicitly and
finally terminates your license, and (b) permanently, if the copyright
holder fails to notify you of the violation by some reasonable means
prior to 60 days after the cessation.
\

Moreover, your license from a particular copyright holder is
reinstated permanently if the copyright holder notifies you of the
violation by some reasonable means, this is the first time you have
received notice of violation of this License (for any work) from that
copyright holder, and you cure the violation prior to 30 days after
your receipt of the notice.
\

Termination of your rights under this section does not terminate the
licenses of parties who have received copies or rights from you under
this License.  If your rights have been terminated and not permanently
reinstated, you do not qualify to receive new licenses for the same
material under section 10.
\

9. Acceptance Not Required for Having Copies.
\

You are not required to accept this License in order to receive or
run a copy of the Program.  Ancillary propagation of a covered work
occurring solely as a consequence of using peer-to-peer transmission
to receive a copy likewise does not require acceptance.  However,
nothing other than this License grants you permission to propagate or
modify any covered work.  These actions infringe copyright if you do
not accept this License.  Therefore, by modifying or propagating a
covered work, you indicate your acceptance of this License to do so.
\

10. Automatic Licensing of Downstream Recipients.

Each time you convey a covered work, the recipient automatically
receives a license from the original licensors, to run, modify and
propagate that work, subject to this License.  You are not responsible
for enforcing compliance by third parties with this License.
\

An \"entity transaction\" is a transaction transferring control of an
organization, or substantially all assets of one, or subdividing an
organization, or merging organizations.  If propagation of a covered
work results from an entity transaction, each party to that
transaction who receives a copy of the work also receives whatever
licenses to the work the party's predecessor in interest had or could
give under the previous paragraph, plus a right to possession of the
Corresponding Source of the work from the predecessor in interest, if
the predecessor has it or can get it with reasonable efforts.
\

You may not impose any further restrictions on the exercise of the
rights granted or affirmed under this License.  For example, you may
not impose a license fee, royalty, or other charge for exercise of
rights granted under this License, and you may not initiate litigation
(including a cross-claim or counterclaim in a lawsuit) alleging that
any patent claim is infringed by making, using, selling, offering for
sale, or importing the Program or any portion of it.
\

11. Patents.
\

A \"contributor\" is a copyright holder who authorizes use under this
License of the Program or a work on which the Program is based.  The
work thus licensed is called the contributor's \"contributor version\".
\

A contributor's \"essential patent claims\" are all patent claims
owned or controlled by the contributor, whether already acquired or
hereafter acquired, that would be infringed by some manner, permitted
by this License, of making, using, or selling its contributor version,
but do not include claims that would be infringed only as a
consequence of further modification of the contributor version.  For
purposes of this definition, \"control\" includes the right to grant
patent sublicenses in a manner consistent with the requirements of
this License.
\

Each contributor grants you a non-exclusive, worldwide, royalty-free
patent license under the contributor's essential patent claims, to
make, use, sell, offer for sale, import and otherwise run, modify and
propagate the contents of its contributor version.
\

In the following three paragraphs, a \"patent license\" is any express
agreement or commitment, however denominated, not to enforce a patent
(such as an express permission to practice a patent or covenant not to
 sue for patent infringement).  To \"grant\" such a patent license to a
party means to make such an agreement or commitment not to enforce a
patent against the party.
\

If you convey a covered work, knowingly relying on a patent license,
and the Corresponding Source of the work is not available for anyone
to copy, free of charge and under the terms of this License, through a
publicly available network server or other readily accessible means,
then you must either (1) cause the Corresponding Source to be so
available, or (2) arrange to deprive yourself of the benefit of the
patent license for this particular work, or (3) arrange, in a manner
consistent with the requirements of this License, to extend the patent
license to downstream recipients.  \"Knowingly relying\" means you have
actual knowledge that, but for the patent license, your conveying the
covered work in a country, or your recipient's use of the covered work
in a country, would infringe one or more identifiable patents in that
country that you have reason to believe are valid.
\

If, pursuant to or in connection with a single transaction or
arrangement, you convey, or propagate by procuring conveyance of, a
covered work, and grant a patent license to some of the parties
receiving the covered work authorizing them to use, propagate, modify
or convey a specific copy of the covered work, then the patent license
you grant is automatically extended to all recipients of the covered
work and works based on it.
\

A patent license is \"discriminatory\" if it does not include within
the scope of its coverage, prohibits the exercise of, or is
conditioned on the non-exercise of one or more of the rights that are
specifically granted under this License.  You may not convey a covered
work if you are a party to an arrangement with a third party that is
in the business of distributing software, under which you make payment
to the third party based on the extent of your activity of conveying
the work, and under which the third party grants, to any of the
parties who would receive the covered work from you, a discriminatory
patent license (a) in connection with copies of the covered work
conveyed by you (or copies made from those copies), or (b) primarily
for and in connection with specific products or compilations that
contain the covered work, unless you entered into that arrangement,
or that patent license was granted, prior to 28 March 2007.
\

Nothing in this License shall be construed as excluding or limiting
any implied license or other defenses to infringement that may
otherwise be available to you under applicable patent law.
\

12. No Surrender of Others' Freedom.
\

If conditions are imposed on you (whether by court order, agreement or
				  otherwise) that contradict the conditions of this License, they do not
excuse you from the conditions of this License.  If you cannot convey a
covered work so as to satisfy simultaneously your obligations under this
License and any other pertinent obligations, then as a consequence you may
not convey it at all.  For example, if you agree to terms that obligate you
to collect a royalty for further conveying from those to whom you convey
the Program, the only way you could satisfy both those terms and this
License would be to refrain entirely from conveying the Program.
\

13. Use with the GNU Affero General Public License.
\

Notwithstanding any other provision of this License, you have
permission to link or combine any covered work with a work licensed
under version 3 of the GNU Affero General Public License into a single
combined work, and to convey the resulting work.  The terms of this
License will continue to apply to the part which is the covered work,
but the special requirements of the GNU Affero General Public License,
section 13, concerning interaction through a network will apply to the
combination as such.
\

14. Revised Versions of this License.
\

The Free Software Foundation may publish revised and/or new versions of
the GNU General Public License from time to time.  Such new versions will
be similar in spirit to the present version, but may differ in detail to
address new problems or concerns.
\

Each version is given a distinguishing version number.  If the
Program specifies that a certain numbered version of the GNU General
Public License \"or any later version\" applies to it, you have the
option of following the terms and conditions either of that numbered
version or of any later version published by the Free Software
Foundation.  If the Program does not specify a version number of the
GNU General Public License, you may choose any version ever published
by the Free Software Foundation.
\

If the Program specifies that a proxy can decide which future
versions of the GNU General Public License can be used, that proxy's
public statement of acceptance of a version permanently authorizes you
to choose that version for the Program.
\

Later license versions may give you additional or different
permissions.  However, no additional obligations are imposed on any
author or copyright holder as a result of your choosing to follow a
later version.
\

15. Disclaimer of Warranty.
\

THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
\

16. Limitation of Liability.
\

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
				     DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
				     PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
\

17. Interpretation of Sections 15 and 16.
\

If the disclaimer of warranty and limitation of liability provided
above cannot be given local legal effect according to their terms,
reviewing courts shall apply local law that most closely approximates
an absolute waiver of all civil liability in connection with the
Program, unless a warranty or assumption of liability accompanies a
copy of the Program in return for a fee.
"]
}

proc CountLines {s} {
    if { [string length $s] < 1} {return 0;}
    return [llength [split $s "\n"]];
}

# Put the entries in the listbox into storage
# and write them to a file. Insert the name of the file into the
# entry field.
proc SaveMultigraphs {} {
    set mf [tk_getSaveFile -initialfile [_ "MultigraphDefinitions"]];
    if {[string equal $mf ""]} {
	ShowMessage [_ "File selection aborted."]
	return
    }
    set mf [MinimizeFileName $mf]
    if {[catch {open $mf "w"} mfh ] != 0} {
	ShowMessage [format [_ "Unable to open file %s."] $mf];
	return ;
    }
    fconfigure $mfh -encoding utf-8
    set contents [string trimright [.mlgd.tf.txt get 1.0 end]]
    puts $mfh $contents;
    close $mfh;
    set ::MultigraphDefinitions $contents;
    .gen.mlg.bot.ent delete 0 end;
    .gen.mlg.bot.ent insert 0 $mf;
    ShowMessage [format [_ "%d multigraphs defined"] [CountLines $contents]];
    destroy .mlgd;
}

proc EnterMultigraphDefinitions {} {
    if {[winfo exists .mlgd]} {
	raise .mlgd;
	return
    }
    toplevel .mlgd -borderwidth 4 -relief raised
    wm title .mlgd "";
    after idle {
	update idletasks
	set xmax [winfo screenwidth .mlgd]
	set ymax [winfo screenheight .mlgd]
	set x0 [expr 1 * ($xmax -[winfo reqwidth .mlgd])/3];
	set y0 [expr 1 * ($ymax -[winfo reqheight .mlgd])/3];
	wm geometry .mlgd "+$x0+$y0";
    }
    label .mlgd.title -text [_ "Define Multigraphs"];
    frame .mlgd.tf
    text  .mlgd.tf.txt -width 20 -height 15 -yscrollcommand {.mlgd.tf.sbar set}\
	-bg $::ColorSpecs(UserTextEntry,Background) \
	-fg $::ColorSpecs(UserTextEntry,Foreground) \
	-font UserTechnicalTextFont
    scrollbar .mlgd.tf.sbar -command {.mlgd.tf.txt yview} -troughcolor $::ColorSpecs(UserTextEntry,Background)\
	-bg $::ColorSpecs(UserTextEntry,Background) -activebackground $::ColorSpecs(UserTextEntry,Background);
    InsertStoredMultigraphsIntoTextWidget
    pack .mlgd.tf.txt  -side left  -expand 1 -fill both
    pack .mlgd.tf.sbar -side right -expand 1 -fill y
    frame .mlgd.finalbs; #Buttons for finalizing - cancel and proceed
    button .mlgd.finalbs.p  -text [_ "Save"]  -command SaveMultigraphs
    button .mlgd.finalbs.c  -text [_ "Cancel"] -command {destroy .mlgd} -anchor w
    pack .mlgd.finalbs.c -side left  -expand 1 -anchor w 
    pack .mlgd.finalbs.p -side right -expand 1 -anchor e 

    #Put it all together
    pack .mlgd.title   -side top -anchor w -expand 1
    pack .mlgd.finalbs -side bottom -expand 1 -fill x
    frame .mlgd.fbsep -height 6 -borderwidth 0 -relief flat
    pack .mlgd.fbsep   -side bottom -expand 1 -fill x
    pack .mlgd.tf  -side top -expand 1 -fill both

    focus .mlgd.tf.txt
    BindKeys .mlgd;
    bind .mlgd <FocusIn> {SetInsertionTargets .mlgd.tf.txt}
    bind .mlgd.tf.txt <Control-v> {InsertClipboardContents %W}
    balloonhelp_for .mlgd.tf.txt "Enter one multigraph per line.";
}

proc InsertStoredMultigraphsIntoTextWidget {} {
    .mlgd.tf.txt insert 1.0 $::MultigraphDefinitions;
}

proc ToggleMultigraphEntryActive {n e o} {
    if {$::DefineMultigraphsP} {
	.gen.mlg.bot.b1 configure -state normal
	.gen.mlg.bot.b2 configure -state normal
	.gen.mlg.bot.ent configure -state normal
    } else {
	.gen.mlg.bot.b1 configure -state disabled
	.gen.mlg.bot.b2 configure -state disabled
	.gen.mlg.bot.ent configure -state disabled
    }
}

proc ToggleMaximumDistanceActive {n e o} {
    if {$::HowManyDiffs == 2} {
	.gen.ctr.mds.spb configure -state normal
    } else {
	.gen.ctr.mds.spb configure -state disabled
    }
}

proc SpecifyMultigraphFile {} {
    set MultigraphFile [tk_getOpenFile];
    if {$MultigraphFile == ""} {
	ShowMessage [_ "File selection aborted."]
	return
    } else { 
	.gen.mlg.bot.ent delete 0 end
	.gen.mlg.bot.ent insert insert [MinimizeFileName $MultigraphFile]
    }
}

proc ActivatePerUseMinpair {n e o} {
    if {$::UseMinpairP} {
	.menubar.file entryconfigure 1 -state normal
	.menubar.file entryconfigure 2 -state normal
	.gen.ctr.trans.lab configure -state normal
	.gen.ctr.trans.rdb1 configure -state normal
	.gen.ctr.trans.rdb2 configure -state normal
	.gen.ctr.idd.lab configure -state normal
	.gen.ctr.idd.rdb1 configure -state normal
	.gen.ctr.idd.rdb2 configure -state normal
	.gen.ctr.onetwo.lab configure -state disabled
	.gen.ctr.onetwo.rdb1 configure -state disabled
	.gen.ctr.onetwo.rdb2 configure -state disabled
	set ::HowManyDiffs 1
    } else {
	.menubar.file entryconfigure 1 -state disabled
	.menubar.file entryconfigure 2 -state disabled
	.gen.ctr.trans.lab configure -state disabled
	.gen.ctr.trans.rdb1 configure -state disabled
	.gen.ctr.trans.rdb2 configure -state disabled
	.gen.ctr.idd.lab configure -state disabled
	.gen.ctr.idd.rdb1 configure -state disabled
	.gen.ctr.idd.rdb2 configure -state disabled
	.gen.ctr.onetwo.lab configure -state normal
	.gen.ctr.onetwo.rdb1 configure -state normal
	.gen.ctr.onetwo.rdb2 configure -state normal
    }
}

proc PackMessageRegion {} {
    pack $::MSG -side left -expand 1 -fill both;
}

proc PackProgressBar {} {
    pack $::PBAR -side right -expand 1 -fill x;
}

# The following sets up a progress bar.
option add *Progress.undoneForeground black  widgetDefault
option add *Progress.undoneBackground $ColorSpecs(Messages,Background)  widgetDefault
option add *Progress.doneForeground   white  widgetDefault
option add *Progress.doneBackground   \#EFAC49 widgetDefault
option add *Progress.borderWidth      3      widgetDefault
option add *Progress$.relief           sunken widgetDefault

namespace eval dkfprogress {
    namespace export Progress SetProgress

    proc Progress {w wid args} {
	uplevel 1 [list frame $w -class Progress] $args

	foreach {val} {
	    undoneForeground doneForeground
	    undoneBackground doneBackground
	} {
	    set class [string toupper [string index $val 0]\
		    ][string range $val 1 end]
	    set $val [option get $w $val $class]
	}

	set varname [namespace current]::progressPercent($w)
	frame $w.l -borderwidth 0 -background $undoneBackground -width $wid
	label $w.l.l -textvariable $varname -borderwidth 0 \
		-foreground $undoneForeground -background $undoneBackground
	$w.l configure -height [expr {int([winfo reqheight $w.l.l]+2)}]
	frame $w.l.fill -background $doneBackground
	label $w.l.fill.l -textvariable $varname -borderwidth 0 \
		-foreground $doneForeground -background $doneBackground
	bind $w.l <Configure> [namespace code [list ProgressConf $w "%w"]]

	pack $w.l -fill both -expand 1
	place $w.l.l -relx 0.5 -rely 0.5 -anchor center
	place $w.l.fill -x 0 -y 0 -relheight 1 -relwidth 0
	place $w.l.fill.l -x 0 -rely 0.5 -anchor center
	SetProgress $w 0
	return $w
    }

    proc ProgressConf {w width} {
	place conf $w.l.fill.l -x [expr {int($width/2)}]
    }

    proc ColorSequence {n} {
	if {$n < 0} {set n 0} elseif {$n>100} {set n 100}
	set green 0; 
	set red 0;
	set blue 0;
	set green  [expr int(2.55 *(100 - $n))];
	set red [expr  int(2.55 * $n)];
	return [format "#%02x%02x%02x" $red $green $blue];
    }

    proc SetProgress {win value {range 100}} {
	set progress [expr int((100.0 * $value)/double($range))];
	set relwidth [expr $value/double($range)]
	variable progressPercent
	place conf $win.l.fill -relwidth $relwidth
	$win.l.fill configure -bg [ColorSequence $progress];
	set progressPercent($win) "${progress}%"
    }
}
namespace import dkfprogress::Progress dkfprogress::SetProgress


proc ChooseColor {} {
    set NewColor [tk_chooseColor];
    option add *Background $NewColor;
}


proc ConfigureCtr {} {
    set iddht  [lindex  [grid bbox .gen.ctr 0 0] 3]
    set transht [lindex [grid bbox .gen.ctr 1 0] 3]
    if {$iddht > $transht} {set max $iddht} else {
	set max $transht}
    grid rowconfigure .gen.ctr 0 -minsize $max

    set sciht  [lindex  [grid bbox .gen.ctr 0 1] 3]
    set mwlht  [lindex  [grid bbox .gen.ctr 1 1] 3]
    if {$sciht > $mwlht} {set max $sciht} else {
	set max $mwlht}
    grid rowconfigure .gen.ctr 1 -minsize $max

    set srtht [lindex [grid bbox .gen.ctr 0 2] 3]
    set dsmht [lindex [grid bbox .gen.ctr 1 2] 3]
    if {$srtht > $dsmht} {set max $srtht} else {
	set max $dsmht}
    grid rowconfigure .gen.ctr 2 -minsize $max
}


#Returns a list of lists, where each entry is a list of the word and a possibly null gloss
#Multigraph mapping is done in passing
proc ReadWordList {fn MultigraphMap} {
    if { [catch {open $fn "r"} fhd ] != 0} {
	ShowMessage [format [_ "Unable to open wordlist %s."] $fn];
	return ;
    }
    fconfigure $fhd -encoding utf-8
    set WordList [list]
    if {[llength $MultigraphMap] > 0} {
	set DoMultigraphsP 1
    } else {set DoMultigraphsP 0}
    while { [gets $fhd line] > 0} {
	set LineList [split $line]
	set word [lindex $LineList 0]
	if {[llength $LineList] > 1} {
	    set gloss [lindex $LineList 1]
	} else {
	    set gloss ""
	}
	if {[string length $word] <= $::MaxWordLength} {
	    if {$DoMultigraphsP} {
		set word [string map $MultigraphMap $word]
	    }
	    lappend WordList [list $word $gloss]
	}
    }
    close $fhd;
    return $WordList
}

#Compare first by string length (longer first), then lexicographically
proc lccomp {a b} {
    set LenA [string length $a]
    set LenB [string length $b]
    set cmp  [expr $LenB - $LenA]
    if {$cmp != 0} {
	return $cmp;
    } else {
	return [string compare $a $b];
    }
}

proc LoadMultigraphs {fn} {
    if {[string length $fn] == 0} return [list]
    if { [catch {open $fn "r"} fhd ] != 0} {
	ShowMessage [format [_ "Unable to open multigraph definitions %s."] $fn]
	return [list]
    }
    fconfigure $fhd -encoding utf-8
    set NextCode 0xd800; #Use high surrogate range
    while { [gets $fhd line] > 0} {
	foreach mg [split $line] {
	    set mgarray($mg) [format "%c" $NextCode]
	    incr NextCode
	}
    }
    close $fhd;
    set mglist [lsort -command lccomp [array names mgarray]]
    set mgm [list]
    foreach m $mglist {
	lappend mgm $m
	lappend mgm $mgarray($m)
    }
    return $mgm
}

proc InvertMap {map} {
    if {[llength $map] == 0} {return [list]}
    foreach {key value} $map {
	set ValueArray($value) $key;
    }
    set ValueList [array names ValueArray]
    foreach v $ValueList {
	lappend imgm $v
	lappend imgm $ValueArray($v)
    }
    return $imgm
} 

proc ExpandMultigraphs {w imgm} {
    return [string map $imgm $w]
}

#Returns a list of word pair indices each preceded by the pair of letters in which they differ
proc FindPairs {wl N MaxDistance} {
    set WordCnt [llength $wl];
    for {set i 0} {$i < $WordCnt} {incr i} {
	for {set j $i} {$j < $WordCnt} {incr j} {
	    set A [lindex [lindex $wl $i] 0]
	    set B [lindex [lindex $wl $j] 0]

	    set DeltaIndex [CheckPair $A $B $N $MaxDistance]
	    if {$DeltaIndex >= 0} {
		set ADiffLetter [string index $A $DeltaIndex]
		set BDiffLetter [string index $B $DeltaIndex]
		if {[string compare $ADiffLetter $BDiffLetter]} {
		    lappend PairList [list $ADiffLetter $BDiffLetter $i $j]
		} else {
		    lappend PairList [list $BDiffLetter $ADiffLetter $i $j]
		}
	    }
	}
    }
    return $PairList
}

#Returns -1 if the words are not a minimal pair.
#Returns the index of the character at which the words differ if they are a minimal pair.
proc CheckPair {a b N MaxDistance} {
    set lenA [string length $a]
    set lenB [string length $b]
    if {$lenA != $lenB} {return -1}
    set delta 0;#Number of differences detected so far
    set FirstDifferenceIndex -1;#Index of character at which first difference is located
    set SecondDifferenceIndex -1;#Index of character at which second difference is located
    for {set i 0} {$i < $lenA} {incr i} {
	if {[string index $a $i] != [string index $b $i]} {
	    incr delta
	    if {$delta > $N} {return -1}
	    if {$delta == 1} {set FirstDifferenceIndex $i}
	    if {$delta == 2} {set SecondDifferenceIndex $i}
	}
    }
    if {$delta < $N} {return -1}
    if {$SecondDifferenceIndex - $FirstDifferenceIndex > $MaxDistance} {
	return -1;
    }
    return $FirstDifferenceIndex
}

proc FindMinimalPairs {WordFileName  OutFileName Diffs MaxDistance {MultigraphFileName ""}} {
    set MultigraphMap [LoadMultigraphs $MultigraphFileName]
    set InverseMultigraphMap [InvertMap $MultigraphMap]
    set ItemList [ReadWordList $WordFileName $MultigraphMap]
    set WordCnt [llength $ItemList]
    if {$WordCnt < 2} {
	ShowMessage [_ "Nothing can be done with fewer than two words."]
	return 0;
    }
    if {[catch {open $OutFileName "w"} OutFileHandle ] != 0} {
	ShowMessage [format [_ "Unable to open file %s."] $OutFileName];
	return 0;
    }
    fconfigure $OutFileHandle -encoding utf-8
    set PairList [FindPairs $ItemList $Diffs $MaxDistance]
    #Each entry consists of: diffletterA diffletter B indexA indexB
    foreach wp $PairList {
	set EntryA [lindex $ItemList [lindex $wp 2]]
	set EntryB [lindex $ItemList [lindex $wp 3]]
	set DiffLetterA [ExpandMultigraphs [lindex $wp 0] $InverseMultigraphMap]
	set DiffLetterB [ExpandMultigraphs [lindex $wp 1] $InverseMultigraphMap]
	set WordA  [ExpandMultigraphs [lindex $EntryA 0] $InverseMultigraphMap]
	set WordB  [ExpandMultigraphs [lindex $EntryB 0] $InverseMultigraphMap]
	if {!$::DoSingleP} {
	    set GlossA [ExpandMultigraphs [lindex $EntryA 1] $InverseMultigraphMap]
	    set GlossB [ExpandMultigraphs [lindex $EntryB 1] $InverseMultigraphMap]
	    lappend OutStrings [format "%s\t%s\t%s\t%s\t%s\t%s" \
				    $DiffLetterA $DiffLetterB \
				    $WordA $GlossA \
				    $WordB $GlossB] 
	} else {
	    lappend OutStrings [format "%s %s\t%s\t%s" \
				    $DiffLetterA $DiffLetterB $WordA $WordB]
	}
    }
    foreach l [lsort $OutStrings] {
	puts $OutFileHandle $l;
    }
    close $OutFileHandle
    return [llength $PairList]
}


#Initialization file stuff

proc SetBalloonHelpShow {b} {
    set v  [Boolean $b]
    set ::BalloonHelpP $v;
    balloonhelp_control $v;
}

proc SelectBrowser {s} {
    set ::DefaultBrowser $s;
}

proc SetDoIndels {b} {
    set ::DoIndelsP [Boolean $b];
}

proc SetDoTranspositions {b} {
    set ::DoTranspositionsP [Boolean $b];
}

proc SetSingleColumnInput {b} {
    set ::DoSingleP [Boolean $b];
}

proc SetSortOutput {b} {
    set ::SortOutputP [Boolean $b];
}

proc SetDefaultInputFile {s} {
    set ::DefaultInputFile $s;
}

proc SetDefaultOutputFile {s} {
    set ::DefaultOutputFile $s;
}

proc SetDefaultMultigraphFile {s} {
    set ::DefaultMultigraphFile $s;
}

proc SetDefaultDifferenceSiteMarker {s} {
    if {[string length $s] != 1} {
	puts [_ "The difference site marker must be a single character "]
	return ;
    }
    set ::DefaultDifferenceSiteMarker $s;
}

proc SetMaximumWordLength {s} {
    if {[catch {expr round($s)} n] !=0} {
	puts [format [_ "%s is not a valid number."] $s];
	return ;
    }
    if {($n < 10) || ($n > 50)} {
	puts [format [_ "%s is not a reasonable word length."] $s];
	puts [_ "Word length should be between 10 and 50."];
	return ;
    }
    set ::DefaultMaxWordLength $n;
}

set CustomCharacterChartFileNameList "";
proc StoreCustomCharacterChartFileName {fn} {
    lappend ::CustomCharacterChartFileNameList $fn;
}

set CustomCharacterChartDataList "";
proc StoreCustomCharacterChartInPlace {dl} {
    lappend ::CustomCharacterChartDataList $dl;
}

proc DefineColorSettingProcs {} {
    foreach cs [array names ::ColorSpecs] {
	set csl [split $cs ","]
	set obj   [lindex $csl 0]
	set which [lindex $csl 1]
	eval [list "proc"  \
		  [format "Set%s%sColor" $obj $which] \
		  "\{cs\}" \
		  "set ::ColorSpecs($obj,$which) \$cs"]
    }
}

proc AliasColorSettings {interp} {
    foreach cs [array names ::ColorSpecs] {
	set csl [split $cs ","]
	set obj   [lindex $csl 0]
	set which [lindex $csl 1]
	eval [split [format "%s alias %s%sColor Set%s%sColor" $interp $obj $which $obj $which]]
    }
}

#Returns a list of init file commands representing the current color settings.
proc SaveColorSettings {} {
    set cl [list]
    foreach cs [array names ::ColorSpecs] {
	set csl [split $cs ","]
	set obj   [lindex $csl 0]
	set which [lindex $csl 1]
	set ps $::ColorSpecs($cs)
	if {[string index $ps 0] == "\#"} {
	    set ps \\$::ColorSpecs($cs)
	} else {
	    set ps [list $ps];
	}
	lappend cl [format "%s%sColor %s" $obj $which $ps]
    }
    return $cl;
}

#Check whether the argument represents a Boolean value
#and interpret it if it is.
proc Boolean {s} {
     switch -regexp $s {
	 1	{return 1}
	 T.*	{return 1}
	 t.*	{return 1}
	 Y.*	{return 1}
	 y.*	{return 1}
	 ok	{return 1}
	 on	{return 1}
	 0 	{return 0}
	 F.*	{return 0}
	 f.*	{return 0}
	 N.*	{return 0}
	 n.*	{return 0}
	 off	{return 0}
	 default {error}
     }
}

proc GetInitCommands {} {
    # We want a fresh interpreter so that we don't
    # list any commands defined in a user's init file.
    if { [interp exists init]} {
	interp delete init
    }
    InitFileSetup;
    set n 0
    #We rely on the fact that all builtin commands have lower-case
    #case names to separate the commands we provide from the builtins.
    return [interp eval init {lsort [info commands \[A-Z\]*]}]
}

proc GetInitTclCommands {} {
    # We want a fresh interpreter so that we don't
    # list any commands defined in a user's init file.
    if { [interp exists init]} {
	interp delete init
    }
    interp create -safe -- init
    set n 0
    #We rely on the fact that all builtin commands have lower-case
    #case names to separate the commands we provide from the builtins.
    return [interp eval init {lsort [info commands \[a-z\]*]}]
}

proc SaveInitCommands {} {
    set SaveFile [tk_getSaveFile -initialfile [_ "InitFileCommands"]];
    if {[string equal $SaveFile ""]} {
	return ;
    } else {
        if { [catch {open $SaveFile "w+"} fh] != 0} {
	    ShowMessage [format \
		     [_ "Unable to open file %s in which to save class definitions."] \
		     $SaveFile];
	    return ;
        }
    }
    set n 0;
    foreach l [GetInitCommands] {
	incr n;
	puts $fh [format "\[%02d\] %s" $n $l]
    }
    close $fh;
}

proc PopupInitCommandList {} {
    global HPLines;
    if {[PopupDown InitCommandList] ==1} {return}
    set po  [CreateTextDisplay [_ "Initialization File Commands"] 65 $HPLines];
    set ::PopupList(InitCommandList) $po;
    BindKeys $po;
    set n 0;
    foreach l [GetInitCommands] {
	incr n;
	AppendToTextDisplay $po [format "\[%02d\] %s\n" $n $l];
    }
}

proc SaveInitTclCommands {} {
    set SaveFile [tk_getSaveFile -initialfile [_ "InitFileTclCommands"]];
    if {[string equal $SaveFile ""]} {
	return ;
    } else {
        if { [catch {open $SaveFile "w+"} fh] != 0} {
	    ShowMessage [format \
		     [_ "Unable to open file %s in which to save Tcl commands."] \
		     $SaveFile];
	    return ;
        }
    }
    set n 0;
    foreach l [GetInitTclCommands] {
	incr n;
	puts $fh [format "\[%03d\] %s" $n $l]
    }
    close $fh;
}

proc PopupInitTclCommandList {} {
    if {[PopupDown InitTclCommandList] ==1} {return}
    set po  [CreateTextDisplay [_ "Initialization File Tcl Commands"] 65 10];
    set ::PopupList(InitTclCommandList) $po;
    BindKeys $po;
    set n 0;
    foreach l [GetInitTclCommands] {
	incr n;
	AppendToTextDisplay $po [format "\[%03d\] %s\n" $n $l];
    }
}

#Write out a list of init file commands that will reconstruct
#the current configuration 
proc SaveState {{SaveFile Ask}} {
    if {[string equal $SaveFile Ask]} { 
	set SaveFile [tk_getSaveFile -initialfile [_ "SavedState"]];
	if {$SaveFile == ""} {
	    ShowMessage [_ "File selection cancelled."]
	    return ;
	}
    }
    if {[catch {open $SaveFile "w"} fh ] != 0} {
	ShowMessage [format [_ "Unable to open file %s."] $SaveFile];
	return ;
    }
    set cl [concat \
	[fontsel::SaveFontSettings] \
	[SaveColorSettings] \
	[SaveMiscellaneousSettings]
	   ];
    foreach x $cl {
	puts $fh $x;
    }
    close $fh;
}

proc InitFileSetup {} {
    interp create -safe -- init
    fontsel::AliasFontSettings init
    AliasColorSettings init
    init alias BalloonHelpShowP SetBalloonHelpShow
    init alias Browser SelectBrowser
    init alias DefaultInputFile SetDefaultInputFile
    init alias DefaultMultigraphFile SetDefaultMultigraphFile
    init alias DefaultOutputFile SetDefaultOutputFile
    init alias DefineCharacterChart StoreCustomCharacterChartInPlace
    init alias DifferenceSiteMarker SetDefaultDifferenceSiteMarker
    init alias DisplayConsonantChartColumnLabelsP ipaentry::SetDisplayConsonantChartColumnLabelsP
    init alias DisplayConsonantChartRowLabelsP ipaentry::SetDisplayConsonantChartRowLabelsP
    init alias DisplayVowelChartColumnLabelsP ipaentry::SetDisplayVowelChartColumnLabelsP
    init alias DisplayVowelChartRowLabelsP ipaentry::SetDisplayVowelChartRowLabelsP
    init alias DoIndelsP SetDoIndels
    init alias DoTranspositionsP SetDoTranspositions
    init alias MaximumWordLength SetMaximumWordLength
    init alias ReadCharacterChart StoreCustomCharacterChartFileName
    init alias SingleColumnInputP SetSingleColumnInput
    init alias SortOutputP SetSortOutput
}


proc SaveMiscellaneousSettings {} {
    set MiscellaneousInitCommandList [list \
	BalloonHelpShowP $::BalloonHelpP \
	Browser $::DefaultBrowser \
	DefaultInputFile $::DefaultInputFile \
	DefaultMultigraphFile $::DefaultMultigraphFile \
	DefaultOutputFile $::DefaultOutputFile \
	DifferenceSiteMarker $::DefaultDifferenceSiteMarker \
	DisplayConsonantChartColumnLabelsP [ipaentry::SetDisplayConsonantChartColumnLabelsP] \
        DisplayConsonantChartRowLabelsP	 [ipaentry::SetDisplayConsonantChartRowLabelsP] \
        DisplayVowelChartColumnLabelsP	 [ipaentry::SetDisplayVowelChartColumnLabelsP] \
        DisplayVowelChartRowLabelsP	 [ipaentry::SetDisplayVowelChartRowLabelsP] \
	DoIndelsP $::DoIndelsP \
	DoTranspositionsP $::DoTranspositionsP \
	MaximumWordLength $::DefaultMaxWordLength\
	SingleColumnInputP $::DoSingleP \
        SortOutputP $::SortOutputP];
    set cl [list]
    foreach {command value} $MiscellaneousInitCommandList {
	lappend cl [format "%s %s" $command $value]
    }
    foreach title [array names ::SpecialCharacterPalette] {
	set info $::SpecialCharacterPalette($title);
	if {[llength $info] > 2} {
	    lappend cl [list ReadCharacterChart [lindex $info 2]];
	} else {
	    lappend cl [ReconstructCharacterChartDefinition $title];
	}
    }
    return $cl;
}

proc ReconstructCharacterChartDefinition {Title} {
    set info $::SpecialCharacterPalette($Title);
    set txt DefineCharacterChart;
    append txt " \{\n"
    append txt [format "%s|%s\n" [lindex $info 1] [Quote $Title]];
    #Each entry consists of a unicode string and a gloss
    foreach x [lindex $info 0] {
	set string [lindex $x 0]
	set gloss  [lindex $x 1]
	append txt [format "%s|%s\n" [UnicodeStringToCtr $string] [Quote $gloss]];
    }
    append txt "\}"
    return  $txt;
}

#This is a kludge but I encountered quoting hell trying to use (\\s) -> \\ \\1
#e.g.   regsub -all "\\s" $s {\ } r

#Escape spaces and tabs
proc Quote {s} {
    regsub -all "\u0020" $s {\ } r
    regsub -all "\t" $r {\	} ret
    return $ret
}

proc UnicodeStringToCtr {s} {
    set rv "";
    foreach c [split $s ""] {
	scan $c "%c" t
	if {$t <= 0xFF} {
	    append rv $c;
	} else {
	    append rv [format "\\u%04X" $t];
	}
    }
    return $rv;
 }

proc SlaveBoolean {s} {
    switch -regexp $s {
	1	{return 1}
	T.*	{return 1}
	t.*	{return 1}
	Y.*	{return 1}
	y.*	{return 1}
	ok	{return 1}
	on	{return 1}
	0 	{return 0}
	F.*	{return 0}
	f.*	{return 0}
	N.*	{return 0}
	n.*	{return 0}
	off	{return 0}
	default {
	    puts [format [msgcat::mc "%s is not a Boolean argument."] $s]
	    return -1; 
	}
    }
}

proc ReadInitFile {name} {
    if { [catch {interp invoke init source $name} msg] } {
	puts [format [_  "Error on reading init file: %s"] $msg]
    } else {
	ShowMessage [format [_ "Executed init file %s"] $name]
    }
    interp delete init
}

proc Usage {} {
    puts {Usage: msg [options]};
    puts [_ "    -d            set debug flag"];
    puts [_ "    -h            help"];
    puts [_ "    -i            do not read init file"];
    puts [_ "    -v            version"];
}

proc argshift {k} {
    set ::argv [lrange $::argv $k end];
}

#Handle command line arguments
while {[string match -* [lindex $argv 0]]} {
    switch -glob -- [lindex $argv 0] {
	-d* {
	    set DebugP 1;
	    argshift 1;
	}
	-h* {
	    Usage;
	    exit 0;
	}
	-i* {
	    set ReadInitFileP 0;
	    argshift 1;
	}
	-v* {
	    puts "mpg $Version";
 	    puts "minpair package version $PackageVersion";
	    exit 0;
	}
	default {
	    puts [format [::msgcat::mc "Command line option %s not recognized."] [lindex $argv 0]];
	    Usage;
	    exit 0;
	}
    }
};

puts "$progname $Version";
puts "Copyright (C) 2004, [lindex [split [lindex [split $::TimeStamp] 0] "-"] 0] William J. Poser.";
set Locale [::msgcat::mclocale];
LoadMessageCatalog;
puts [_ "This program is free software; you can redistribute it
and/or modify it under the terms of version 3 of the GNU General
Public License as published by the Free Software Foundation.\n"];

fontsel::DefineFontSettingProcs
DefineColorSettingProcs
InitFileSetup
wm withdraw [fontsel::CreateFontControlPanel]

if {$ReadInitFileP} {
    #Look first in current directory, then in home directory.
    if { [file exists $InitFile] } {
	ReadInitFile $InitFile;
    } else {
	set cwd [pwd];
	cd;
	if {[file exists $InitFile] } {
	    ReadInitFile $InitFile;
	}
	cd $cwd;
    }
}

fontsel::SetFontSelectionDefaults
fontsel::RecordDefaults

fontsel::SetFontSelectionDefaults
option add *ipaec.Button.font CharacterEntryFont 100
option add *ipaec.Label.background  $ColorSpecs(CharacterEntry,Background) 100
option add *ipaec.Button.background $ColorSpecs(CharacterEntry,Background) 100
option add *ipaec.Label.relief raised
option add *Button.foreground $ColorSpecs(CharacterEntry,Foreground) 100
#option add *Button.foreground blue 100

#option add *CharEntry*Button.Foreground $ColorSpecs(CharacterEntry,Foreground) 100

option add *ipaev.Button.font CharacterEntryFont 100
option add *ipaev.Label.background  $ColorSpecs(CharacterEntry,Background)
option add *ipaev.Button.background $ColorSpecs(CharacterEntry,Background)
option add *ipaev.Button.foreground $ColorSpecs(CharacterEntry,Foreground)
option add *ipaev.Label.relief raised

option add *ipaea.Button.font AccentedLetterFont 100
option add *ipaea.Label.background  "\#E0E0E0"
option add *ipaea.Button.background "\#FFFFFF"
option add *ipaea.Button.foreground "\#000000"
option add *ipaea.Label.relief raised

option add *ipaed.Button.font DiacriticFont 100
#option add *ipaed.Label.background  $ColorSpecs(CharacterEntry,Background)
option add *ipaed.Button.background "\#E0E0FF"
option add *ipaed.Label.background "\#E0E0E0"
option add *ipaed.Button.foreground "\#000000"
option add *ipaed.Label.relief raised

option add *Background $ColorSpecs(Default,Background) 60;
#option add *Entry.Background "\#d4e4fe" 60;
option add *Entry.Background  "\#ecf0f0" 60;
#option add *DisabledBackground "\#e9c4dc" 60;
option add *DisabledBackground "\#8c74d0" 60;
option add *Button.activeBackground "\#FF0044" 60;
option add *Radiobutton.activeBackground "\#FF0044" 100;
#option add *Radiobutton.selectColor  "\#00AA00" 60;
option add *Radiobutton.selectColor  "\#e1741b" 60;

option add *Menu.activeBackground $ColorSpecs(Menu,ActiveBackground) 100

set MinpairAvailableP [MinpairAvailableQ]
#Construct the user interface
set m [menu .menubar -tearoff 0  -bg $ColorSpecs(Menubar,Background)\
	   -font MainFont\
	   -fg $ColorSpecs(Menubar,Foreground) \
	   -activebackground $ColorSpecs(Menubar,ActiveBackground) \
	   -activeforeground $ColorSpecs(Menubar,Background)]
$m add cascade -label [_ "File"]   -menu [menu $m.file]
$m.file add command -label [_ "Save Command Line"] -command SaveCommandLine
$m.file add command -label [_ "Show Command Line"] -command ShowCommandLine
$m.file add command -label [_ "Quit"]  -command ShutDown
if {!$AquaP} {
    $m add command -label [_ "Execute"]  -command Execute
    if {![string equal $System) MSWindows]} {
	$m add command -label [_ "Abort Minpair"]  -command AbortMinpair
    }
}
set CharacterEntryIndex [mdo 0];
$m add cascade -label [_ "Character Entry"] -menu [menu $m.charentry];

incr CharacterEntryIndex;
set CustomCharacterChartIndex $CharacterEntryIndex;
$m.charentry add command \
-label [_ "Load Custom Character Chart Definition"] -command LoadCustomCharacterChart;

incr CharacterEntryIndex;
set ToggleIPAAIndex $CharacterEntryIndex;
$m.charentry add command -label [_ "Display Accented Letter Chart"] -command ToggleIPAA;

incr CharacterEntryIndex;
set ToggleIPACIndex $CharacterEntryIndex;
$m.charentry add command -label [_ "Display IPA Consonant Chart"] -command ToggleIPAC;

incr CharacterEntryIndex;
set ToggleIPADIndex $CharacterEntryIndex;
$m.charentry add command -label [_ "Display IPA Diacritic Chart"] -command ToggleIPAD;

incr CharacterEntryIndex;
set ToggleIPAVIndex $CharacterEntryIndex;
$m.charentry add command -label [_ "Display IPA Vowel Chart"] -command ToggleIPAV;

incr CharacterEntryIndex;
set ToggleCharEntryByCodeIndex $CharacterEntryIndex;
$m.charentry add command \
-label [_ "Display Widget for Entering Characters by Unicode Code"] -command ToggleCharEntryByCode;


$m add cascade -label [_ "Configure"]   -menu [menu $m.configure];
set ind 0
incr ind;;
$m.configure add command -label [_ "Select Font"] -command fontsel::CreateFontControlPanel;

incr ind;
$m.configure add command -label [_ "Remove Consonant Chart Column Labels"] -command ipaentry::UnpackConsonantColumnLabels;

incr ind;
$m.configure add command -label [_ "Remove Consonant Chart Row Labels"] -command ipaentry::UnpackConsonantRowLabels;

incr ind;
$m.configure add command -label [_ "Remove Vowel Chart Column Labels"] -command ipaentry::UnpackVowelColumnLabels;

incr ind;
$m.configure add command -label [_ "Remove Vowel Chart Row Labels"] -command ipaentry::UnpackVowelRowLabels;

incr ind;
$m.configure add command -label [_ "Save Configuration"] -command {SaveState};

incr ind;
set BalloonHelpIndex $ind;
if {$::BalloonHelpP} {
    set lab [_ "Disable Irritating Balloon Help"] 
} else {
    set lab [_ "Show Balloon Help"]
}
$m.configure add command -label $lab -command ToggleBalloonHelp;

$m add cascade -label [_ "Help"]   -menu [menu $m.help];
set ind [mdo 0];
$m.help add command -label [_ "About"] -command About;
incr ind;
$m.help add command -label [_ "Bug Reports"] -command BugReports;
incr ind;
$m.help add command -label [_ "How To Use This Program"] -command HowTo;
incr ind;
$m.help add cascade -label [_ "Initialization File Commands"]  -menu [menu $m.help.ifc];
$m.help.ifc add command -label [_ "Popup List"] -command PopupInitCommandList;
$m.help.ifc add command -label [_ "Save to File"] -command SaveInitCommands;
incr ind;
$m.help add cascade -label [_ "Initialization File Tcl Commands"]  -menu [menu $m.help.iftc];
$m.help.iftc add command -label [_ "Popup List"] -command PopupInitTclCommandList;
$m.help.iftc add command -label [_ "Save to File"] -command SaveInitTclCommands;
incr ind;
$m.help add command -label [_ "Key Bindings"] -command DescribeKeyBindings;
incr ind;
$m.help add command -label [_ "License"] -command ShowGPL;
incr ind;
. configure -menu .menubar

#wm configure .menubar -maxwidth 600

frame .mf
text .mf.msg -bg $ColorSpecs(Messages,Background) -fg $ColorSpecs(Messages,Foreground) \
    -height 1  -relief flat -font MainFont -exportselection 1 -width 60
balloonhelp_for $::MSG [_ "Messages from the program appear here."]
Progress .mf.pbar 150
PackMessageRegion;

if {$AquaP} {
    frame .cmnds
    button .cmnds.execute -text [_ "Execute"] -command Execute \
	-bg $::ColorSpecs(Menubar,Background) -fg $::ColorSpecs(Menubar,Foreground)
    button .cmnds.abort -text [_ "Abort Minpair"] -command AbortMinpair \
	-bg $::ColorSpecs(Menubar,Background) -fg $::ColorSpecs(Menubar,Foreground)
    pack .cmnds.execute -side left -expand 1 -fill both
    pack .cmnds.abort -side left -expand 1 -fill both
    BindKeys .cmnds
    frame .sepcmnds -height 2 -borderwidth 1 -relief sunken
}


frame .gen -bg $ColorSpecs(MainFrame,Background);
frame .gen.ctr -relief ridge -border 2
#set ctrrelief flat
set ctrrelief solid

frame       .gen.ctr.dsm -relief $ctrrelief -border 1
label       .gen.ctr.dsm.lab  -text [_ "Difference Site Marker"] -anchor w
entry       .gen.ctr.dsm.ent -width 2 -relief flat \
    -foreground $::ColorSpecs(UserTextEntry,Foreground)  -font UserTechnicalTextFont
.gen.ctr.dsm.ent insert 0 $DefaultDifferenceSiteMarker;
bind 	    .gen.ctr.dsm.ent <Control-v> {InsertClipboardContents %W}
bind .gen.ctr.dsm.ent <FocusIn> {SetInsertionTargets .gen.ctr.dsm.ent}
pack .gen.ctr.dsm.lab -side top -expand 1 -fill both  -anchor n
pack .gen.ctr.dsm.ent -side left -expand 1 -fill none -padx 6 -pady {0 8} -anchor w

set dsmbh [_ "The location of the characters that differ\nin the minimal pair is marked by a character\nwhich by default is an underscore. You may\nspecify a different character if you wish."];
balloonhelp_for .gen.ctr.dsm.lab $dsmbh;
balloonhelp_for .gen.ctr.dsm.ent [_ "Enter the character that marks the difference site here."];

#Maximum word length
frame .gen.ctr.mwl -relief $ctrrelief -border 1
label .gen.ctr.mwl.lab -text "Maximum Word Length" -anchor w
spinbox .gen.ctr.mwl.spb \
    -width 3 \
    -from 0 \
    -to 50 \
    -wrap 0
.gen.ctr.mwl.spb set $DefaultMaxWordLength
balloonhelp_for .gen.ctr.mwl.spb [_ "There is usually no point in considering\nlong words since they produce very few\nminimal pairs. Words longer than the\nmaximum set here are ignored."]
pack .gen.ctr.mwl.lab -side top -expand 1 -fill x -anchor n -padx 2
pack .gen.ctr.mwl.spb -side top -expand 1 -fill y -anchor e -pady {15 25} -padx 25

#Maximum distance
frame .gen.ctr.mds -relief $ctrrelief -border 1
label .gen.ctr.mds.lab -text "Maximum Distance" -anchor w
spinbox .gen.ctr.mds.spb \
    -width 3 \
    -from 0 \
    -to 50 \
    -wrap 0
.gen.ctr.mds.spb set $MaximumDistance
balloonhelp_for .gen.ctr.mds [_ "What is the maximum distance permitted between differences?"]
pack .gen.ctr.mds.lab -side top -expand y -fill x -anchor n -padx 2
pack .gen.ctr.mds.spb -side top -expand y -fill y -anchor e -pady {15 25} -padx 25

#Sort output?
frame       .gen.ctr.srt -relief $ctrrelief -border 1
label       .gen.ctr.srt.lab  -text [_ "Sort Output?"]   -anchor w
radiobutton .gen.ctr.srt.rdb1 -variable SortOutputP -value 0 -text [_ "no"]     -anchor w
radiobutton .gen.ctr.srt.rdb2 -variable SortOutputP -value 1 -text [_ "yes"]    -anchor w
pack .gen.ctr.srt.lab -side top -expand y -fill x -anchor n
pack .gen.ctr.srt.rdb1 -side top    -expand 1 -fill y  -anchor w -pady {6 1} -padx 5
pack .gen.ctr.srt.rdb2 -side bottom -expand 1 -fill y  -anchor w -pady {1 6} -padx 5
set bh [_ "Should the list of minimal pairs\nbe sorted so that tokens of the\nsame pair are grouped together?"];
balloonhelp_for .gen.ctr.srt      $bh;
balloonhelp_for .gen.ctr.srt.rdb1 $bh;
balloonhelp_for .gen.ctr.srt.rdb2 $bh;
balloonhelp_for .gen.ctr.srt.lab  $bh;
bind .gen.ctr.srt       <<B3>> ExplainSorting
bind .gen.ctr.srt.lab   <<B3>> ExplainSorting
bind .gen.ctr.srt.rdb1  <<B3>> ExplainSorting
bind .gen.ctr.srt.rdb2  <<B3>> ExplainSorting

#Do Indels?
frame       .gen.ctr.idd -relief $ctrrelief -border 1
label       .gen.ctr.idd.lab  -text [_ "Include Indels?"]   -anchor w
radiobutton .gen.ctr.idd.rdb1 -variable DoIndelsP -value 0 -text [_ "no"]     -anchor w
radiobutton .gen.ctr.idd.rdb2 -variable DoIndelsP -value 1 -text [_ "yes"]    -anchor w
.gen.ctr.idd.rdb1 select
pack .gen.ctr.idd.lab -side top -expand y -fill x -anchor w
pack .gen.ctr.idd.rdb1 .gen.ctr.idd.rdb2 -expand 1 -fill y  -anchor w
set bh [_ "Should words differing only in that\none contains a segment not found in the\nother be included as minimal pairs?"];
balloonhelp_for .gen.ctr.idd      $bh;
balloonhelp_for .gen.ctr.idd.rdb1 $bh;balloonhelpd_for .gen.ctr.idd.rdb2 $bh;
balloonhelp_for .gen.ctr.idd.lab  $bh;
bind .gen.ctr.idd       <<B3>> ExplainIndels;
bind .gen.ctr.idd.lab   <<B3>> ExplainIndels;
bind .gen.ctr.idd.rdb1  <<B3>> ExplainIndels;
bind .gen.ctr.idd.rdb2  <<B3>> ExplainIndels;
if {!$MinpairAvailableP} {
    .gen.ctr.idd.rdb1 configure -state disabled;
    .gen.ctr.idd.rdb2 configure -state disabled;
}

#Do Transpositions?
frame       .gen.ctr.trans -relief $ctrrelief -border 1
label       .gen.ctr.trans.lab  -text [_ "Include Transpositions?"]   -anchor w
radiobutton .gen.ctr.trans.rdb1 -variable DoTranspositionsP -value 0 -text [_ "no"]     -anchor w
radiobutton .gen.ctr.trans.rdb2 -variable DoTranspositionsP -value 1 -text [_ "yes"]    -anchor w
.gen.ctr.trans.rdb1 select
pack .gen.ctr.trans.lab -side top -expand y -fill x -anchor w
pack .gen.ctr.trans.rdb1 .gen.ctr.trans.rdb2 -expand 1 -fill y  -anchor w
set bh [_ "Should words differing only in that one\npair of segments occur in the  opposite\norder be included as minimal pairs?"];
balloonhelp_for .gen.ctr.trans      $bh;
balloonhelp_for .gen.ctr.trans.rdb1 $bh;
balloonhelp_for .gen.ctr.trans.rdb2 $bh;
balloonhelp_for .gen.ctr.trans.lab  $bh;
bind .gen.ctr.trans       <<B3>> ExplainTranspositions
bind .gen.ctr.trans.lab   <<B3>> ExplainTranspositions
bind .gen.ctr.trans.rdb1  <<B3>> ExplainTranspositions
bind .gen.ctr.trans.rdb2  <<B3>> ExplainTranspositions
if {!$MinpairAvailableP} {
    .gen.ctr.trans.rdb1 configure -state disabled;
    .gen.ctr.trans.rdb2 configure -state disabled;
}


#Single Column Input?
set ypad 1
frame       .gen.ctr.sci -relief $ctrrelief -border 1
label       .gen.ctr.sci.lab  -text [_ "Single Column Input?"]   -anchor w
radiobutton .gen.ctr.sci.rdb1 -variable DoSingleP -value 0 -text [_ "no"]     -anchor w
radiobutton .gen.ctr.sci.rdb2 -variable DoSingleP -value 1 -text [_ "yes"]    -anchor w
.gen.ctr.sci.rdb2 select
pack .gen.ctr.sci.lab  -side top -expand 1 -fill both -anchor w -pady $ypad
pack .gen.ctr.sci.rdb1 -side top -expand 1 -fill both -anchor w -pady $ypad
pack .gen.ctr.sci.rdb2 -side top -expand 1 -fill both -anchor w -pady $ypad
bind .gen.ctr.sci       <<B3>> ExplainSingleColumnInput
bind .gen.ctr.sci.lab   <<B3>> ExplainSingleColumnInput
bind .gen.ctr.sci.rdb1  <<B3>> ExplainSingleColumnInput
bind .gen.ctr.sci.rdb2  <<B3>> ExplainSingleColumnInput
set bh [_ "Does the input consist of words only\nor words together with identifiers?"];
balloonhelp_for .gen.ctr.sci      $bh;
balloonhelp_for .gen.ctr.sci.rdb1 $bh;
balloonhelp_for .gen.ctr.sci.rdb2 $bh;
balloonhelp_for .gen.ctr.sci.lab  $bh;

#Minimal Pairs or pairs differing in two places?
frame       .gen.ctr.onetwo -relief $ctrrelief -border 1
label       .gen.ctr.onetwo.lab  -text [_ "How Many Differences?"]   -anchor w
radiobutton .gen.ctr.onetwo.rdb1 -variable HowManyDiffs -value 1 -text [_ "1"]    -anchor w
radiobutton .gen.ctr.onetwo.rdb2 -variable HowManyDiffs -value 2 -text [_ "2"]    -anchor w
pack .gen.ctr.onetwo.lab -side top -expand y -fill x -anchor n
pack .gen.ctr.onetwo.rdb1 -side top    -expand 1 -fill y  -anchor w -pady {6 1} -padx 5
pack .gen.ctr.onetwo.rdb2 -side bottom -expand 1 -fill y  -anchor w -pady {1 6} -padx 5
set bh [_ "In how many places should the pairs differ?"];
balloonhelp_for .gen.ctr.onetwo      $bh;
balloonhelp_for .gen.ctr.onetwo.rdb1 $bh;
balloonhelp_for .gen.ctr.onetwo.rdb2 $bh;
balloonhelp_for .gen.ctr.onetwo.lab  $bh;
bind .gen.ctr.onetwo       <<B3>> ExplainOneTwo
bind .gen.ctr.onetwo.lab   <<B3>> ExplainOneTwo
bind .gen.ctr.onetwo.rdb1  <<B3>> ExplainOneTwo
bind .gen.ctr.onetwo.rdb2  <<B3>> ExplainOneTwo
trace variable HowManyDiffs w ToggleMaximumDistanceActive;
ToggleMaximumDistanceActive a b c


#Use Minpair?
frame       .gen.ctr.use -relief $ctrrelief -border 1
label       .gen.ctr.use.lab  -text [_ "Use Minpair?"]   -anchor w
radiobutton .gen.ctr.use.rdb1 -variable UseMinpairP -value 0 -text [_ "no"]     -anchor w
radiobutton .gen.ctr.use.rdb2 -variable UseMinpairP -value 1 -text [_ "yes"]    -anchor w
pack .gen.ctr.use.lab -side top -expand y -fill x -anchor n
pack .gen.ctr.use.rdb1 -side top    -expand 1 -fill y  -anchor w -pady {6 1} -padx 5
pack .gen.ctr.use.rdb2 -side bottom -expand 1 -fill y  -anchor w -pady {1 6} -padx 5
set bh [_ "Execute the minpair program or do the work internally?"];
balloonhelp_for .gen.ctr.use      $bh;
balloonhelp_for .gen.ctr.use.rdb1 $bh;
balloonhelp_for .gen.ctr.use.rdb2 $bh;
balloonhelp_for .gen.ctr.use.lab  $bh;
#bind .gen.ctr.use       <<B3>> ExplainSorting
#bind .gen.ctr.use.lab   <<B3>> ExplainSorting
#bind .gen.ctr.use.rdb1  <<B3>> ExplainSorting
#bind .gen.ctr.use.rdb2  <<B3>> ExplainSorting

#Assemble pieces
set gxpd 4
set gypd 6

grid .gen.ctr.trans    -row 0 -column 0 -sticky wens -padx 5     -pady $gypd
grid .gen.ctr.idd      -row 0 -column 1 -sticky wens -padx $gxpd -pady $gypd
grid .gen.ctr.sci      -row 1 -column 0 -sticky wens -padx 5     -pady $gypd
grid .gen.ctr.mwl      -row 1 -column 1 -sticky wens -padx $gxpd -pady $gypd
grid .gen.ctr.mds      -row 1 -column 2 -sticky wens -padx $gxpd -pady $gypd
grid .gen.ctr.srt      -row 2 -column 0 -sticky wens -padx 5     -pady $gypd
grid .gen.ctr.dsm      -row 2 -column 1 -sticky wens -padx $gxpd -pady $gypd
grid .gen.ctr.onetwo   -row 0 -column 2 -sticky wens -padx $gxpd -pady $gypd
grid .gen.ctr.use      -row 2 -column 2 -sticky wens -padx $gxpd -pady $gypd

#Name of input file
frame  .gen.if -relief ridge -border 1
label  .gen.if.title  -text [_ "Specify Input File"]   -anchor w
set btxt [_ "Browse"]
set blen [string length $btxt];
button .gen.if.brb  -text $btxt -width $blen -anchor w -command SelectInputFile
entry  .gen.if.ent  -foreground $::ColorSpecs(UserTextEntry,Foreground) -font UserPlainTextFont
bind .gen.if.ent <FocusIn> {SetInsertionTargets .gen.if.ent}
.gen.if.ent insert 0 $::DefaultInputFile;

set ifypd 3
pack .gen.if.title -side top -expand y -fill x -anchor w -pady $ifypd
pack .gen.if.brb  -expand 0 -anchor w -side left -padx 4 -pady $ifypd
pack .gen.if.ent  -expand 1 -fill x  -anchor w -side left -pady $ifypd -padx 3
set bhmsg [_ "Specify the name of the input file."];
balloonhelp_for .gen.if $bhmsg;
balloonhelp_for .gen.if.title $bhmsg;
balloonhelp_for .gen.if.ent [_ "Input will be read from the file whose name is shown in\nthis entry box. You may enter its name directly or\npress the Browse button and use the file selection dialogue."];
balloonhelp_for .gen.if.brb [_ "Press this button to select the name of the input file."]

#Name of output file
frame  .gen.of -relief ridge -border 1
label  .gen.of.title  -text [_ "Specify Output File"]   -anchor w
button .gen.of.brb  -text $btxt -width $blen -anchor w -command SelectOutputFile
entry  .gen.of.ent -foreground $::ColorSpecs(UserTextEntry,Foreground) \
    -font UserPlainTextFont 
bind .gen.of.ent <FocusIn> {SetInsertionTargets .gen.of.ent}
.gen.of.ent insert 0 $::DefaultOutputFile
set ofypd 3
pack .gen.of.title -side top -expand y -fill x -anchor w  -pady $ofypd
pack .gen.of.brb  -expand 0 -anchor w -side left -padx 4  -pady $ofypd
pack .gen.of.ent  -expand 1 -fill x  -anchor w -side left -pady $ofypd -padx 3
set bhmsg [_ "Specify the name of the output file."];
balloonhelp_for .gen.of $bhmsg;
balloonhelp_for .gen.of.title $bhmsg;
balloonhelp_for .gen.of.ent [_ "Output will be written to the file whose name is shown in\nthis entry box. You may enter its name directly or\npress the Browse button and use the file selection dialogue."];
balloonhelp_for .gen.of.brb [_ "Press this button to select the name of the output file."]

#Multigraphs
frame       .gen.mlg -relief ridge -border 2
label       .gen.mlg.lab  -text [_ "Define Multigraphs?"] -anchor w
radiobutton .gen.mlg.rdb1 -variable DefineMultigraphsP -value 0 -text [_ "no"] -anchor w
frame       .gen.mlg.bot 
radiobutton .gen.mlg.bot.rdb2 -variable DefineMultigraphsP -value 1 -text [_ "yes"] -anchor w
.gen.mlg.rdb1 select
button      .gen.mlg.bot.b1  -text [_ "Specify File"] -state disabled \
    -command SpecifyMultigraphFile
button      .gen.mlg.bot.b2  -text [_ "Interactive"]  -state disabled \
    -command EnterMultigraphDefinitions
entry  .gen.mlg.bot.ent -foreground $::ColorSpecs(UserTextEntry,Foreground) \
    -font UserTechnicalTextFont;
.gen.mlg.bot.ent insert 0 $DefaultMultigraphFile
.gen.mlg.bot.ent configure -state disabled
bind .gen.mlg.bot.ent <FocusIn> {SetInsertionTargets .gen.mlg.bot.ent}
pack .gen.mlg.bot.rdb2 -side left -expand 0 -fill both -anchor w -padx 1
pack .gen.mlg.bot.b2   -side left -expand 0 -fill y    -anchor w -padx 1
pack .gen.mlg.bot.b1   -side left -expand 0 -fill y    -anchor w -padx 1
pack .gen.mlg.bot.ent  -side left -expand 1 -fill x    -anchor w -padx 1

set mlgypd 3
pack .gen.mlg.lab -side top -anchor w -pady $mlgypd
pack .gen.mlg.bot -side bottom -expand 1 -fill x  -anchor w -pady 1
pack .gen.mlg.rdb1 -side top -expand 1 -fill y  -anchor w -pady $mlgypd
trace variable DefineMultigraphsP w ToggleMultigraphEntryActive;

bind .gen.mlg.lab <<B3>> ExplainMultigraphs;
set bh [_ "A multigraph is a sequence of two or more characters\ntreated as a single segment. This section\nallows you to define multigraphs if you wish to."]
balloonhelp_for .gen.mlg $bh;
balloonhelp_for .gen.mlg.lab $bh;
balloonhelp_for .gen.mlg.rdb1 [_ "Select this if you do not need to define any multigraphs."]
balloonhelp_for .gen.mlg.bot.rdb2 [_ "Select this if you need to define multigraphs."]
balloonhelp_for .gen.mlg.bot.b1 [_ "Multigraph definitions will be read from the file shown\nin the entry box to the right. You can type the name of the\nfile in the entry box if you like, or you can press this button\nto bring up a file selection dialogue."]
balloonhelp_for .gen.mlg.bot.b2 [_ "Press this button if you want to define multigraphs\ninteractively. The result will be written out to a file,\nwhose name will appear in the entry box to the right."]
balloonhelp_for .gen.mlg.bot.ent [_ "Multigraph definitions will be read from the file\nwhose name  is shown in this entry box. You may\nselect an existing file by entering its name directly\nor by using the file selection dialogue. You may\nalso define multigraphs interactively, after which\nnthey it will be written to a file, whose name\nwill appear in the entry box."]

set mxpd 8
set mypd 4
frame .gen.sep3  -height 4 -borderwidth 1 -relief raised
pack  .gen.if -side top -expand 1 -fill both -padx $mxpd -pady $mypd -anchor w

frame .gen.sep4  -height 4 -borderwidth 1 -relief raised
pack  .gen.of -side top -expand 1 -fill both -padx $mxpd -pady $mypd -anchor w

frame .gen.sep2 -height 4 -borderwidth 1 -relief raised
pack  .gen.mlg -side top -expand 1 -fill both -anchor w -padx $mxpd -pady $mypd

pack  .gen.ctr  -side top -expand 1 -fill both -padx $mxpd -pady $mypd -anchor w
frame .gen.sep7 -height 4 -borderwidth 0 -relief flat
pack  .gen.sep7 -side bottom -expand 1 -fill both -padx $mxpd -pady 3

if {$::AquaP} {
    pack .cmnds -side top -expand 1 -fill x
}
pack .mf  -side top -expand 1 -fill both
pack .gen -side top -expand 1 -fill both

trace add variable UseMinpairP write ActivatePerUseMinpair

#We put this stuff here in anticipation of the point at which
#it is possible to set the default browser from the init file.
#We want the default browser to be set before this code
#is executed.
#If the default browser is on the list, remove it.
set di [lsearch -exact $BrowserList $DefaultBrowser]
if {$di >= 0} {
    set BrowserList [lreplace $BrowserList $di $di]
}
#Add the default browser to the beginning of the list.
set BrowserList [linsert $BrowserList 0 $DefaultBrowser];

foreach fn $CustomCharacterChartFileNameList {
    wm withdraw [LoadCustomCharacterChart $fn];
}
foreach dl $CustomCharacterChartDataList {
    wm withdraw [DefineCustomCharacterChart $dl];
}

if {!$MinpairAvailableP} {
    if {$UseMinpairP} {
	ShowMessage [_ "Minpair is not available on this machine or is not in your path."]
    }
    wm title . [format "mpg %s" $Version];
} else {
   wm title . [format "mpg %s    minpair %s" $Version $::MinpairVersion];
}

set UseMinpairP $UseMinpairP
BindKeys .
SetCommandGlosses;
tk_focusFollowsMouse;
after idle {
    update idletasks
    ConfigureCtr;
}
